/*
	Example of how to utilize the External Call-In Interface

	Copyright (c) 1992 - 2002 by InterSystems.
	Cambridge, Massachusetts, U.S.A.  All rights reserved.
	Confidential, unpublished property of InterSystems.

======================================================================

Modification History: (most recent first)

LRS588	3/15/02 correct a couple of typos in dialog
CDS010 11/24/98 make all message delimiters cr/lf instead of lf/cr
MEB043	8/06/97	rename file and use new call names and symbols
SJ1110	8/24/96	checked for Unicode project phase 1 issues
JAO620  1/17/93 changes for AlphaVMS
PJH130	3/12/93	made it support the new error handling
PJH116  1/04/93 Rewrote for common code.
JAO491  4/23/92 Created this module.


***********************************************************************/

#include <stdio.h>
#include <string.h>
#include "ccallin.h"
#include <errno.h>
#ifdef	VMS		/* defined by the DEC C compiler */
#include <stdlib.h>
#include <unixio.h>
#endif

int readline(char *);
int displayerr(int);
int callin();
int dispres();

/*
	This is a sample of how to utilize the call-in interface.  It asks the
	user for the various parameters to CacheStart() and then it invokes
	CacheStart() to create a connection. 

	It then transfers control to the callin() subroutine which prints
	a ':' for a prompt and reads a string from the terminal.  If the
	string starts with an '=' sign the string is passed to CacheEval for
	processing. Otherwise it is passed to CacheExecute to be executed.
	If an expression is evaluated callin() calls CacheConvert to
	return the result of the expression as a string and it prints
	the result on the terminal.

	This process is terminated when the user enters a line that begins
	with a ~ or if he Halts out of M.

	At this point callin() returns and the main routine calls CacheEnd().
	Then the user is given the opportunity to start the process over
	again or to exit.

	callin() can also be called from a $ZF function.

*/

main()
{

int	rc;
char	cmdline[300], ch;
CACHE_STR pout,pin;
int	i;
int	termflag, timeout, exit_hand();
struct	sgttyb *arg;

top:
printf("\r\n");
printf("CACHE_TTALL   =%04x Init tty across each call\r\n",CACHE_TTALL);
printf("CACHE_TTCALLIN=%04x Init tty on call in but don't restore on return\r\n",
       CACHE_TTCALLIN); 
printf("CACHE_TTSTART =%04x Init tty on connection and reset on termination\r\n",
       CACHE_TTSTART);
printf("CACHE_TTNEVER =%04x Do not initialize tty\r\n",CACHE_TTNEVER);
printf("CACHE_TTNONE  =%04x Do not do any IO to PRINCIPAL IO\r\n",CACHE_TTNONE);
printf("CACHE_PROGMODE=%04x Start in the programmer mode\r\n",CACHE_PROGMODE);
	printf("Enter term flags (in hex): ");
	fflush(stdout);

	readline(cmdline);
	sscanf(cmdline,"%x",&termflag);

	printf("\r\nEnter timeout: ");
	fflush(stdout);
	readline(cmdline);
	sscanf(cmdline,"%d",&timeout);

	printf("\r\nEnter principal output device: ");
	fflush(stdout);
	readline((char *)pout.str);		/* JAO620: added cast */
	pout.len = strlen((char *)pout.str);	/* JAO620: added cast */

	printf("\r\nEnter principal input device: ");
	fflush(stdout);
	readline((char *)pin.str);		/* JAO620: added cast */
	pin.len = strlen((char *)pin.str);	/* JAO620: added cast */
	printf("\r\n");
	fflush(stdout);

	rc = CacheStart(termflag,timeout,&pin,&pout);
	if (rc != CACHE_SUCCESS) exit(1);
	rc = callin();
	CacheEnd();
	printf("Restart? ");
	fflush(stdout);
	readline(cmdline);
	ch = cmdline[0];
	if (ch >= 'a' && ch <= 'z') ch -= 'a' - 'A';
	if (ch == 'Y') goto top;
	exit(0);
}


int	callin()
{
	CACHE_STR mstr;
	char	buf[512];
	char	*s, *t;
	struct	{
	short	len;
	char	str[10];
	} prompt;
	int	rc;

	CacheOflush();		/* flush pending output */
	printf("\r\n:");
	fflush(stdout);
	while(readline(buf)) {
	   if (buf[0] == '~')
	      return 0;
	   if (buf[0] != '=') {
	      strcpy((char *)mstr.str,buf);	   /* JAO620: added cast */
	      mstr.len = strlen((char *)mstr.str); /* JAO620: added cast */
	      rc = CacheExecute(&mstr);
	      if (rc > 0) displayerr(rc);
	      CacheOflush();
	   } else {
	      strcpy((char *)mstr.str,buf+1);	   /* JAO620: added cast */
	      mstr.len = strlen((char *)mstr.str); /* JAO620: added cast */
	      rc = CacheEval(&mstr);
	      CacheOflush();
	      if (rc == CACHE_SUCCESS) dispres();
	   }
	   if (rc == CACHE_CONBROKEN) {
	      printf("CACHE_CONBROKEN\r\n");
	      fflush(stdout);
	      return 1;
	   }
	   if (rc == CACHE_NOCON) {
	      printf("CACHE_NOCON\r\n");
	      fflush(stdout);
	      return 1;
	   }

	   prompt.len = 10;
	   CachePrompt((CACHE_STRP)&prompt);
	   printf("\r\n%.*s:",prompt.len,prompt.str);
	   fflush(stdout);
	}
	printf("read failed!\r\n");
	fflush(stdout);
	return 0;
}
dispres()
{
	CACHE_STR	retstr;
	int	rc;

	retstr.len = sizeof(retstr.str);
	rc = CacheConvert(CACHE_STRING,(unsigned char *) &retstr);
	printf("\t result = %d/%.*s/",rc,retstr.len,retstr.str);
	fflush(stdout);
}
displayerr(code)
int	code;
{
	CACHE_STR errmsg;
	CACHE_STR srcline;
	int	offset;
	int	rc;
	char	*s;
	int	i;

	errmsg.len = CACHE_MAXSTRLEN;
	if ((rc = CacheErrxlate(code,&errmsg)) != CACHE_ERUNKNOWN) 
	   printf("\r\nerror code: %3d\t%.*s",code,errmsg.len,errmsg.str); 
	else
	   printf("\r\nerror code: %3d\tUNKNOWN ERROR CODE",code);
	errmsg.len = srcline.len = CACHE_MAXSTRLEN;
	if ((rc = CacheError(&errmsg, &srcline, &offset)) != CACHE_SUCCESS) {
	   printf("\r\nfailed to diplay error - rc = %d",rc);
	   fflush(stdout);
	   return 0;
	}
	if (srcline.len) {
	   printf("\r\n");
						/* JAO620: added cast */
	   for (i=0,s=(char *)srcline.str;i<srcline.len;i++,s++) {
	      putchar(*s);
	      if ((i%79) == 0 && i < (srcline.len - 1)) printf("\r\n");
	   }
	   printf("\r\n");
						/* JAO620: added cast */
	   for (i=0,s=(char *)srcline.str;i<srcline.len;i++,s++) {
	      if (i == offset) {
		 putchar('^');
		 break;
	      }
	      putchar(' ');
	      if ((i%79) == 0 && i < (srcline.len - 1)) printf("\r\n");
	   }
	}
	if (errmsg.len) printf("\r\n%.*s",errmsg.len,errmsg.str);
	printf("\r\n");
	fflush(stdout);
	return 0;
}
readline(buf)
char	*buf;
{
	char	c, *s;
	int	i;

	s = buf;
	for (;;) {
	   i=read(0,&c,1);
#ifdef	EWOULDBLOCK
	   if (i == -1 && errno == EWOULDBLOCK)
	      continue;
#endif	
#ifdef	EAGAIN
	   if (i == -1 && errno == EAGAIN)
	      continue;
#endif	
	   if (i == -1 && errno == EINTR)
	      continue;
	   if (i == -1)
	      break;
	   if (i) {
	      if (c == '\n' || c == '\r') {
		 *s = '\0';
		 write(1,"\r\n",2);

		 return 1;
	      }

	      if ((c == 8) || c == 127) {
		 if (s > buf) {
		    --s;
		    write(1,"\b \b",3);
		 }
		 continue;
	      }

	      *s++ = c;
	      write(1,&c,1);
	   }
	}
	return 0;
}

