/*
	Definitions for the Call-In Interface to Cache

	Copyright (c) 1997-2003 by InterSystems.
	Cambridge, Massachusetts, U.S.A.  All rights reserved.
	Confidential, unpublished property of InterSystems.

	This module contains the definitions necessary to build a 'C'
	language program that utilizes the InterSystems Cache call-in
	interface.
*/

/*			 Data Structures
 */

/*	The following typedefs produce pointers to counted strings of the type
	the Cache interface routines expect as input and return as output.
	The length element for a string being passed to an interface routines
	is set to the actual length of the string.  When a string is to be
	passed back to the caller, the caller provides a counted string buffer
	with the length element set to the length of the buffer (i.e. the
	maximum possible length of the returned string).  Failure to set the
	length element can result in either errors from the interface if the
	length field is out of range, or overwriting other memory structures
	if a string larger than the allocated space is returned.

*/
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__VMS							/* LRS666+ */
#pragma	__required_pointer_size	__save
#pragma	__required_pointer_size	__short
#endif								/* LRS666- */

#define CACHE_MAXSTRLEN 32767
/*	Structure to pass ASCII (one byte) character strings */
typedef struct {
	unsigned short len;
	unsigned char  str[CACHE_MAXSTRLEN];
	} CACHE_ASTR, *CACHE_ASTRP;

/*	Structure to pass Unicode (two byte) character strings */
typedef struct {
	unsigned short len;
	unsigned short str[CACHE_MAXSTRLEN];
	} CACHEWSTR, *CACHEWSTRP;

/*
	Name mappings to internal entrypoints
*/

#define	CacheAbort	msqlabort
#define	CacheContext	msqlcontext
#define	CacheConvert	msqlconvert
#define	CacheCtrl	msqlctrl
#define	CacheCvtInA	msqlcvtin
#define	CacheCvtInW	msqlcvtinW
#define	CacheCvtOutA	msqlcvtout
#define	CacheCvtOutW	msqlcvtoutW
#define	CacheEnd	msqlend
#define	CacheErrorA	msqlerror
#define	CacheErrorW	msqlerrorW
#define	CacheErrxlateA	msqlerrxlate
#define	CacheErrxlateW	msqlerrxlateW
#define	CacheEvalA	msqleval
#define	CacheEvalW	msqlevalW
#define	CacheExecuteA	msqlexecute
#define	CacheExecuteW	msqlexecuteW
#define	CacheOflush	msqloflush
#define	CachePromptA	msqlprompt
#define	CachePromptW	msqlpromptW
#ifndef	__VMS
#define	CacheSignal	msqlsignal
#endif
#define	CacheStartA	msqlstart
#define	CacheStartW	msqlstartW
#define	CacheType	msqltype
/* 			Character type interface definitions
*/

#ifdef	CACHE_UNICODE	/* Unicode character strings */
#define	CACHESTR	CACHEWSTR
#define	CACHE_STR	CACHEWSTR
#define	CACHESTRP	CACHEWSTRP
#define CACHE_STRP	CACHEWSTRP
#define	CACHE_STRING	CACHE_WSTRING
#else			/* ASCII character strings */
#define	CACHESTR	CACHE_ASTR
#define	CACHE_STR	CACHE_ASTR
#define	CACHESTRP	CACHE_ASTRP
#define CACHE_STRP	CACHE_ASTRP
#define	CACHE_STRING	CACHE_ASTRING
#endif

/*				Function Prototypes
*/

#if defined(__STDC__) || defined(__cplusplus)
extern int  msqlstart(unsigned long,int,CACHESTRP,CACHESTRP);
extern int  msqlstartW(unsigned long,int,CACHEWSTRP,CACHEWSTRP);
extern int  msqlexecute(CACHESTRP volatile);
extern int  msqlexecuteW(CACHEWSTRP volatile);
extern int  msqleval(CACHESTRP volatile);
extern int  msqlevalW(CACHEWSTRP volatile);
extern int  msqlcontext(void);
extern int  msqlcvtin(CACHESTRP,CACHESTRP,CACHESTRP);
extern int  msqlcvtinW(CACHESTRP,CACHEWSTRP,CACHEWSTRP);
extern int  msqlcvtout(CACHESTRP,CACHESTRP,CACHESTRP);
extern int  msqlcvtoutW(CACHEWSTRP,CACHEWSTRP,CACHESTRP);
extern int  msqlprompt(CACHESTRP);
extern int  msqlpromptW(CACHEWSTRP);
extern int  msqlerrxlate(int,CACHESTRP);
extern int  msqlerrxlateW(int,CACHEWSTRP);
extern int  msqlctrl(unsigned long);
extern int  msqlconvert(unsigned long,unsigned char *);
extern int  msqltype(void);
extern int  msqlabort(unsigned long);
extern int  msqlend(void);
extern void msqloflush(void);
#ifndef	__VMS
extern int  msqlsignal(int);
#endif
extern int  msqlerror(CACHESTRP,CACHESTRP,int *);
extern int  msqlerrorW(CACHEWSTRP,CACHEWSTRP,int *);
#if	0	/* For Relational Gateway, no longer used */	/* LRS666+ */
extern int  msqlgetstr(char *,int,char *);
extern int  msqlsetstr(char *,int,char *);
extern char *msqlrefparse(int,unsigned char *);
extern int  msqlreffree(char *);
#endif		/* For Relational Gateway */			/* LRS666- */
extern int  CachePushInt(int);
extern int  CachePushDbl(double);
extern int  CachePushOref(unsigned int);			/* CDS188 */
extern int  CachePushStr(int,unsigned char *);
extern int  CachePushStrW(int,unsigned short *);
extern int  CachePushRtn(unsigned int *,int,unsigned char *,int,unsigned char *);
extern int  CachePushRtnX(unsigned int *,int,unsigned char *,int,int,unsigned char*,int,unsigned char *);
extern int  CachePushRtnW(unsigned int *,int,unsigned short *,int,unsigned short *);
extern int  CachePushRtnXW(unsigned int *,int,unsigned short *,int,int,unsigned short*,int,unsigned short *);
extern int  CachePushFunc(unsigned int *,int,unsigned char *,int,unsigned char *);
extern int  CachePushFuncX(unsigned int *,int,unsigned char *,int,int,unsigned char*,int,unsigned char *);
extern int  CachePushFuncW(unsigned int *,int,unsigned short *,int,unsigned short *);
extern int  CachePushFuncXW(unsigned int *,int,unsigned short *,int,int,unsigned short*,int,unsigned short *);
extern int  CachePushGlobal(int,unsigned char *);
extern int  CachePushGlobalX(int,unsigned char *,int,unsigned char *);
extern int  CachePushGlobalW(int,unsigned short *);
extern int  CachePushGlobalXW(int,unsigned short *,int,unsigned short *);
extern int  CacheUnPop();
extern int  CachePop(void **);
extern int  CachePopInt(int *);
extern int  CachePopDbl(double *);
extern int  CachePopOref(unsigned int *);			/* CDS188 */
extern int  CachePopStr(int *,unsigned char **);
extern int  CachePopStrW(int *,unsigned short **);
extern int  CacheSetVar(int,unsigned char *);
extern int  CacheGetVar(int,unsigned char *);
extern int  CacheDoRtn(unsigned int,int);
extern int  CacheDoFun(unsigned int,int);			/* LRS512 */
extern int  CacheExtFun(unsigned int,int);
extern int  CacheGlobalGet(int,int);
extern int  CacheGlobalSet(int);
extern int  CacheGlobalIncrement(int);				/* JO1759 */
								/* RAM968+ */
extern int  CachePushClassMethod(int,unsigned char *,int,unsigned char *,int);
extern int  CachePushClassMethodW(int,unsigned short *,int,unsigned short *,int);
extern int  CachePushMethod(unsigned int,int,unsigned char *,int);
extern int  CachePushMethodW(unsigned int,int,unsigned short *,int);
extern int  CacheInvokeClassMethod(int);
extern int  CacheInvokeMethod(int);
extern int  CachePushProperty(unsigned int,int,unsigned char *);
extern int  CachePushPropertyW(unsigned int,int,unsigned short *);
extern int  CacheGetProperty();
extern int  CacheSetProperty();
								/* RAM968- */
extern int  CacheIncrementCountOref(unsigned int);		/* CDS188 */
extern int  CacheCloseOref(unsigned int);			/* CDS188 */
extern int  CachePushCvtW(int,unsigned short *);		/* LRS582+ */
extern int  CachePopCvtW(int *,unsigned short **);
extern int  CachePushList(int,unsigned char *);
extern int  CachePopList(int *,unsigned char **);		/* LRS582- */
extern int  CachePushPtr(void *);				/* LRS609+ */
extern int  CachePopPtr(void **);				/* LRS609- */
#else
extern int  CacheAbort();
extern int  CacheContext();
extern int  CacheConvert();
extern int  CacheCtrl();
extern int  CacheCvtInA();
extern int  CacheCvtInW();
extern int  CacheCvtOutA();
extern int  CacheCvtOutW();
extern int  CacheEnd();
extern int  CacheErrorA();
extern int  CacheErrorW();
extern int  CacheErrxlateA();
extern int  CacheErrxlateW();
extern int  CacheEvalA();
extern int  CacheEvalW();
extern int  CacheExecuteA();
extern int  CacheExecuteW();
extern void CacheOflush();
extern int  CachePromptA();
extern int  CachePromptW();
#ifndef	__VMS
extern int  CacheSignal();
#endif
extern int  CacheStartA();
extern int  CacheStartW();
extern int  CacheType();
extern int  CachePushInt();
extern int  CachePushDbl();
extern int  CachePushOref();					/* CDS188 */
extern int  CachePushStr();
extern int  CachePushStrW();
extern int  CachePushRtn();
extern int  CachePushRtnX();
extern int  CachePushRtnW();
extern int  CachePushRtnXW();
extern int  CachePushFunc();
extern int  CachePushFuncX();
extern int  CachePushFuncW();
extern int  CachePushFuncXW();
extern int  CachePushGlobal();
extern int  CachePushGlobalX();
extern int  CachePushGlobalW();
extern int  CachePushGlobalXW();
extern int  CacheUnPop();
extern int  CachePop();
extern int  CachePopInt();
extern int  CachePopDbl();
extern int  CachePopOref();					/* CDS188 */
extern int  CachePopStr();
extern int  CachePopStrW();
extern int  CacheSetVar();
extern int  CacheGetVar();
extern int  CacheDoRtn();
extern int  CacheDoFun();					/* LRS512 */
extern int  CacheExtFun();
extern int  CacheGlobalGet();
extern int  CacheGlobalSet();
extern int  CacheGlobalIncrement();				/* JO1759 */
extern int  CachePushClassMethod();				/* RAM968+ */
extern int  CachePushClassMethodW();
extern int  CachePushMethod();
extern int  CachePushMethodW();
extern int  CacheInvokeClassMethod();
extern int  CacheInvokeMethod();
extern int  CachePushProperty();
extern int  CachePushPropertyW();
extern int  CacheGetProperty();
extern int  CacheSetProperty();					/* RAM968- */
extern int  CacheIncrementCountOref();				/* CDS188 */
extern int  CacheCloseOref();					/* CDS188 */
extern int  CachePushCvtW();					/* LRS582+ */
extern int  CachePopCvtW();
extern int  CachePushList();
extern int  CachePopList();					/* LRS582- */
extern int  CachePushPtr();					/* LRS609+ */
extern int  CachePopPtr();					/* LRS609- */
#endif

/* 				Constants
*/

/*	The following are the definitions that are used to form the flags
	argument to CacheStart() to control how Cache treats, or doesn't
	treat, the principal input device.

	Setting more than one of these flags (with the exception of the
	modifiers) will cause unpredictable results
*/
#define	CACHE_TTALL	1	/* Indicates that Cache should initialize the
				   terminal's settings and restore them across
				   each call into, and return from, the
				   interface.  */
#define CACHE_TTCALLIN	2	/* Indicates that Cache should initialize the
				   terminal each time its called but should
				   restore it only when the connection is
				   closed (egs. when CacheEnd() is called or
				   the connection is broken). */
#define CACHE_TTSTART	4	/* Indicates that Cache should initialize the
				   terminal when the connection is formed and
				   reset it when the connection is terminated.
				   */
#define	CACHE_TTNEVER	8	/* Indicates that Cache should not alter the
				   terminal's settings */
#define CACHE_TTNONE	16	/* Indicates that Cache should not do any
				   output or input from the principle input
				   device. This is equivilent to specifying
				   the null device for princin and princout.
				   Output will be discarded and reads will
				   complete with end-of-file */
#define CACHE_TTMASK	(CACHE_TTALL|CACHE_TTCALLIN|CACHE_TTSTART| \
			 CACHE_TTNEVER|CACHE_TTNONE)
#define CACHE_PROGMODE	32	/* Cache errors do not cause halt in the
				   absense of error handler */
/*	The following modifier is allowed in conjunction with
		CACHE_TTALL, CACHE_TTCALLIN and CACHE_TTSTART.
*/
#define CACHE_TTNOUSE	0x10000 /* If set this indicates that Cache OPEN and
				   USE commands are not allowed to alter the
				   terminal as well.  This is implicitly
				   set by CACHE_TTNEVER and CACHE_TTNONE.
				   The default is to allow Cache OPEN and
				   USE commands to alter the terminal in the
				   other modes. */

/*	The following are the definitions that can be used to form the
	arguments to CacheCtrl().  These flags are in pairs and if both
	members of a set are specified the results will be unpredictable.
*/
#define CACHE_DISACTRLC	 1	/* CTRL/C is ignored by Cache */
#define CACHE_ENABCTRLC	 2	/* CTRL/C in Cache generates an <INTERRUPT> */

/*	The following are the arguments to CacheAbort() that control the
	mechanism used to terminate Cache processing.

	CACHE_CTRLC simulates a CTRL/C at the keyboard and takes effect even
	if CacheCtrl() specified that CTRL/C is to be ignored. The Cache
	call that is interrupted returns CACHE_INTERRUPT.

	CACHE_RESJOB simulates a resjob. This results in the Cache connection
	being broken and the Cache call that's interrupted completes with
	a status of CACHE_CONBROKEN.
*/
#define	CACHE_CTRLC	 1
#define CACHE_RESJOB	 2

/* 	The following are the return values from CacheType() that describe
	the internal datatype of the result from the call to CacheEval()
*/
#define	CACHE_INT	 1	/* native (32-bit on the VAX) integer */
#define CACHE_DOUBLE	 2	/* 64-bit floating point */
#define CACHE_ASTRING	 3	/* ASCII character string */

/*	In addition to the above the following can be used as the type
	argument to the CacheConvert() function

	Note that floating point types (CACHE_FLOAT, CACHE_HFLOAT and
	CACHE_DOUBLE) that have decimal parts (including negative exponents),
	as well as the 64 bit integer types CACHE_INT8 and CACHE_UINT8,
	involve division when calculating the return value. Therefore the
	results may not be 100% accurate.  CACHE_STRINT, CACHE_INT8 and
	CACHE_UINT8 can return the status CACHE_RETTRUNC if the result of the
	last CacheEval() call will not fit in the space allocated in
	CacheConvert(). In the case of CACHE_INT8 and CACHE_UINT8 this means
	that the expression resulted in a floating point # that could not be
	normalized to fit within 64 bits.
*/
#define	CACHE_CHAR	 4	/*   8 bit signed integer */
#define	CACHE_INT2	 5	/*  16 bit signed integer */
#define	CACHE_INT4	 6	/*  32 bit signed integer */
#define	CACHE_INT8	 7	/*  64 bit signed integer */
#define	CACHE_UCHAR	 8	/*   8 bit unsigned integer */
#define	CACHE_UINT2	 9	/*  16 bit unsigned integer */
#define	CACHE_UINT4	10	/*  32 bit unsigned integer */
#define	CACHE_UINT8	11	/*  64 bit unsigned integer */
#define CACHE_FLOAT	12	/*  32 bit floating point */
#define CACHE_HFLOAT	13	/* 128 bit floating point */
#define	CACHE_UINT	14	/* native size unsigned integer
				   (egs. 32 bit on the VAX) */
#define CACHE_WSTRING	15	/* Unicode character string */
#define	CACHE_OREF	16	/* object reference */


/*	The following are error codes that are generated by the call-in
	interface as opposed to error codes that are generated by
	Cache and returned by the call-in interface. */

#define CACHE_NORES	  -13	/* CacheConvert() was called to get a result
				   but there's no result to return */
#define CACHE_BADARG	  -12	/* A function was called with an argument
				   that was out of range or otherwise
				   invalid */
#define CACHE_NOTINCACHE  -11	/* Returned from CacheAbort() if the process
			  	   wasn't executing on the Cache side */
#define CACHE_RETTRUNC 	  -10	/* The return buffer was too small so only
				   part of the requested item was returned */
#define CACHE_ERUNKNOWN	   -9	/* Unknown error code in persave */
#define CACHE_RETTOOSMALL  -8	/* return buffer too small, nothing returned */
#define CACHE_NOCON 	   -7	/* not connected to Cache */
#define CACHE_INTERRUPT    -6	/* Failed due to CTRL/C (special case of
				   CacheError() */
#define CACHE_CONBROKEN    -4	/* We broke the connection */
#define CACHE_STRTOOLONG   -3	/* A string argument is too long */
#define CACHE_ALREADYCON   -2
#define CACHE_FAILURE	   -1
#define CACHE_SUCCESS 	    0

/*	The following are possible errors that Cache could return. To get
	the text associated with a particular error pass one of the following
	codes to CacheErrxlate().  After a call to one of the Cache interface
	routines returns one of these errors, calling CacheError() will return
	the contents of $ZE.  Note that $ZE is not set for all errors (egs.
	the <BREAK> generated from an argumentless break).

	Calling CacheErrxlate() with an error code out of range results in an
	CACHE_ERUNKNOWN return status

	Some Cache errors (CACHE_ERINTERRUPT and CACHE_ERCONBROKEN) are of
	special interest and are translated to one of the negative codes
	above.
*/

#define	CACHE_ERMAXNUM		  1	/* max number */
#define	CACHE_ERSELECT		  2	/* select */
#define	CACHE_ERSYNTAX		  3	/* syntax */
#define	CACHE_ERLABELREDEF	  4	/* labelredef */
#define	CACHE_ERFUNCTION	  6	/* function */
#define	CACHE_ERINTERRUPT	  7	/* interrupt */
#define	CACHE_ERNOLINE		  8	/* noline */
#define	CACHE_ERUNDEF		  9	/* undefined */
#define	CACHE_ERSYSTEM		 10	/* system */
#define	CACHE_EREDITED		 11	/* edited */
#define	CACHE_ERCOMMAND  	 12	/* command */
#define	CACHE_ERMAXARRAY	 13	/* maxarray */
#define	CACHE_ERNAME		 14	/* name */
#define	CACHE_ERSTORE		 15	/* store */
#define	CACHE_ERSUBSCR		 16	/* subscript */
#define	CACHE_ERNOROUTINE	 17	/* noroutine */
#define	CACHE_ERDIVIDE		 18	/* divide */
#define	CACHE_ERSTRINGSTACK	 20	/* string stack */
#define	CACHE_ERFRAMESTACK	 21	/* framestack */
#define	CACHE_ERUNIMPLEMENTED	 22	/* unimplemented */
#define	CACHE_EREXQUOTA		 23	/* exceeded quota (wsextent) */
#define	CACHE_ERNOTOPEN		 24	/* not open */
#define	CACHE_ERARGSTACK	 25	/* argstack */
#define	CACHE_ERM11TAPE		 26	/* m11 tape error */
#define	CACHE_ERPROTECT		 27	/* protect */
#define	CACHE_ERDATABASE	 28	/* database */
#define	CACHE_ERFILEFULL	 29	/* file full */
#define	CACHE_ERNAKED		 30	/* naked */
#define	CACHE_ERROUTINELOAD	 31 	/* routineload */
#define	CACHE_ERTERMINATOR	 32	/* terminator */
#define	CACHE_ERDISKHARD	 33	/* disk hard */
#define	CACHE_ERBLOCKNUM	 34	/* block # */
#define	CACHE_ERDEVNAME		 35	/* device name */
#define	CACHE_ERNOJOB		 36	/* nojob */
#define	CACHE_ERREAD		 37	/* read */
#define	CACHE_ERWRITE		 38	/* write */
#define	CACHE_EREOF		 39	/* end of file */
#define	CACHE_ERPARAMETER	 40	/* parameter */
#define	CACHE_ERNETWORK		 41	/* network */
#define	CACHE_ERRECOMPILE	 42	/* recompile */
#define	CACHE_ERDIR		 43	/* directory */
#define	CACHE_ERMAXSCOPE	 44	/* max scope */
#define	CACHE_ERCLUSTERFAIL	 45	/* cluster failed */
#define	CACHE_ERMAXERROR	 46	/* MAXERROR (internal problem) */
#define	CACHE_ERCONBROKEN	 47	/* lost an external connection */
#define	CACHE_ERNOMBX		 48
#define	CACHE_ERNOCP		 49
#define	CACHE_ERNOWD		 50
#define	CACHE_ERNOGC		 51
#define	CACHE_ERMTERR		 52	/* <MAGTAPE> */
#define	CACHE_ERDOWN		 54
#define	CACHE_ERCLNUP		 55
#define	CACHE_ERMAXPID		 56
#define	CACHE_ERNOSIGNON	 57	/* <SIGNONS INHIBITED> */
#define	CACHE_ERHALT		 58	/* <HALTED> */
#define	CACHE_ERWAITINIT	 59	/* <WAIT FOR M/xx INIT.. TO COMPLT> */
#define	CACHE_ERPRIMARY		 60	/* <NOT PRIMARY VOLUME>	  */
#define	CACHE_ERCORRUPT		 61	/* <CORRUPT VOLUME SET>	  */
#define	CACHE_ERMAPS		 62	/* <INVALID MAP NUMBER>	  */
#define	CACHE_ERVOLSET1		 63	/* <VOLUME SET ALREADY CREATED> */
#define	CACHE_ERFORMAT		 64	/* <VOLUME IS NOT FORMATTED>	   */
#define	CACHE_ERMAPRSV		 65	/* <CONFLICTING BLOCK NUMBERS>  */
#define	CACHE_ERCOLLATE		 66	/* <COLLATION> */
#define	CACHE_ERZTRP		 67	/* <ZTRAP> */
#define	CACHE_ERNODEV		 68	/* <NODEV> */
#define	CACHE_ERLANG		 69	/* <LANGUAGE MISMATCH> */
#define	CACHE_ERNOSYS		 70	/* <NOSYS> */
#define	CACHE_ERGIMPL		 71	/* <BADIMPLICIT> */
#define	CACHE_ERNGLOREF		 72	/* <NETGLOREF> */
#define	CACHE_ERNFORMAT		 73	/* <NETFORMAT> */
#define	CACHE_ERDSCON		 74	/* <DSCON> */
#define	CACHE_ERNVERSION	 75	/* <NETVERSION> */
#define CACHE_ERNJOBMAX		 76	/* <NETJOBMAX> */
#define CACHE_ERNRETRY		 77	/* <NETRETRY> */
#define	CACHE_ERNAMADD		 78	/* <NAMEADD> */
#define	CACHE_ERNETLOCK		 79	/* <NETLOCK> */
#define	CACHE_ERVALUE		 80	/* <ILLEGAL VALUE> */
#define	CACHE_ERNUMFIL		 81	/* <TOOMANYFILES> */
#define	CACHE_ERLIST		 82	/* <LIST> */
#define	CACHE_ERNAMSP		 83	/* <NAMESPACE> */
#define	CACHE_ERCMTFAIL		 84	/* <COMMITFAIL> */
#define	CACHE_ERROLLFAIL	 85	/* <ROLLFAIL> */
#define	CACHE_ERNSRVFAIL	 86	/* <NETSRVFAIL> */
#define	CACHE_ERCOLMISMAT	 87	/* <COLLATEMISMATCH> */
#define	CACHE_ERSLMSPAN		 88	/* <SLMSPAN> */
#define	CACHE_ERWIDECHAR	 89	/* <WIDE CHAR> */
#define	CACHE_ERLINELEV		 90	/* <LINELEVEL> */
#define	CACHE_ERARGDUP		 91	/* <DUPLICATEARG> */
#define	CACHE_ERGLODEF		 92	/* <GLODEFCHANGE> */
#define	CACHE_ERCOLCHG		 93	/* <COLLATECHANGE> */
#define	CACHE_ERRESTART		 94	/* <NORESTART> */
#define	CACHE_ERROUDEF		 95	/* <ROUDEFCHANGE> */
#define	CACHE_ERMNESPACE	 96	/* <MNEMONICSPACE> */
#define	CACHE_ERTRANSLOCK	 97	/* <INVTRANSLOCK> */
#define	CACHE_ERTRANSQUIT	 98	/* <INVTRANSQUIT> */
#define	CACHE_ERECODETRAP	 99	/* <ECODETRAP> */
#define	CACHE_LOCKLOST		100	/* <LOCKLOST> */
#define	CACHE_DATALOST		101	/* <DATALOST> */
#define	CACHE_TRANSLOST		102	/* <TRANSLOST> */
#define	CACHE_ERZOLE		103	/* <ZOLE> */
#define	CACHE_ERLICENSE		104	/* <LICENSE LIMIT EXCEEDED> */
#define	CACHE_ERNOLICSERV	105 	/* <LICENSE SERVER UNAVAILABLE> */
#define	CACHE_ERCOLNSUP		106	/* <COLLATION NOT SUPPORTED> */
#define	CACHE_ERNLSTAB		107	/* <NLS TABLE> */
#define	CACHE_ERXLATE		108	/* <TRANSLATE> */
#define	CACHE_ERNULLVAL		109	/* <NULL VALUE> */
#define	CACHE_ERNOVAL		110	/* <MISSING VALUE> */
#define	CACHE_ERUNKTYPE		111	/* <UNKNOWN TYPE> */
#define	CACHE_ERVALRANGE	112	/* <VALUE OUT OF RANGE> */
#define	CACHE_ERRANGE		113	/* <RANGE> */
#define	CACHE_ERDOMAINSPACERETRY 114	/* <DOMAINSPACERETRY> */
#define CACHE_ERCSMISMATCH	115	/* <CLIENT-SERVER MISMATCH> */
#define CACHE_ERSTRMISMATCH	116	/* <STRMISMATCH> */
#define CACHE_ERBADCLASS	117	/* <INVALID CLASS> */
#define CACHE_ERIOBJECT		118	/* <INTERNAL OBJECT ERROR> */
#define CACHE_ERBADOREF		119	/* <INVALID OREF> */
#define CACHE_ERNOMETHOD	120	/* <METHOD DOES NOT EXIST> */
#define CACHE_ERNOPROPERTY	121	/* <PROPERTY DOES NOT EXIST> */
#define CACHE_ERNOCLASS		122	/* <CLASS DOES NOT EXIST> */
#define CACHE_ERCLASSDESC	123	/* <CLASS DESCRIPTOR> */
#define CACHE_ERCLASSCOMP	124	/* <CLASS COMPILING> */
#define CACHE_ERCLASSRECOMP	125	/* <CLASS RECOMPILED> */
#define	CACHE_ERCLASSMEMORY	126	/* <INSUFFICIENT CLASS MEMORY> */
#define	CACHE_ERNUMCLASS	127	/* <TOO MANY USERS OF CLASS> */
#define	CACHE_ERNUMOREF		128	/* <TOO MANY OREFS> */
#define	CACHE_ERGETINVALID	129	/* <CANNOT GET THIS PROPERTY> */
#define	CACHE_ERSETINVALID	130	/* <CANNOT SET THIS PROPERTY> */
#define CACHE_ERREMCLASSRECOMP	131	/* <REMOTE CLASS RECOMPILED> */
#define CACHE_ERREMCLASSEDITED	132	/* <REMOTE CLASS EDITED> */
#define CACHE_ERCLASSEDITED	133	/* <CLASS EDITED> */
#define CACHE_ERCLASSLOAD	134	/* <CLASS TOO BIG TO LOAD> */
#define CACHE_ERCLASSSIZE	135	/* <CLASS TOO BIG TO SAVE> */
#define CACHE_ERCORRUPTOBJECT	136	/* <CORRUPT OBJECT> */
#define CACHE_ERDISCONNECT	137	/* <DISCONNECT> */
#define CACHE_ERNOCURRENTOBJECT	138	/* <NO CURRENT OBJECT> */
#define CACHE_ERZFHEAPSPACE	139	/* <OUT OF $ZF HEAP SPACE> */
#define CACHE_ERTOOMANYCLASSES	140	/* <TOO MANY CLASSES> */
#define CACHE_ERNETSTRMISMATCH  141 /* <NETWORK DATA UPDATE FAILED - STRMISMATCH> */
#define CACHE_ERNETSSTACK       142 /* <NETWORK DATA UPDATE FAILED - STRINGSTACK> */
#define CACHE_ERNETDIR          143 /* <NETWORK DATA UPDATE FAILED - DIRECTORY> */
#define	CACHE_ERNETDKHER        144 /* <NETWORK DATA UPDATE FAILED - DISKHARD> */
#define CACHE_ERNETDBDGD        145 /* <NETWORK DATA UPDATE FAILED - DATABASE> */
#define	CACHE_ERNETCLUSTER      146 /* <NETWORK DATA UPDATE FAILED - CLUSTERFAILED> */
#define	CACHE_ERNETCSMISMATCH   147 /* <NETWORK DATA .... - CLIENT-SERVER MISMATCH> */
#define CACHE_ERNETPROT         148 /* <NETWORK DATA UPDATE FAILED - PROTECT> */
#define	CACHE_ERNETDKSER        149 /* <NETWORK DATA UPDATE FAILED - BLOCKNUMBER> */
#define	CACHE_ERNETDKFUL        150 /* <NETWORK DATA UPDATE FAILED - DISKFULL> */
#define	CACHE_ERNETMXSTR        151 /* <NETWORK DATA UPDATE FAILED - MAXSTRING> */
#define	CACHE_ERNETSBSCR        152 /* <NETWORK DATA UPDATE FAILED - SUBSCRIPT> */
#define	CACHE_ERNETSYSTM        153 /* <NETWORK DATA UPDATE FAILED - SYSTEM> */
#define	CACHE_ERNETWIDECHAR     154 /* <NETWORK DATA UPDATE FAILED - WIDECHAR> */
#define CACHE_MSQLERNETNVERSION 155 /* <NETWORK DATA UPDATE FAILED - NETVERSION> */
#define CACHE_MSQLERNETNFORMAT  156 /* <NETWORK DATA UPDATE FAILED - NETFORMAT> */
#define	CACHE_MSQLERNETNGLOREF  157 /* <NETWORK DATA UPDATE FAILED - NETGLOREF>  */
#define CACHE_ERBADBITSTR	158	/* <INVALID BIT STRING> */
#define CACHE_ERROUTINESIZE	159	/* <ROUTINE TOO BIG TO SAVE> */
#define CACHE_ERNOSOURCE	160	/* <NO SOURCE> */
#define CACHE_ERMALLOC		161	/* <STORE> - malloc failed */
#define CACHE_EROREFNO		162	/* <INVALID TYPE> */
#define CACHE_ERNOOREF		163	/* <INVALID OREF> */
#define	CACHE_ERINVARG		164	/* <INVALID ARGUMENT> */
#define	CACHE_ERDYNLOAD		165	/* <DYNAMIC LIBRARY LOAD> */
#define CACHE_NOJVM		166	/* <Java VM not loaded> */
#define CACHE_ERPRIVPROP	167	/* <PRIVATE PROPERTY> */
#define CACHE_ERPRIVMETH	168	/* <PRIVATE METHOD> */
#define CACHE_ERJAVAEXCEPTION	169	/* <Java Exception> */

#define CACHE_ERMAX		169	/* Maximum error number */

#ifdef	__VMS
/* CacheSignal is not implemented on Cache for VMS systems as VMS versions
   of Cache do not use signal handling. It is provided here as returning
   CACHE_SUCCESS so that programs compiled for VMS and for Unix do not need
   to be sensitive to this difference
*/
#define CacheSignal(x) CACHE_SUCCESS
#endif

#ifdef	CACHE_UNICODE	/* Unicode character strings */
#define	CacheCvtIn	CacheCvtInW
#define	CacheCvtOut	CacheCvtOutW
#define	CacheError	CacheErrorW
#define	CacheErrxlate	CacheErrxlateW
#define	CacheEval	CacheEvalW
#define	CacheExecute	CacheExecuteW
#define	CachePrompt	CachePromptW
#define	CacheStart	CacheStartW
#else			/* ASCII character strings */
#define	CACHESTR	CACHE_ASTR
#define	CACHESTRP	CACHE_ASTRP
#define	CACHE_STRING	CACHE_ASTRING
#define	CacheCvtIn	CacheCvtInA
#define	CacheCvtOut	CacheCvtOutA
#define	CacheError	CacheErrorA
#define	CacheErrxlate	CacheErrxlateA
#define	CacheEval	CacheEvalA
#define	CacheExecute	CacheExecuteA
#define	CachePrompt	CachePromptA
#define	CacheStart	CacheStartA
#endif



typedef const struct  CacheExportFuncTab {
   int (*CacheExecuteA)();		/* Traditional callin functions */
   int (*CacheExecuteW)();
   int (*CacheEvalA)();
   int (*CacheEvalW)();
   int (*CachePromptA)();
   int (*CachePromptW)();
   int (*CacheErrxlateA)();
   int (*CacheErrxlateW)();
   int (*CacheErrorA)();
   int (*CacheErrorW)();
   int (*CacheConvert)();
   int (*CacheType)();
   int (*CacheAbort)();
#ifndef	__VMS		/* doesn't exist on VMS. Macro CACHESIGNAL defined
			   on VMS as a noop.*/
   int (*CacheSignal)();
#endif

   int (*CachePushInt)();		/* Higher performance interface */
   int (*CachePushDbl)();
   int (*CachePushStr)();
   int (*CachePushStrW)();
   int (*CachePushRtn)();
   int (*CachePushRtnX)();
   int (*CachePushRtnW)();
   int (*CachePushRtnXW)();
   int (*CachePushFunc)();
   int (*CachePushFuncX)();
   int (*CachePushFuncW)();
   int (*CachePushFuncXW)();
   int (*CachePushGlobal)();
   int (*CachePushGlobalX)();
   int (*CachePushGlobalW)();
   int (*CachePushGlobalXW)();
   int (*CacheUnPop)();
   int (*CachePop)();
   int (*CachePopInt)();
   int (*CachePopDbl)();
   int (*CachePopStr)();
   int (*CachePopStrW)();
   int (*CacheSetVar)();
   int (*CacheGetVar)();
   int (*CacheDoRtn)();
   int (*CacheExtFun)();
   int (*CacheGlobalGet)();
   int (*CacheGlobalSet)();
   int (*CacheDoFun)();						/* LRS512 */
   int (*CachePushClassMethod)();				/* RAM968+ */
   int (*CachePushClassMethodW)();
   int (*CachePushMethod)();
   int (*CachePushMethodW)();
   int (*CacheInvokeClassMethod)();
   int (*CacheInvokeMethod)();
   int (*CachePushProperty)();
   int (*CachePushPropertyW)();
   int (*CacheGetProperty)();
   int (*CacheSetProperty)();					/* RAM968- */
   int (*CachePushOref)();					/* CDS188+ */
   int (*CachePopOref)();
   int (*CacheIncrementCountOref)();
   int (*CacheCloseOref)();					/* CDS188- */
   int (*CachePushCvtW)();					/* LRS582+ */
   int (*CachePopCvtW)();
   int (*CachePushList)();
   int (*CachePopList)();					/* LRS582- */
   int (*CachePushPtr)();					/* LRS609+ */
   int (*CachePopPtr)();					/* LRS609- */
   int (*CacheGlobalIncrement)();				/* JO1759 */
} CACHEEFT, *CACHEEFTP;

/* Interface Function Definition */
#define	CACHEEXECUTE(cmd)	(*cachefunctabp->CacheExecute)(cmd)
#define	CACHEEXECUTEW(cmd)	(*cachefunctabp->CacheExecuteW)(cmd)
#define	CACHEEVAL(expr)		(*cachefunctabp->CacheEval)(expr)
#define	CACHEEVALW(expr)	(*cachefunctabp->CacheEvalW)(expr)
#define	CACHEPROMPT(rbuf)	(*cachefunctabp->CachePrompt)(rbuf)
#define	CACHEPROMPTW(rbuf)	(*cachefunctabp->CachePromptW)(rbuf)
#define	CACHEERRXLATE(code,rbuf) (*cachefunctabp->CacheErrxlate)(code,rbuf)
#define	CACHEERRXLATEW(code,rbuf) (*cachefunctabp->CacheErrxlateW)(code,rbuf)
#define CACHEERROR(msg,src,offp) (*cachefunctabp->CacheError)(msg,src,offp)
#define CACHEERRORW(msg,src,offp) (*cachefunctabp->CacheErrorW)(msg,src,offp)
#define	CACHECONVERT(type,rbuf)	(*cachefunctabp->CacheConvert)(type,rbuf)
#define	CACHETYPE()		(*cachefunctabp->CacheType)()
#define	CACHEABORT(type)	(*cachefunctabp->CacheAbort)(type)
#ifdef	__VMS
#define	CACHESIGNAL(signal)	MSQLSUCCESS
#else
#define	CACHESIGNAL(signal)	(*cachefunctabp->CacheSignal)(signal)
#endif

#define	CACHEPUSHINT(num)	(*cachefunctabp->CachePushInt)(num)
#define	CACHEPUSHDBL(num)	(*cachefunctabp->CachePushDbl)(num)
#define	CACHEPUSHOREF(oref)	(*cachefunctabp->CachePushOref)(oref)
#define	CACHEPUSHSTR(len,ptr)	(*cachefunctabp->CachePushStr)(len,ptr)
#define	CACHEPUSHSTRW(len,ptr)  (*cachefunctabp->CachePushStrW)(len,ptr)
#define	CACHEPUSHRTN(rflag,tlen,tptr,nlen,nptr)		\
		(*cachefunctabp->CachePushRtn)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHRTNX(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
		(*cachefunctabp->CachePushRtnX)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHRTNW(rflag,tlen,tptr,nlen,nptr)		\
		(*cachefunctabp->CachePushRtnW)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHRTNXW(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
		(*cachefunctabp->CachePushRtnXW)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHFUNC(rflag,tlen,tptr,nlen,nptr)		\
		(*cachefunctabp->CachePushFunc)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHFUNCX(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
		(*cachefunctabp->CachePushFuncX)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHFUNCW(rflag,tlen,tptr,nlen,nptr)		\
		(*cachefunctabp->CachePushFuncW)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHFUNCXW(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
		(*cachefunctabp->CachePushFuncXW)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHGLOBAL(nlen,nptr)  (*cachefunctabp->CachePushGlobal)(nlen,nptr)
#define	CACHEPUSHGLOBALX(nlen,nptr,elen,eptr) \
		(*cachefunctabp->CachePushGlobalX)(nlen,nptr,elen,eptr)
#define	CACHEPUSHGLOBALW(nlen,nptr)  (*cachefunctabp->CachePushGlobalW)(nlen,nptr)
#define	CACHEPUSHGLOBALXW(nlen,nptr,elen,eptr) \
		(*cachefunctabp->CachePushGlobalXW)(nlen,nptr,elen,eptr)
#define CACHEUNPOP()		(*cachefunctabp->CacheUnPop)()
#define CACHEPOP(arg)		(*cachefunctabp->CachePop)(arg)
#define CACHEPOPINT(nump)	(*cachefunctabp->CachePopInt)(nump)
#define CACHEPOPDBL(nump)	(*cachefunctabp->CachePopDbl)(nump)
#define CACHEPOPOREF(orefp)	(*cachefunctabp->CachePopOref)(orefp)
#define CACHEPOPSTR(lenp,strp)	(*cachefunctabp->CachePopStr)(lenp,strp)
#define CACHEPOPSTRW(lenp,strp)	(*cachefunctabp->CachePopStrW)(lenp,strp)
#define CACHESETVAR(len,namep) 	(*cachefunctabp->CacheSetVar)(len,namep)
#define CACHEGETVAR(len,namep) 	(*cachefunctabp->CacheGetVar)(len,namep)
#define CACHEDORTN(flags,narg)	(*cachefunctabp->CacheDoRtn)(flags,narg)
#define CACHEEXTFUN(flags,narg)	(*cachefunctabp->CacheExtFun)(flags,narg)
#define CACHEGLOBALGET(narg,flag) (*cachefunctabp->CacheGlobalGet)(narg,flag)
#define CACHEGLOBALSET(narg)	(*cachefunctabp->CacheGlobalSet)(narg)
								/* JO1759+ */
#define CACHEGLOBALINCREMENT(narg) (*cachefunctabp->CacheGlobalIncrement)(narg)
								/* JO1759- */
#define CACHEDOFUN(flags,narg)	(*cachefunctabp->CacheDoFun)(flags,narg)
								/* RAM968+ */
#define	CACHEPUSHCLASSMETHOD(clen,cptr,mlen,mptr,flg)			\
		(*cachefunctabp->CachePushClassMethod)(clen,cptr,mlen,mptr,flg)
#define	CACHEPUSHCLASSMETHODW(clen,cptr,mlen,mptr,flg)			\
	       (*cachefunctabp->CachePushClassMethodW)(clen,cptr,mlen,mptr,flg)
#define	CACHEPUSHMETHOD(oref,mlen,mptr,flg)				\
		(*cachefunctabp->CachePushMethod)(oref,mlen,mptr,flg)
#define	CACHEPUSHMETHODW(oref,mlen,mptr,flg)				\
		(*cachefunctabp->CachePushMethodW)(oref,mlen,mptr,flg)
#define CACHEINVOKECLASSMETHOD(narg)					\
		(*cachefunctabp->CacheInvokeClassMethod)(narg)
#define CACHEINVOKEMETHOD(narg)	(*cachefunctabp->CacheInvokeMethod)(narg)
#define	CACHEPUSHPROPERTY(oref,plen,pptr)				\
		(*cachefunctabp->CachePushProperty)(oref,plen,pptr)
#define	CACHEPUSHPROPERTYW(oref,plen,pptr)				\
		(*cachefunctabp->CachePushPropertyW)(oref,plen,pptr)
#define CACHEGETPROPERTY()	(*cachefunctabp->CacheGetProperty)()
#define CACHESETPROPERTY()	(*cachefunctabp->CacheSetProperty)()
								/* RAM968- */
#define	CACHEINCREMENTCOUNTOREF(oref)	(*cachefunctabp->CacheIncrementCountOref)(oref)
#define	CACHECLOSEOREF(oref)	(*cachefunctabp->CacheCloseOref)(oref)
#define	CACHEPUSHCVTW(len,ptr)  (*cachefunctabp->CachePushCvtW)(len,ptr)
#define CACHEPOPCVTW(lenp,strp)	(*cachefunctabp->CachePopCvtW)(lenp,strp)
#define	CACHEPUSHLIST(len,ptr)  (*cachefunctabp->CachePushList)(len,ptr)
#define CACHEPOPLIST(lenp,strp)	(*cachefunctabp->CachePopList)(lenp,strp)
#define CACHEPUSHPTR(ptr)	(*cachefunctabp->CachePushPtr)(ptr)
#define CACHEPOPPTR(ptrp)	(*cachefunctabp->CachePopPtr)(ptrp)

#ifdef	__VMS							/* LRS666+ */
#pragma	__required_pointer_size	__restore
#endif								/* LRS666- */

#ifdef __cplusplus
}
#endif
