/*
	File: cdzf.h

	Structures used for $ZF callout functions

	Copyright (c) 1986-2003 by InterSystems.
	Cambridge, Massachusetts, U.S.A.  All rights reserved.

Modification History: (most recent first)

LRS666	2/21/03	change macro UNICODE to CACHE_UNICODE to avoid conflict
LRS605	6/03/02	add defines ZF_SUCCESS and ZF_FAILURE
LRS582	4/09/02	add types 'h', 'q', and 'l', for Unicode boundary translation
JLC334	4/18/01	adapt to 64-bit VMS. Make all pointers short (32-bit).
CDS188	4/12/01 Chuck Sorenson, system oref type
CLN040	9/06/00	Huayong Yang, Cleaned
CDS073	5/03/00 add type 'k' for argstk pointer.  This is for internal
		use only!
LRS421	3/13/00 add tests for C++ compilation, add 'const' to struct zfestr
LRS412	1/28/00 add zfresjob() to callback function table, modified some
		declarations to allow compiling with C++
LRS409	1/27/00 add callback function table for DLL's and shared objects,
		restructure this source file
CDS057 12/08/99 Add type 'y' for generic void pointer.  This is for
		internal use only!
LRS302	1/29/99 modify procedure for defining a DLL or shared object. The
		ZFDLL macro invocation will no longer be used, instead just
		define ZF_DLL for any platform. ZFDLLBEGIN is replaced by
		the standard ZFBEGIN call.
RYF091	7/29/98 add type 'z' for call-in linkage vector. This is a pseudo
	        argument in that it does not correspond to an argument in
		the $ZF call (although the position must be filled in in
		the $ZF call) but it passes a pointer to a CACHEEFT
		(see callin header files) structure which is the set of
		vectors you need to call back into Cache. This is used by
		DLL's which do not link against the image and so cannot
		call these routine directly. Change #if OSVMS to #ifdef __VMS
		as this must build w/out our standard include files.
RAM631	9/08/97 added the definition of the ZFDLL macro, which generates
		the GetZFTable function for a DLL containing user-written
		routines which are to be invoked via the
		$ZF(-3,"dll_name","routine_name") function.
MEB040	6/02/97	add support for translated character strings
MEB027	3/20/97	add support for Unicode character strings
CFL001  7/29/96 added prototypes for signal handling functions.
RJW050	7/03/96	create dll version of openm. 
AHS069	2/25/93	Support for byte arrays.
JAO552  1/12/93 port to VMS.
SPJ169	9/14/88	Rewrote to fix bugs and make portable
KSO212  8/11/88	Part of $ZF for common code.

***********************************************************************/

#ifdef	__cplusplus						/* LRS421+ */
extern	"C" {
#endif								/* LRS421- */

#ifdef	__VMS							/* JLC334+ */
#pragma	__required_pointer_size	__save
#pragma	__required_pointer_size	__short
#endif								/* JLC334- */

/*
	Counted string argument descriptors
*/
typedef struct zarray {						/* AHS069+ */
	unsigned short	len;
	unsigned char	data[1];	/* 1 is a dummy value */
	} *ZARRAYP;						/* AHS069- */

typedef struct zwarray {					/* MEB027+ */
	unsigned short	len;
	unsigned short	data[1];	/* 1 is a dummy value */
	} *ZWARRAYP;						/* MEB027- */

/*	Argument descriptor values 

	reserve 'u' for unsigned integer
		'x' for extended precision
*/
#define DINT	'i'	/* Integer */
#define DPINT	'p'	/* Pointer to integer */
#define DPCHAR	'c'	/* Zero terminated string (8 bit characters) */
#define DPFLOAT	'f'	/* Float */
#define DPDBL	'd'	/* Double */
#define DPVDESC	'v'	/* VMS fixed length string descriptor */
#define	DPBYTE	'b'	/* Counted byte array */		/* AHS069 */
#define	DPTSTR	't'	/* Translated string (in byte array) */	/* MEB040 */
#define	DPCWSTR 's'	/* Counted string (16 bit characters) *//* MEB027+ */
#define	DPWCHAR	'w'	/* Zero terminated string (16 bit characters) */
#define DPARGSTK 'k'	/* Pointer to argstk element */		/* CDS073 */
#define DPVOID	'y'	/* Generic void pointer */		/* CDS057 */
#define DOREF	'o'	/* Object reference */			/* CDS188 */
								/* RYF091+ */
#define DCILINK 'z'	/* Call-in linkage vector. input only. Need */
			/* to include mcallin.h or ccallin.h as well */
								/* RYF091- */
								/* LRS582+ */
#define	DCVWSTR 'h'	/* Counted wide string with translation */
#define	DCVWCHR	'q'	/* Zero terminated string with translation */
#define	DCVLIST	'l'	/* $LIST object with translation */
								/* LRS582- */
/* Argument descriptor characters may be either lowercase or uppercase.
   The lowercase value is used in switch statement to distinguish
   argument types. Uppercase counterparts indicate that the argument
   is an output argument. The following macros work with an argument
   descriptor character that may be either upper or lower case.
*/
#define ZFARGTYPE(x)  ((x) | 0x20)	/* OR in bit that coerces character
					   to lowercase */
#define ISZFOUTARG(x) !((x) & 0x20)	/* Check for absence of bit that
					   indicates uppercase character */
								/* MEB027- */

#ifdef	__VMS							/* JAO552+ */
#define NOPRIV 1	/* Execute $ZF function at user's priv level */
#define PRIV   0	/* Execute $ZF function at elevated priv level */
#endif								/* JAO552- */

/*
	Table of remote routine descriptors
*/
struct zfestr {
	const char  *zfename;	/* Address of function name string */
	const char  *zfead;	/* Address of argument descriptor string */
	int	   (*zfeep)();	/* Function entry point address */
#ifdef	__VMS							/* JAO552+ */
	int	     zfprivs;	/* Set if we drop privs for this $ZF func */
#endif								/* JAO552- */
};

typedef	int	(*zffunc)();

/*	To allow czf.c files to be moved unchanged between VMS and other
	systems (eg. Unix and NT), the following macros have the following
	effects:
		ZFENTRY on VMS defines a $ZF function that executes at
			the caller's normal privilege level. This is the
			way $ZF functions behave on Unix.
		ZFPRIV allows the programmer to specify whether the $ZF
			function should execute at the caller's privilege
			level or at the elevated privilege level processes
			have when they're in Cache (all privs except share).
			For Unix this function is identical to ZFENTRY as
			all $ZF functions execute at the caller's level.

	Note: The original default behavior on the VAX was to execute
	      $ZF functions at an elevated privilege level. This module
	      defaults (via ZFENTRY) to a normal privilege level.
*/

								/* LRS409+ */
/*
	Function Prototypes
*/

#if	defined(__STDC__) || defined(__cplusplus)		/* LRS412 */
#define	ZFPROTO(func,args)	func args
#else
#define	ZFPROTO(func,args)	func ()
#endif

typedef const struct  CacheCallbackTab {
   int	ZFPROTO((*ZFSTRSIZE),(void));
   int	ZFPROTO((*ZFERROR),(int,int));
   int	ZFPROTO((*CACHEMGRDIRA),(int,char *));
   int	ZFPROTO((*CACHEMGRDIRW),(int,unsigned short *));
#if	!defined(__VMS) && !defined(WIN32)	/* These are only for UNIX */
   int	ZFPROTO((*SIGRTCHK),());
   int	ZFPROTO((*SIGRTCLR),());
   int	ZFPROTO((*DZFALARM),());
#endif
   int	ZFPROTO((*ZFRESJOB),(void));				/* LRS412 */
} CACHECBT, *CACHECBTP;

#ifndef	ZF_DLL		/*==============================*/

/*
	Direct (linked-in) Callback Function Declarations

	zfstrsize - returns max length of $ZF reply string
	zferror - used to report errors as though from within Cache
	CacheMgrDirA - obtains Cache manager's directory (narrow)
	CacheMgrDirW - obtains Cache manager's directory (wide)
*/
extern	int	ZFPROTO(zfstrsize,(void));
extern	int	ZFPROTO(zferror,(int,int));
extern	int	ZFPROTO(zfresjob,(void));			/* LRS412 */
extern	int	ZFPROTO(CacheMgrDirA,(int,char *));
extern	int	ZFPROTO(CacheMgrDirW,(int,unsigned short *));
								/* CFL001+ */
/*
	Prototypes for signal handling helper functions in $ZF
	for UNIX systems only.
*/

/*
*  sigrtchk() checks for asynchronous events.
*      Should be called whenever a system call fails because of a
*      received signal (usually open(), close(), read(), write(),
*      ioctl() and others that may take an unpredictable amount
*      of time to complete). It returns a code indicating the action
*      the user should take:
*
*     Return code   Cause                Action
*     ----------    -----------------    -------------------------
*         -1        Not a signal         Check for I/O error.
*          1        SIGINT or SIGTERM    Exit $ZF.
*          0        Other signal         Restart operation.
*/
extern int sigrtchk();

/*
*  sigrtclr() clears retry flag.
*      Should be called once before using sigrtchk()
*/
extern int sigrtclr();

/*
*  dzfalarm() establishes new SIGALRM handler.
*      On entry to $ZF, the previous handler is saved automatically.
*      On exit, it's restored automatically.
*/
extern int dzfalarm();
								/* CFL001- */
#else	/* ZF_DLL */	/*==============================*/

/*
	Indirect (DLL) Callback Function Table
*/

#ifndef	CACHETYPE		/* Were callin declarations included? */
typedef void *	CACHEEFTP;	/* If not, create a dummy pointer type */
#endif
extern	CACHEEFTP	cachefunctabp;
extern	CACHECBTP	cachecallbackp;

/*
	Interface Function Definition
*/
#define	zfstrsize()		(*cachecallbackp->ZFSTRSIZE)()
#define	zferror(errnum,mode)	(*cachecallbackp->ZFERROR)(errnum,mode)
#define	zfresjob()		(*cachecallbackp->ZFRESJOB)()
#define CacheMgrDirA(len,namep)	(*cachecallbackp->CACHEMGRDIRA)(len,namep)
#define CacheMgrDirW(len,namep)	(*cachecallbackp->CACHEMGRDIRW)(len,namep)
#if	!defined(__VMS) && !defined(WIN32)
#define	sigrtchk()		(*cachecallbackp->SIGRTCHK)();
#define	sigrtclr()		(*cachecallbackp->SIGRTCLR)();
#define	dzfalarm()		(*cachecallbackp->DZFALARM)();
#endif

#endif	/* ZF_DLL */	/*==============================*/

#ifdef	CACHE_UNICODE
#define	CacheMgrDir(len,namep)	CacheMgrDirW(len,namep)
#else
#define	CacheMgrDir(len,namep)	CacheMgrDirA(len,namep)
#endif
								/* LRS409- */

#ifdef  OPENM_DLL_IMPORT					/* RJW050+ */
_declspec( dllimport )	struct zfestr zfe[];
#else								/* RJW050- */
#ifdef	ZF_DLL							/* RAM631+ */
extern struct zfestr zfedll[];
#else								/* LRS409 */
extern struct zfestr zfe[];
#endif								/* RAM631- */
#endif								/* RJW050 */

#ifdef	ZF_DLL							/* LRS409+ */
#define	ZFBEGIN				\
	CACHECBTP	cachecallbackp;	\
	CACHEEFTP	cachefunctabp;	\
struct zfestr *GetZFTable(		\
	CACHECBTP	cbtp,		\
	CACHEEFTP	eftp)		\
{					\
	cachecallbackp = cbtp;		\
	cachefunctabp = eftp;		\
	return zfedll;			\
}					\
struct zfestr zfedll[] = {
#else	/* ZF_DLL */						/* LRS409- */
#define	ZFBEGIN	struct zfestr zfe[] = {
#endif	/* ZF_DLL */						/* LRS302 */

#ifndef	__VMS							/* JAO552+ */
#define	ZFENTRY(n,a,f)	{n,a,f},
#define	ZFPRIV		ZFENTRY
#define	ZFEND		{(char *) 0, (char *) 0, (zffunc) 0}};
#define	ZF_SUCCESS	0	/* Return code for success */	/* LRS605 */
#define	ZF_FAILURE	1	/* Return code for failure */	/* LRS605 */
#else								/* JAO552= */
#define	ZFPRIV(n,a,f,p) {n,a,(zffunc)f,p},
#define	ZFENTRY(n,a,f)	ZFPRIV(n,a,f,NOPRIV)
#define	ZFEND		{(char *) 0, (char *) 0, (zffunc) 0,0}};
#define	ZF_SUCCESS	1	/* Return code for success */	/* LRS605 */
#define	ZF_FAILURE	3	/* Return code for failure */	/* LRS605 */
#pragma	__required_pointer_size	__restore			/* JLC334 */
#endif								/* JAO552- */

#ifdef	__cplusplus						/* LRS421+ */
}
#endif								/* LRS421- */
