/*
	Definitions for InterSystems' Call-In Interface

	Copyright (c) 1992-2003 by InterSystems.
	Cambridge, Massachusetts, U.S.A.  All rights reserved.
	Confidential, unpublished property of InterSystems.

	This module contains the definitions necessary to build a 'C'
	language program that utilizes InterSystem's call-in
	interface.
*/

/*
	Starting with Cache V2.1 new customer applications should be compiled
	using the definitions in ccallin.h. This file will continue to be used
	for internal purposes and must be maintained accordingly. New function
	prototypes should be added to the function prototype section below and
	to the internal entrypoint name mapping section and function prototype
	section in ccallin.h. New sybmbol definitions should be added to the
	appropriate section below and to ccallin.h.
*/

/*			 Data Structures
 */

/*	The following typedefs produces a pointer to a counted string
	of the type the Cache interface routines expect as input.  If this
	were the actual string then the str[] element would have a length
	representing either the required string length to pass an element
	in, or the maximum string length of the expected item being returned.

	When passing an element in to a function as the return buffer, the
	length field should be filled in with the size of the largest string
	the structure can hold.  Failure to do this can result in either
	errors from the interface if the length field is out of range, or
	overwriting other memory structures if a string larger than the
	allocated space is returned.

	All strings handled by the calling support routines are treated as
	"CACHE strings". This means they can have a maximum size of
	CACHE_MAXSTRLEN. MSQL_MAXSTRLEN was maintained at the old value
	for customers who do not need the additional space and are concerned
	about allocating unnecessary storage (MSQL_MAXSTRINGs are 512 chars,
	CACHE_MAXSTRINGs are 32KB chars). These can really be used
	interchangable because whenever a structure is passed in it
	either specifies the length of the input string or the maximum length
	of the return string. Thus if you provide a MSQL_MAXSTRING size
	function to a routine that returns something larger, you will get
	an error rather than overwriting some other memory.
*/

#ifdef	__VMS							/* LRS666+ */
#pragma	__required_pointer_size	__save
#pragma	__required_pointer_size	__short
#endif								/* LRS666- */

#define MSQL_MAXSTRLEN 510

typedef struct msqlstr {
	unsigned short len;
	unsigned char  str[MSQL_MAXSTRLEN];
	} MSQLSTR, *MSQLSTRP;

typedef struct msqlwstr {
	unsigned short len;
	unsigned short str[MSQL_MAXSTRLEN];
	} MSQLWSTR, *MSQLWSTRP;


#define CACHE_MAXSTRLEN 32767

typedef struct cachestr {
   unsigned short len;
   unsigned char  str[CACHE_MAXSTRLEN];
} CACHESTR, *CACHESTRP;

typedef struct cachewstr {
   unsigned short len;
   unsigned short str[CACHE_MAXSTRLEN];
} CACHEWSTR, *CACHEWSTRP;


/*				Function Prototypes
*/
#ifdef	__STDC__
extern int  msqlstart(unsigned long,int,CACHESTRP,CACHESTRP);
extern int  msqlstartW(unsigned long,int,CACHEWSTRP,CACHEWSTRP);
extern int  msqlexecute(CACHESTRP VOL);
extern int  msqlexecuteW(CACHEWSTRP VOL);
extern int  msqleval(CACHESTRP VOL);
extern int  msqlevalW(CACHEWSTRP VOL);
extern int  msqlcontext(void);
extern int  msqlcvtin(CACHESTRP,CACHESTRP,CACHESTRP);
extern int  msqlcvtinW(CACHESTRP,CACHEWSTRP,CACHEWSTRP);
extern int  msqlcvtout(CACHESTRP,CACHESTRP,CACHESTRP);
extern int  msqlcvtoutW(CACHEWSTRP,CACHEWSTRP,CACHESTRP);
extern int  msqlprompt(CACHESTRP);
extern int  msqlpromptW(CACHEWSTRP);
extern int  msqlerrxlate(int,CACHESTRP);
extern int  msqlerrxlateW(int,CACHEWSTRP);
extern int  msqlctrl(unsigned long);
extern int  msqlconvert(unsigned long,unsigned char *);
extern int  msqltype(void);
extern int  msqlabort(unsigned long);
extern int  msqlend(void);
extern void msqloflush(void);
#ifndef	__VMS
extern int  msqlsignal(int);
#endif
extern int  msqlerror(CACHESTRP,CACHESTRP,int *);
extern int  msqlerrorW(CACHEWSTRP,CACHEWSTRP,int *);
#if	0	/* For Relational Gateway, no longer used */	/* LRS666+ */
extern int  msqlgetstr(char *,int,char *);
extern int  msqlsetstr(char *,int,char *);
extern char *msqlrefparse(int,unsigned char *);
extern int  msqlreffree(char *);
#endif		/* For Relational Gateway */			/* LRS666- */
extern int  CachePushInt(int);
extern int  CachePushDbl(double);
extern int  CachePushOref(unsigned int);			/* CDS188 */
extern int  CachePushStr(int,unsigned char *);
extern int  CachePushStrW(int,unsigned short *);
extern int  CachePushRtn(unsigned int *,int,unsigned char *,int,unsigned char *);
extern int  CachePushRtnX(unsigned int *,int,unsigned char *,int,int,unsigned char*,int,unsigned char *);
extern int  CachePushRtnW(unsigned int *,int,unsigned short *,int,unsigned short *);
extern int  CachePushRtnXW(unsigned int *,int,unsigned short *,int,int,unsigned short *,int,unsigned short *);
extern int  CachePushFunc(unsigned int *,int,unsigned char *,int,unsigned char *);
extern int  CachePushFuncX(unsigned int *,int,unsigned char *,int,int,unsigned char*,int,unsigned char *);
extern int  CachePushFuncW(unsigned int *,int,unsigned short *,int,unsigned short *);
extern int  CachePushFuncXW(unsigned int *,int,unsigned short *,int,int,unsigned short*,int,unsigned short *);
extern int  CachePushGlobal(int,unsigned char *);
extern int  CachePushGlobalX(int,unsigned char *,int,unsigned char *);
extern int  CachePushGlobalW(int,unsigned short *);
extern int  CachePushGlobalXW(int,unsigned short *,int,unsigned short *);
extern int  CacheUnPop();
extern int  CachePop(void **);
extern int  CachePopInt(int *);
extern int  CachePopDbl(double *);
extern int  CachePopOref(unsigned int *);			/* CDS188 */
extern int  CachePopStr(int *,unsigned char **);
extern int  CachePopStrW(int *,unsigned short **);
extern int  CacheSetVar(int,unsigned char *);
extern int  CacheGetVar(int,unsigned char *);
extern int  CacheDoRtn(unsigned int,int);
extern int  CacheDoFun(unsigned int,int);			/* LRS512 */
extern int  CacheExtFun(unsigned int,int);
extern int  CacheGlobalGet(int,int);
extern int  CacheGlobalSet(int);
extern int  CacheGlobalIncrement(int);				/* JO1759 */
								/* RAM968+ */
extern int  CachePushClassMethod(int,unsigned char *,int,unsigned char *,int);
extern int  CachePushClassMethodW(int,unsigned short *,int,unsigned short *,int);
extern int  CachePushMethod(unsigned int,int,unsigned char *,int);
extern int  CachePushMethodW(unsigned int,int,unsigned short *,int);
extern int  CacheInvokeClassMethod(int);
extern int  CacheInvokeMethod(int);
extern int  CachePushProperty(unsigned int,int,unsigned char *);
extern int  CachePushPropertyW(unsigned int,int,unsigned short *);
extern int  CacheGetProperty();
extern int  CacheSetProperty();
								/* RAM968- */
extern int  CacheIncrementCountOref(unsigned int);		/* CDS188 */
extern int  CacheCloseOref(unsigned int);			/* CDS188 */
extern int  CachePushCvtW(int,unsigned short *);		/* LRS582+ */
extern int  CachePopCvtW(int *,unsigned short **);
extern int  CachePushList(int,unsigned char *);
extern int  CachePopList(int *,unsigned char **);		/* LRS582- */
extern int  CachePushPtr(void *);				/* LRS609+ */
extern int  CachePopPtr(void **);				/* LRS609- */
#else
extern int  msqlstart();
extern int  msqlstartW();
extern int  msqlexecute();
extern int  msqlexecuteW();
extern int  msqleval();
extern int  msqlevalW();
extern int  msqlcontext();
extern int  msqlprompt();
extern int  msqlpromptW();
extern int  msqlerrxlate();
extern int  msqlerrxlateW();
extern int  msqlctrl();
extern int  msqlconvert();
extern int  msqltype();
extern int  msqlabort();
extern int  msqlend();
extern void msqloflush();
#ifndef	__VMS
extern int  msqlsignal();
#endif
extern int  msqlerror();
extern int  msqlerrorW();
extern int  msqlgetstr(); 	/* (char *ref,int len,char *str) */
extern int  msqlsetstr();	/* (char *ref,int len,char *str) */
extern char *msqlrefparse();	/* (int len,unsigned char *name) */
extern int  msqlreffree();	/* (char *ref) */
extern int  CachePushInt();
extern int  CachePushDbl();
extern int  CachePushOref();					/* CDS188 */
extern int  CachePushStr();
extern int  CachePushStrW();
extern int  CachePushRtn();
extern int  CachePushRtnX();
extern int  CachePushRtnW();
extern int  CachePushRtnXW();
extern int  CachePushFunc();
extern int  CachePushFuncX();
extern int  CachePushFuncW();
extern int  CachePushFuncXW();
extern int  CachePushGlobal();
extern int  CachePushGlobalX();
extern int  CachePushGlobalW();
extern int  CachePushGlobalXW();
extern int  CacheUnPop();
extern int  CachePop();
extern int  CachePopInt();
extern int  CachePopDbl();
extern int  CachePopOref();					/* CDS188 */
extern int  CachePopStr();
extern int  CachePopStrW();
extern int  CacheSetVar();
extern int  CacheGetVar();
extern int  CacheDoRtn();
extern int  CacheDoFun();					/* LRS512 */
extern int  CacheExtFun();
extern int  CacheGlobalGet();
extern int  CacheGlobalSet();
extern int  CacheGlobalIncrement();				/* JO1759 */
extern int  CachePushClassMethod();				/* RAM968+ */
extern int  CachePushClassMethodW();
extern int  CachePushMethod();
extern int  CachePushMethodW();
extern int  CacheInvokeClassMethod();
extern int  CacheInvokeMethod();
extern int  CachePushProperty();
extern int  CachePushPropertyW();
extern int  CacheGetProperty();
extern int  CacheSetProperty();					/* RAM968- */
extern int  CacheIncrementCountOref();				/* CDS188 */
extern int  CacheCloseOref();					/* CDS188 */
extern int  CachePushCvtW();					/* LRS582+ */
extern int  CachePopCvtW();
extern int  CachePushList();
extern int  CachePopList();					/* LRS582- */
extern int  CachePushPtr();					/* LRS609+ */
extern int  CachePopPtr();					/* LRS609- */
#endif

/* 				Constants
*/

/*	The following are the definitions that are used to form the flags
	argument to msqlstart() to control how M/SQL treats, or doesn't
	treat, the principal input device.

	Setting more than one of these flags (with the exception of the
	modifiers) will cause unpredictable results
*/
#define	MSQLTTALL	1	/* Indicates that M/SQL should initialize the
				   terminal's settings and restore them across
				   each call into, and return from, the
				   interface.  */
#define MSQLTTCALLIN	2	/* Indicates that M/SQL should initialize the
				   terminal each time its called but should
				   restore it only when the connection is
				   closed (eg. when msqlend() is called or
				   the connection is broken). */
#define MSQLTTSTART	4	/* Indicates that M/SQL should initialize the
				   terminal when the connection is formed and
				   reset it when the connection is terminated.
				   */
#define	MSQLTTNEVER	8	/* Indicates that M/SQL should not alter the
				   terminal's settings */
#define MSQLTTNONE	16	/* Indicates that M/SQL should not do any
				   output or input from the principle input
				   device. This is equivilent to specifying
				   the null device for princin and princout.
				   Output will be discarded and reads will
				   complete with end-of-file */
#define MSQLTTMASK	(MSQLTTALL|MSQLTTCALLIN|MSQLTTSTART|MSQLTTNEVER| \
			 MSQLTTNONE)
#define MSQLPROGMODE	32	/* mumps error do not cause halt in the
				   absense of error handler */
/*	The following modifier is allowed in conjunction with
		MSQLTTALL, MSQLTTCALLIN and MSQLTTSTART.
*/
#define MSQLTTNOUSE	0x10000 /* If set this indicates that M/SQL open and
				   use commands are not allowed to alter the
				   terminal as well.  This is implicitly
				   set by MSQLTTNEVER and MSQLTTNONE.
				   The default is to allow M/SQL open and
				   use commands to alter the terminal in the
				   other modes. */

/*	The following are the definitions that can be used to form the
	arguments to MSQLCTRL().  These flags are in pairs and if both
	members of a set are specified the results will be unpredictable.
*/
#define MSQLDISACTRLC	1	/* CTRL/C is ignored by M/SQL */
#define MSQLENABCTRLC	2	/* CTRL/C's in M/SQL generate an <INTERRUPT> */

/*	The following are the arguments to MSQLABORT that control the mechanism
	used to terminate M/SQL processing.

	MSQLCTRLC simulates a ctrl/c at the keyboard and takes effect even
	if MSQLCTRL() specified that ctrl/c's were to be ignored. The M/SQL
	call that is interrupted returns MSQLINTERRUPT.

	MSQLRESJOB simulates a resjob. This results in the M/SQL connection
	being broken and the M/SQL call that's interrupted completes with
	a status of MSQLCONBROKEN.
*/
#define	MSQLCTRLC 1
#define MSQLRESJOB 2

/* 	The following are the return values from msqltype() that describe
	the internal datatype of the result from the call to msqleval()
*/
#define	MSQLINT		 1	/* native (32-bit on the VAX) integer */
#define MSQLDOUBLE	 2	/* 64-bit floating point */
#define MSQLSTRING	 3	/* character string */

/*	In addition to the above the following can be used as the type
	argument to the MSQLCONVERT() function

	Note that floating point types (MSQLFLOAT, HFLOAT and DOUBLE) that
	have decimal parts (including negative exponents), as well as the
	64 bit integer types MSQLINT8 and MSQLUINT8, involve division when
	calculating the return value. Therefore the results may not be
	100% accurate.  MSQLSTRINT, MSQLINT8 and MSQLUINT8 can return the
	status MSQLRETTRUNC if the result of the last msqleval() call will
	not fit in the space allocated in msqlconvert(). In the case of
	MSQLINT8 and MSQLUINT8 this means that the expression resulted in
	a floating point # that could not be normalized to fit within
	64 bits.
*/
#define	MSQLCHAR	 4	/*   8 bit signed integer */
#define	MSQLINT2	 5	/*  16 bit signed integer */
#define	MSQLINT4	 6	/*  32 bit signed integer */
#define	MSQLINT8	 7	/*  64 bit signed integer */
#define	MSQLUCHAR	 8	/*   8 bit unsigned integer */
#define	MSQLUINT2	 9	/*  16 bit unsigned integer */
#define	MSQLUINT4	10	/*  32 bit unsigned integer */
#define	MSQLUINT8	11	/*  64 bit unsigned integer */
#define MSQLFLOAT	12	/*  32 bit floating point */
#define MSQLHFLOAT	13	/* 128 bit floating point */
#define	MSQLUINT	14	/* native size (32 bit on the VAX)
						 unsigned integer */
#define MSQLWSTRING	15	/* Unicode character string */
#define	MSQLOREF	16	/* object reference */


/*	The following are error codes that are generated by the callin
	interface as opposed to error codes that are generated by
	M/SQL and returned by the callin interface. */

#define MSQLNORES	-13	/* MSQLCONVERT was called to get a result but
				   there's no result to return */
#define MSQLBADARG	-12	/* A function was called with an argument
				   that was out of range or otherwise
				   invalid */
#define MSQLNOTINMSQL 	-11	/* Returned from MSQLABORT if the process
			  	   wasn't executing on the M/SQL side */
#define MSQLRETTRUNC 	-10	/* The return buffer was too small so only
				   part of the requested item was returned */
#define MSQLERUNKNOWN	-9	/* Unknown error code in persave */
#define MSQLRETTOOSMALL -8	/* return buffer too small, nothing returned */
#define MSQLNOCON 	-7	/* not connected to M/SQL */
#define MSQLINTERRUPT 	-6	/* Failed due to ctrl/c (special case of
					MSQLERROR) */
#define MSQLCONBROKEN 	-4	/* We broke the connection */
#define MSQLMAXSTRLEN 	-3	/* an argument is too long */
#define MSQLALREADYCON 	-2
#define MSQLFAILURE	-1
#define MSQLSUCCESS 	0

/*	The following are possible errors that M/SQL could return.  To
	get the text associated with a particular error pass one of the
	following codes to msqlxlate().  After a call to one of the msql
	interface routines returns one of these errors, calling msqlerrinfo
	will return the contents of $ZE.  Note that $ZE is not set for
	all errors (eg. the <BREAK> generated from an argumentless break).

	Calling msqlerrxlate with an error code out of range results
	in an MSQLERUNKNOWN return status

	Some M/SQL errors (MSQLERINTERRUPT and MSQLERCONBROKEN) are of
	special interest and are translated to one of the negative codes
	above.
*/

#define	MSQLERMAXNUM		1	/* max number */
#define	MSQLERSELEXT 		2	/* select */
#define	MSQLERSELECT		2	/* select */
#define	MSQLERSYNTAX		3	/* syntax */
#define	MSQLERADDNAME		4	/* nameadd */
#define	MSQLERLABELREDEF	4	/* labelredef */
#define	MSQLERFUNCTION		6	/* function */
#define	MSQLERINTERRUPT		7	/* interrupt */
#define	MSQLERNOLONE		8	/* noline */
#define	MSQLERNOLINE		8	/* noline */
#define	MSQLERUNDEF		9	/* undefined */
#define	MSQLERSYSTEM		10	/* system */
#define	MSQLEREDITED		11	/* edited */
#define	MSQLERCOMMAND  		12	/* command */
#define	MSQLERMAXARRAY		13	/* maxarray */
#define	MSQLERNAME		14	/* name */
#define	MSQLERSTORE		15	/* store */
#define	MSQLERSUBSCR		16	/* subscript */
#define	MSQLERNOROUTINE		17	/* noroutine */
#define	MSQLERDIVIDE		18	/* divide */
/* #define	MSQLERSYNTAX		19 */	/* syntax */
#define	MSQLERSTRINGSTACK	20	/* string stack */
#define	MSQLERFRAMESTACK	21	/* framestack */
#define	MSQLERUNIMPLEMENTED	22	/* unimplemented */
#define	MSQLEREXQUOTA		23	/* exceeded quota (wsextent) */
#define	MSQLERNOTOPEN		24	/* not open */
#define	MSQLERARGSTACK		25	/* argstack */
#define	MSQLERM11TAPE		26	/* m11 tape error */
#define	MSQLERPROTECT		27	/* protect */
#define	MSQLERDATABASE		28	/* database */
#define	MSQLERFILEFULL		29	/* file full */
#define	MSQLERNAKED		30	/* naked */
#define	MSQLERROUTINELOAD	31 	/* routineload */
#define	MSQLERTERMINATOR	32	/* terminator */
#define	MSQLERDISKHARD		33	/* disk hard */
#define	MSQLERBLOCKNUM		34	/* block # */
#define	MSQLERDEVNAME		35	/* device name */
#define	MSQLERNOJOB		36	/* nojob */
#define	MSQLERREAD		37	/* read */
#define	MSQLERWRITE		38	/* write */
#define	MSQLEREOF		39	/* end of file */
#define	MSQLERPARAMPASS		40	/* parampass */
#define	MSQLERPARAMETER		40	/* parameter */
#define	MSQLERNETWORK		41	/* network */
#define	MSQLERRECOMPILE		42	/* recompile */
#define	MSQLERDIR		43	/* directory */
#define	MSQLERMAXSCOPE		44	/* max scope */
#define	MSQLERCLUSTERFAIL	45	/* cluster failed */
#define	MSQLERMAXERROR		46	/* MAXERROR (internal problem) */
#define	MSQLERCONBROKEN		47	/* lost an external connection */
#define	MSQLERNOMBX		48
#define	MSQLERNOCP		49
#define	MSQLERNOWD		50
#define	MSQLERNOGC		51
#define	MSQLERMTERR		52	/* <MAGTAPE> */
#define	MSQLERDOWN		54
#define	MSQLERCLNUP		55
#define	MSQLERMAXPID		56
#define	MSQLERNOSIGNON		57	/* <SIGNONS INHIBITED> */
#define	MSQLERHALT		58	/* <HALTED> */
#define	MSQLERWAITINIT		59	/* <WAIT FOR M/xx INIT.. TO COMPLT> */
#define	MSQLERPRIMARY		60	/* <NOT PRIMARY VOLUME>	  */
#define	MSQLERCORRUPT		61	/* <CORRUPT VOLUME SET>	  */
#define	MSQLERMAPS		62	/* <INVALID MAP NUMBER>	  */
#define	MSQLERVOLSET1		63	/* <VOLUME SET ALREADY CREATED> */
#define	MSQLERFORMAT		64	/* <VOLUME IS NOT FORMATTED>	   */
#define	MSQLERMAPRSV		65	/* <CONFLICTING BLOCK NUMBERS>  */
#define	MSQLERCOLLATE		66	/* <COLLATION> */
#define	MSQLERZTRP		67	/* <ZTRAP> */
#define	MSQLERNODEV		68	/* <NODEV> */
#define	MSQLERLANG		69	/* <LANGUAGE MISMATCH> */
#define	MSQLERNOSYS		70	/* <NOSYS> */
#define	MSQLERGIMPL		71	/* <BADIMPLICIT> */
#define	MSQLERNGLOREF		72	/* <NETGLOREF> */
#define	MSQLERNFORMAT		73	/* <NETFORMAT> */
#define	MSQLERDSCON		74	/* <DSCON> */
#define	MSQLERNVERSION		75	/* <NETVERSION> */
#define MSQLERNJOBMAX		76	/* <NETJOBMAX> */
#define MSQLERNRETRY		77	/* <NETRETRY> */
#define	MSQLERNAMADD		78	/* <NAMEADD> */
#define	MSQLERNETLOCK		79	/* <NETLOCK> */
#define	MSQLERVALUE		80	/* <ILLEGAL VALUE> */
#define	MSQLERNUMFIL		81	/* <TOOMANYFILES> */
#define	MSQLERLIST		82	/* <LIST> */
#define	MSQLERNAMSP		83	/* <NAMESPACE> */
#define	MSQLERCMTFAIL		84	/* <COMMITFAIL> */
#define	MSQLERROLLFAIL		85	/* <ROLLFAIL> */
#define	MSQLERNSRVFAIL		86	/* <NETSRVFAIL> */
#define	MSQLERCOLMISMAT		87	/* <COLLATEMISMATCH> */
#define	MSQLERSLMSPAN		88	/* <SLMSPAN> */
#define	MSQLERWIDECHAR		89	/* <WIDE CHAR> */
#define	MSQLERLINELEV		90	/* <LINELEVEL> */
#define	MSQLERARGDUP		91	/* <DUPLICATEARG> */
#define	MSQLERGLODEF		92	/* <GLODEFCHANGE> */
#define	MSQLERCOLCHG		93	/* <COLLATECHANGE> */
#define	MSQLERRESTART		94	/* <NORESTART> */
#define	MSQLERROUDEF		95	/* <ROUDEFCHANGE> */
#define	MSQLERMNESPACE		96	/* <MNEMONICSPACE> */
#define	MSQLERTRANSLOCK		97	/* <INVTRANSLOCK> */
#define	MSQLERTRANSQUIT		98	/* <INVTRANSQUIT> */
#define	MSQLERECODETRAP		99	/* <ECODETRAP> */
#define	MSQLLOCKLOST		100	/* <LOCKLOST> */
#define	MSQLDATALOST		101	/* <DATALOST> */
#define	MSQLTRANSLOST		102	/* <TRANSLOST> */
#define	MSQLERZOLE		103	/* <ZOLE> */
#define	MSQLERLICENSE		104	/* <LICENSE LIMIT EXCEEDED> */
#define	MSQLERNOLICSERV		105 	/* <LICENSE SERVER UNAVAILABLE> */
#define	MSQLERCOLNSUP		106	/* <COLLATION NOT SUPPORTED> */
#define	MSQLERNLSTAB		107	/* <NLS TABLE> */
#define	MSQLERXLATE		108	/* <TRANSLATE> */
#define	MSQLERNULLVAL		109	/* <NULL VALUE> */
#define	MSQLERNOVAL		110	/* <MISSING VALUE> */
#define	MSQLERUNKTYPE		111	/* <UNKNOWN TYPE> */
#define	MSQLERVALRANGE		112	/* <VALUE OUT OF RANGE> */
#define	MSQLERRANGE		113	/* <RANGE> */
#define	MSQLERDOMAINSPACERETRY	114	/* <DOMAINSPACERETRY> */
#define	MSQLERCSMISMATCH	115	/* <CLIENT-SERVER MISMATCH> */
#define MSQLERSTRMISMATCH	116	/* <STRMISMATCH> */
#define MSQLERBADCLASS		117	/* <INVALID CLASS> */
#define MSQLERIOBJECT		118	/* <INTERNAL OBJECT ERROR> */
#define MSQLERBADOREF		119	/* <INVALID OREF> */
#define MSQLERNOMETHOD		120	/* <METHOD DOES NOT EXIST> */
#define MSQLERNOPROPERTY	121	/* <PROPERTY DOES NOT EXIST> */
#define MSQLERNOCLASS		122	/* <CLASS DOES NOT EXIST> */
#define MSQLERCLASSDESC		123	/* <CLASS DESCRIPTOR> */
#define MSQLERCLASSCOMP		124	/* <CLASS COMPILING> */
#define MSQLERCLASSRECOMP	125	/* <CLASS RECOMPILED> */
#define	MSQLERCLASSMEMORY	126	/* <INSUFFICIENT CLASS MEMORY> */
#define	MSQLERNUMCLASS		127	/* <TOO MANY USERS OF CLASS> */
#define	MSQLERNUMOREF		128	/* <TOO MANY OREFS> */
#define	MSQLERGETINVALID	129	/* <CANNOT GET THIS PROPERTY> */
#define	MSQLERSETINVALID	130	/* <CANNOT SET THIS PROPERTY> */
#define MSQLERREMCLASSRECOMP	131	/* <REMOTE CLASS RECOMPILED> */
#define MSQLERREMCLASSEDITED	132	/* <REMOTE CLASS EDITED> */
#define MSQLERCLASSEDITED	133	/* <CLASS EDITED> */
#define MSQLERCLASSLOAD		134	/* <CLASS TOO BIG TO LOAD> */
#define MSQLERCLASSSIZE		135	/* <CLASS TOO BIG TO SAVE> */
#define MSQLERCORRUPTOBJECT	136	/* <CORRUPT OBJECT> */
#define MSQLERDISCONNECT	137	/* <DISCONNECT> */
#define MSQLERNOCURRENTOBJECT	138	/* <NO CURRENT OBJECT> */
#define MSQLERZFHEAPSPACE	139	/* <OUT OF $ZF HEAP SPACE> */
#define MSQLERTOOMANYCLASSES	140	/* <TOO MANY CLASSES> */
#define MSQLERNETSTRMISMATCH    141 /* <NETWORK DATA UPDATE FAILED - STRMISMATCH> */
#define MSQLERNETSSTACK         142 /* <NETWORK DATA UPDATE FAILED - STRINGSTACK> */
#define MSQLERNETDIR            143 /* <NETWORK DATA UPDATE FAILED - DIRECTORY> */
#define	MSQLERNETDKHER          144 /* <NETWORK DATA UPDATE FAILED - DISKHARD> */
#define MSQLERNETDBDGD          145 /* <NETWORK DATA UPDATE FAILED - DATABASE> */
#define	MSQLERNETCLUSTER        146 /* <NETWORK DATA UPDATE FAILED - CLUSTERFAILED> */
#define	MSQLERNETCSMISMATCH     147 /* <NETWORK DATA .... - CLIENT-SERVER MISMATCH> */
#define MSQLERNETPROT           148 /* <NETWORK DATA UPDATE FAILED - PROTECT> */
#define	MSQLERNETDKSER          149 /* <NETWORK DATA UPDATE FAILED - BLOCKNUMBER> */
#define	MSQLERNETDKFUL          150 /* <NETWORK DATA UPDATE FAILED - DISKFULL> */
#define	MSQLERNETMXSTR          151 /* <NETWORK DATA UPDATE FAILED - MAXSTRING> */
#define	MSQLERNETSBSCR          152 /* <NETWORK DATA UPDATE FAILED - SUBSCRIPT> */
#define	MSQLERNETSYSTM          153 /* <NETWORK DATA UPDATE FAILED - SYSTEM> */
#define	MSQLERNETWIDECHAR       154 /* <NETWORK DATA UPDATE FAILED - WIDECHAR> */
#define MSQLERNETNVERSION       155 /* <NETWORK DATA UPDATE FAILED - NETVERSION> */
#define MSQLERNETNFORMAT        156 /* <NETWORK DATA UPDATE FAILED - NETFORMAT> */
#define	MSQLERNETNGLOREF        157 /* <NETWORK DATA UPDATE FAILED - NETGLOREF>  */
#define MSQLERBADBITSTR		158	/* <INVALID BIT STRING> */
#define MSQLERROUTINESIZE	159	/* <ROUTINE TOO BIG TO SAVE> */
#define MSQLERNOSOURCE		160	/* <NO SOURCE> */
#define MSQLERMALLOC		161	/* <STORE> - malloc failed */
#define MSQLEROREFNO		162	/* <INVALID TYPE> */
#define MSQLERNOOREF		163	/* <INVALID OREF> */
#define	MSQLERINVARG		164	/* <INVALID ARGUMENT> */
#define	MSQLERDYNLOAD		165	/* <DYNAMIC LIBRARY LOAD> */
#define MSQLERNOJVM		166	/* <Java VM not loaded> */
#define MSQLERPRIVPROP		167	/* <PRIVATE PROPERTY> */
#define MSQLERPRIVMETH		168	/* <PRIVATE METHOD> */
#define MSQLERJAVAEXCEPTION	169	/* <Java Exception> */
#define MSQLERNETLICENSE	170	/* <NETWORK UNLICENSED> */
#define MSQLERDDPJOBOVFL	171	/* <DDP JOB OVERFLOW> */
#define MSQLERMAX		171	/* Maximum error number */

#ifdef	__VMS
/* msqlsignal is not implemented on M/SQL for VMS systems as VMS versions
   of M/SQL do not use signal handling. It is provided here as returning
   MSQLSUCCESS so that programs compiled for VMS and for Unix do not need
   to be sensitive to this difference
*/
#define msqlsignal(x) MSQLSUCCESS
#endif

typedef const struct  CacheExportFuncTab {
        int (*CacheExecute)();		/* Traditional callin functions */
	int (*CacheExecuteW)();
	int (*CacheEval)();
	int (*CacheEvalW)();
	int (*CachePrompt)();
	int (*CachePromptW)();
	int (*CacheErrxlate)();
	int (*CacheErrxlateW)();
	int (*CacheError)();
	int (*CacheErrorW)();
	int (*CacheConvert)();
	int (*CacheType)();
	int (*CacheAbort)();
#ifndef	__VMS		/* doesn't exist on VMS. Macro CACHESIGNAL defined
			   on VMS as a noop.*/
	int (*CacheSignal)();
#endif

	int (*CachePushInt)();		/* Higher performance interface */
	int (*CachePushDbl)();
	int (*CachePushStr)();
	int (*CachePushStrW)();
	int (*CachePushRtn)();
	int (*CachePushRtnX)();
	int (*CachePushRtnW)();
	int (*CachePushRtnXW)();
	int (*CachePushFunc)();
	int (*CachePushFuncX)();
	int (*CachePushFuncW)();
	int (*CachePushFuncXW)();
	int (*CachePushGlobal)();
	int (*CachePushGlobalX)();
	int (*CachePushGlobalW)();
	int (*CachePushGlobalXW)();
	int (*CacheUnPop)();
	int (*CachePop)();
	int (*CachePopInt)();
	int (*CachePopDbl)();
	int (*CachePopStr)();
	int (*CachePopStrW)();
	int (*CacheSetVar)();
	int (*CacheGetVar)();
	int (*CacheDoRtn)();
	int (*CacheGlobalGet)();
	int (*CacheGlobalSet)();
	int (*CacheDoFun)();					/* LRS512 */
	int (*CachePushClassMethod)();				/* RAM968+ */
	int (*CachePushClassMethodW)();
	int (*CachePushMethod)();
	int (*CachePushMethodW)();
	int (*CacheInvokeClassMethod)();
	int (*CacheInvokeMethod)();
	int (*CachePushProperty)();
	int (*CachePushPropertyW)();
	int (*CacheGetProperty)();
	int (*CacheSetProperty)();				/* RAM968- */
	int (*CachePushOref)();					/* CDS188+ */
	int (*CachePopOref)();
	int (*CacheIncrementCountOref)();
	int (*CacheCloseOref)();				/* CDS188- */
	int (*CachePushCvtW)();					/* LRS582+ */
	int (*CachePopCvtW)();
	int (*CachePushList)();
	int (*CachePopList)();					/* LRS582- */
	int (*CachePushPtr)();					/* LRS609+ */
	int (*CachePopPtr)();					/* LRS609- */
	int (*CacheGlobalIncrement)();				/* JO1759 */
} CACHEEFT, *CACHEEFTP;

/* Interface Function Definition */
#define	CACHEEXECUTE(cmd)	(*cachefunctabp->CacheExecute)(cmd)
#define	CACHEEXECUTEW(cmd)	(*cachefunctabp->CacheExecuteW)(cmd)
#define	CACHEEVAL(expr)		(*cachefunctabp->CacheEval)(expr)
#define	CACHEEVALW(expr)	(*cachefunctabp->CacheEvalW)(expr)
#define	CACHEPROMPT(rbuf)	(*cachefunctabp->CachePrompt)(rbuf)
#define	CACHEPROMPTW(rbuf)	(*cachefunctabp->CachePromptW)(rbuf)
#define	CACHEERRXLATE(code,rbuf) (*cachefunctabp->CacheErrxlate)(code,rbuf)
#define	CACHEERRXLATEW(code,rbuf) (*cachefunctabp->CacheErrxlateW)(code,rbuf)
#define CACHEERROR(msg,src,offp) (*cachefunctabp->CacheError)(msg,src,offp)
#define CACHEERRORW(msg,src,offp) (*cachefunctabp->CacheErrorW)(msg,src,offp)
#define	CACHECONVERT(type,rbuf)	(*cachefunctabp->CacheConvert)(type,rbuf)
#define	CACHETYPE()		(*cachefunctabp->CacheType)()
#define	CACHEABORT(type)	(*cachefunctabp->CacheAbort)(type)
#ifdef	__VMS
#define	CACHESIGNAL(signal)	MSQLSUCCESS
#else
#define	CACHESIGNAL(signal)	(*cachefunctabp->CacheSignal)(signal)
#endif

#define	CACHEPUSHINT(num)	(*cachefunctabp->CachePushInt)(num)
#define	CACHEPUSHDBL(num)	(*cachefunctabp->CachePushDbl)(num)
#define	CACHEPUSHOREF(oref)	(*cachefunctabp->CachePushOref)(oref)
#define	CACHEPUSHSTR(len,ptr)	(*cachefunctabp->CachePushStr)(len,ptr)
#define	CACHEPUSHSTRW(len,ptr)  (*cachefunctabp->CachePushStrW)(len,ptr)
#define	CACHEPUSHRTN(rflag,tlen,tptr,nlen,nptr)		\
	(*cachefunctabp->CachePushRtn)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHRTNX(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
	(*cachefunctabp->CachePushRtnX)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHRTNW(rflag,tlen,tptr,nlen,nptr)		\
	(*cachefunctabp->CachePushRtnW)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHRTNXW(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
	(*cachefunctabp->CachePushRtnXW)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHFUNC(rflag,tlen,tptr,nlen,nptr)		\
	(*cachefunctabp->CachePushFunc)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHFUNCX(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
	(*cachefunctabp->CachePushFuncX)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHFUNCW(rflag,tlen,tptr,nlen,nptr)		\
	(*cachefunctabp->CachePushFuncW)(rflag,tlen,tptr,nlen,nptr)
#define	CACHEPUSHFUNCXW(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)		\
	(*cachefunctabp->CachePushFuncXW)(rflag,tlen,tptr,off,elen,eptr,nlen,nptr)
#define	CACHEPUSHGLOBAL(nlen,nptr)  (*cachefunctabp->CachePushGlobal)(nlen,nptr)
#define	CACHEPUSHGLOBALX(nlen,nptr,elen,eptr) \
	      (*cachefunctabp->CachePushGlobalX)(nlen,nptr,elen,eptr)
#define	CACHEPUSHGLOBALW(nlen,nptr)  (*cachefunctabp->CachePushGlobalW)(nlen,nptr)
#define	CACHEPUSHGLOBALXW(nlen,nptr,elen,eptr) \
	      (*cachefunctabp->CachePushGlobalXW)(nlen,nptr,elen,eptr)
#define CACHEUNPOP()		(*cachefunctabp->CacheUnPop)()
#define CACHEPOP(arg)		(*cachefunctabp->CachePop)(arg)
#define CACHEPOPINT(nump)	(*cachefunctabp->CachePopInt)(nump)
#define CACHEPOPDBL(nump)	(*cachefunctabp->CachePopDbl)(nump)
#define CACHEPOPOREF(orefp)	(*cachefunctabp->CachePopOref)(orefp)
#define CACHEPOPSTR(lenp,strp)	(*cachefunctabp->CachePopStr)(lenp,strp)
#define CACHEPOPSTRW(lenp,strp)	(*cachefunctabp->CachePopStrW)(lenp,strp)
#define CACHESETVAR(len,namep) 	(*cachefunctabp->CacheSetVar)(len,namep)
#define CACHEGETVAR(len,namep) 	(*cachefunctabp->CacheGetVar)(len,namep)
#define CACHEDORTN(flags,narg)	(*cachefunctabp->CacheDoRtn)(flags,narg)
#define CACHEEXTFUN(flags,narg)	(*cachefunctabp->CacheExtFun)(flags,narg)
#define CACHEGLOBALGET(narg,flag) (*cachefunctabp->CacheGlobalGet)(narg,flag)
#define CACHEGLOBALSET(narg)	(*cachefunctabp->CacheGlobalSet)(narg)
								/* JO1759+ */
#define CACHEGLOBALINCREMENT(narg) (*cachefunctabp->CacheGlobalIncrement)(narg)
								/* JO1759- */
#define CACHEDOFUN(flags,narg)	(*cachefunctabp->CacheDoFun)(flags,narg)
								/* RAM968+ */
#define	CACHEPUSHCLASSMETHOD(clen,cptr,mlen,mptr,flg)			\
		(*cachefunctabp->CachePushClassMethod)(clen,cptr,mlen,mptr,flg)
#define	CACHEPUSHCLASSMETHODW(clen,cptr,mlen,mptr,flg)			\
	       (*cachefunctabp->CachePushClassMethodW)(clen,cptr,mlen,mptr,flg)
#define	CACHEPUSHMETHOD(oref,mlen,mptr,flg)				\
		(*cachefunctabp->CachePushMethod)(oref,mlen,mptr,flg)
#define	CACHEPUSHMETHODW(oref,mlen,mptr,flg)				\
		(*cachefunctabp->CachePushMethodW)(oref,mlen,mptr,flg)
#define CACHEINVOKECLASSMETHOD(narg)					\
		(*cachefunctabp->CacheInvokeClassMethod)(narg)
#define CACHEINVOKEMETHOD(narg)	(*cachefunctabp->CacheInvokeMethod)(narg)
#define	CACHEPUSHPROPERTY(oref,plen,pptr)				\
		(*cachefunctabp->CachePushProperty)(oref,plen,pptr)
#define	CACHEPUSHPROPERTYW(oref,plen,pptr)				\
		(*cachefunctabp->CachePushPropertyW)(oref,plen,pptr)
#define CACHEGETPROPERTY()	(*cachefunctabp->CacheGetProperty)()
#define CACHESETPROPERTY()	(*cachefunctabp->CacheSetProperty)()
								/* RAM968- */
#define	CACHEINCREMENTCOUNTOREF(oref)	(*cachefunctabp->CacheIncrementCountOref)(oref)
#define	CACHECLOSEOREF(oref)	(*cachefunctabp->CacheCloseOref)(oref)
#define	CACHEPUSHCVTW(len,ptr)  (*cachefunctabp->CachePushCvtW)(len,ptr)
#define CACHEPOPCVTW(lenp,strp)	(*cachefunctabp->CachePopCvtW)(lenp,strp)
#define	CACHEPUSHLIST(len,ptr)  (*cachefunctabp->CachePushList)(len,ptr)
#define CACHEPOPLIST(lenp,strp)	(*cachefunctabp->CachePopList)(lenp,strp)
#define CACHEPUSHPTR(ptr)	(*cachefunctabp->CachePushPtr)(ptr)
#define CACHEPOPPTR(ptrp)	(*cachefunctabp->CachePopPtr)(ptrp)

#ifdef	__VMS							/* LRS666+ */
#pragma	__required_pointer_size	__restore
#endif								/* LRS666- */
