<?xml version='1.0' encoding='ISO-8859-1'?>

<!-- =============================================================== -->
<!--                                                                 -->
<!-- This stylesheet renders XEP documentation to HTML.              -->
<!--                                                                 -->
<!--     Author: Nikolai Grigoriev                                   -->
<!--                                                                 -->
<!--    (c) RenderX, 1999-2001                                       -->
<!--                                                                 -->
<!-- =============================================================== -->

<!DOCTYPE xsl:stylesheet [
  <!ENTITY % winansi SYSTEM "../winansi.entities">
  %winansi;
]>

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="html"
            version="1.0"
            indent="no"
            encoding="ISO-8859-1"/>

<!-- =============================================================== -->
<!-- Parameters and attribute sets                                   -->
<!-- =============================================================== -->

<xsl:param name="title-color">#0060A0</xsl:param>
<xsl:param name="href-color">#0040C0</xsl:param>

<!-- *************************************************************** -->
<!-- Top-level template: page layout                                 -->
<!-- *************************************************************** -->

<xsl:template match="book | article | /qandaset">
  <html>
    <head>
      <title><xsl:value-of select="title"/></title>
      <style type="text/css">
        a { color: <xsl:value-of select="$href-color"/>; }
        p { font-family: Helvetica; text-align: justify;
            margin-top: 6pt; margin-bottom: 6pt; margin-right: 6pt; }
        li { font-family: Helvetica; text-align: justify;
            margin-top: 3pt; margin-bottom: 3pt; margin-right: 6pt; }
        dt { font-family: Helvetica; text-align: left; font-weight: bold; }
h1, h2, h3 {font-family: Helvetica; text-align: left;
            color: <xsl:value-of select="$title-color"/>;
            margin-right: 6pt; }
    h4, h5 {font-family: Helvetica; text-align: left;
            margin-right: 6pt; }
    td, th { font-family: Helvetica; }
      code { font-family: Courier; font-size: 115%; }
      </style>
    </head>
    <body bgcolor="white" marginwidth="6" marginheight="6" leftmargin="6" topmargin="6">

      <h1><xsl:apply-templates select="title"/></h1>

      <xsl:call-template name="toc"/>       <!-- table of contents -->
      <xsl:apply-templates select="chapter | section | qandaset | qandadiv"/>  <!-- sections -->

      <hr/>
      <p style="font-size: smaller">
        &copy; RenderX 2000-2001. All rights reserved.
      </p>
    </body>
  </html>

</xsl:template>


<!-- *************************************************************** -->
<!-- Regular Divisions                                                -->
<!-- *************************************************************** -->

<xsl:template match="section | chapter | */qandaset | qandadiv">
  <xsl:variable name="id" select="generate-id()"/>
  <div style="margin-top: 18pt;">
    <a name="{$id}"/>
    <xsl:apply-templates select="title"/>
    <xsl:apply-templates select="*[not(self::title) and not(self::section)]"/>
    <xsl:if test="descendant::footnote">
      <table>
        <xsl:apply-templates select="descendant::footnote[generate-id(ancestor::*[self::section or self::chapter][1]) = $id]" mode="footnote-body"/>
      </table>
    </xsl:if>
    <xsl:apply-templates select="section"/>
  </div>
</xsl:template>


<!-- *************************************************************** -->
<!-- Division title                                                  -->
<!-- *************************************************************** -->

<xsl:template match="/qandaset/title" priority="2">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="chapter/title |
                     section/title[count(ancestor::section | ancestor::chapter) = 1] |
                     qandaset/title[count(ancestor::section | ancestor::chapter) = 0] |
                     qandadiv/title[count(ancestor::section | ancestor::chapter | ancestor::qandaset | ancestor::qandadiv) = 1]">
  <h2><xsl:apply-templates select="." mode="numbered-text"/></h2>
</xsl:template>

<xsl:template match="section/title[count(ancestor::section | ancestor::chapter) = 2] |
                     qandaset/title[count(ancestor::section | ancestor::chapter) = 1] |
                     qandadiv/title[count(ancestor::section | ancestor::chapter | ancestor::qandaset | ancestor::qandadiv) = 2]">
  <h3><xsl:apply-templates select="." mode="numbered-text"/></h3>
</xsl:template>

<xsl:template match="section/title[count(ancestor::section | ancestor::chapter) = 3] |
                     qandaset/title[count(ancestor::section | ancestor::chapter) = 2] |
                     qandadiv/title[count(ancestor::section | ancestor::chapter | ancestor::qandaset | ancestor::qandadiv) = 3]">
  <h4><xsl:apply-templates select="." mode="numbered-text"/></h4>
</xsl:template>

<xsl:template match="section/title | qandaset/title | qandadiv/title" priority="-1">
  <p><b><xsl:apply-templates select="." mode="numbered-text"/></b></p>
</xsl:template>

<xsl:template match="section/title | chapter/title | qandaset/title" mode="numbered-text">
  <xsl:number format="1.1.1.1.1.1." count="section | chapter | qandaset" level="multiple"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates select="." mode="plain-text"/>
</xsl:template>

<xsl:template match="qandadiv/title" mode="numbered-text">
  <xsl:number format="A.1.1.1.1.1." count="qandadiv" level="multiple"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates select="." mode="plain-text"/>
</xsl:template>

<xsl:template match="table/title" mode="numbered-text">
  <xsl:text>Table </xsl:text>
  <xsl:number format="1." count="table"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates select="." mode="plain-text"/>
</xsl:template>

<xsl:template match="figure/title" mode="numbered-text">
  <xsl:text>Figure </xsl:text>
  <xsl:number format="1." count="figure"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates select="." mode="plain-text"/>
</xsl:template>

<xsl:template match="example/title" mode="numbered-text">
  <xsl:text>Example </xsl:text>
  <xsl:number format="1." count="example"/>
  <xsl:text> </xsl:text>
  <xsl:apply-templates select="." mode="plain-text"/>
</xsl:template>

<xsl:template match="title" mode="plain-text">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="*" mode="toc" priority="-1"/>


<xsl:template match="chapter | qandaset[../chapter]" mode="toc" priority="1">
  <div style="margin-top: 6pt; margin-bottom: 2pt; font-family: Helvetica;">
    <a href="#{generate-id(.)}"><b><xsl:apply-templates select="title" mode="numbered-text"/></b></a>
    <xsl:apply-templates select="section" mode="toc"/>
  </div>
</xsl:template>

<xsl:template match="section | qandaset | qandadiv" mode="toc">
  <div style="margin-top: 2pt; margin-bottom: 2pt; margin-left: 30pt; font-family: Helvetica;">
    <a href="#{generate-id(.)}"><xsl:apply-templates select="title" mode="numbered-text"/></a>
    <xsl:apply-templates select="section" mode="toc"/>
  </div>
</xsl:template>


<!-- *************************************************************** -->
<!-- Standard elements                                               -->
<!-- *************************************************************** -->

<!-- Paragraph -->

<xsl:template match="para">
  <p><xsl:apply-templates/></p>
</xsl:template>

<!-- Cited Title -->

<xsl:template match="citetitle">
  <i>"<xsl:apply-templates/>"</i>
</xsl:template>

<!-- Emphasis -->

<xsl:template match="emphasis[not(@role)]">
  <i><xsl:apply-templates/></i>
</xsl:template>

<xsl:template match="emphasis[@role]">
  <xsl:variable name="italics-applied">
    <xsl:choose>
      <xsl:when test="contains(@role, 'italic')"><i><xsl:apply-templates/></i></xsl:when>
      <xsl:otherwise><xsl:apply-templates/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="bold-applied">
    <xsl:choose>
      <xsl:when test="contains(@role, 'bold')"><b><xsl:copy-of select="$italics-applied"/></b></xsl:when>
      <xsl:otherwise><xsl:copy-of select="$italics-applied"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="contains(@role, 'underline')"><b><xsl:copy-of select="$bold-applied"/></b></xsl:when>
    <xsl:otherwise><xsl:copy-of select="$bold-applied"/></xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!-- Code excerpts -->

<xsl:template match="literal">
  <code><xsl:apply-templates/></code>
</xsl:template>

<xsl:template match="function | classname | filename | property">
  <b><xsl:apply-templates/></b>
</xsl:template>

<xsl:template match="errorname">
  <i><xsl:apply-templates/></i>
</xsl:template>

<xsl:template match="command">
  <b><code><xsl:apply-templates/></code></b>
</xsl:template>

<xsl:template match="sgmltag">
  <b>&lt;<xsl:apply-templates/>&gt;</b>
</xsl:template>


<!-- Hyperlinks -->

<xsl:template match="ulink">
  <a href="{@url}"><xsl:apply-templates/></a>
</xsl:template>

<!-- *************************************************************** -->
<!-- Notes                                                           -->
<!-- *************************************************************** -->

<!-- Simple note -->

<xsl:template match="note">
  <table cellspacing="3" border="0">
    <tr valign="top">
      <td><p><b>Note:</b></p></td>
      <td><xsl:apply-templates/></td>
    </tr>
  </table>
</xsl:template>

<!-- =============================================================== -->
<!-- Lists: ordered and unordered                                    -->
<!-- =============================================================== -->

<xsl:template match="itemizedlist">
  <ul><xsl:apply-templates/></ul>
</xsl:template>

<xsl:template match="orderedlist">
  <ol><xsl:apply-templates/></ol>
</xsl:template>

<xsl:template match="itemizedlist/listitem | orderedlist/listitem">
  <li><xsl:apply-templates/></li>
</xsl:template>

<xsl:template match="variablelist">
  <dl><xsl:apply-templates/></dl>
</xsl:template>

<xsl:template match="varlistentry">
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="varlistentry/term">
  <dt><xsl:apply-templates/></dt>
</xsl:template>

<xsl:template match="varlistentry/listitem">
  <dd><xsl:apply-templates/></dd>
</xsl:template>


<!-- *************************************************************** -->
<!-- Commented code examples                                         -->
<!-- *************************************************************** -->

<xsl:template match="example | informalexample">
  <div style="border: 1px solid gray;
              margin-top: 6pt;
              margin-bottom: 6pt;
              padding: 6pt">
    <xsl:apply-templates select="programlisting"/>
    <xsl:apply-templates select="calloutlist"/>
  </div>
</xsl:template>

<xsl:template match="example/programlisting | informalexample/programlisting">
  <pre><xsl:apply-templates/></pre>
</xsl:template>

<xsl:template match="programlisting" priority="-1">
  <div style="border: 1px solid gray;
              margin-top: 6pt;
              margin-bottom: 6pt;
              padding: 6pt">
    <pre><xsl:apply-templates/></pre>
  </div>
</xsl:template>

<xsl:template match="synopsis">
  <pre><xsl:apply-templates/></pre>
</xsl:template>

<xsl:template match="calloutlist">
  <table><xsl:apply-templates/></table>
</xsl:template>

<xsl:template match="callout">
  <tr>
    <td width="30" valign="top">
      <xsl:variable name="ids" select="concat (' ', @arearefs, ' ')"/>
      <p style="font-weight:bold">
        <xsl:apply-templates mode="list-mode"
              select="ancestor::informalexample/programlisting/co
                      [contains ($ids, concat (' ', @id, ' '))]"/>
      </p>
    </td>
    <td valign="top" style="text-align: left; font-family: Courier;"><xsl:apply-templates/></td>
  </tr>
</xsl:template>


<xsl:template match="co" mode="list-mode">
  <a name="{generate-id()}"/>
  <xsl:call-template name="callout-number"/>
</xsl:template>

<xsl:template match="co">
  <a href="#{generate-id()}"><xsl:call-template name="callout-number"/></a>
</xsl:template>

<xsl:template name="callout-number">
  <xsl:variable name="pos"><xsl:number
         count="co" from="programlisting"
         format="1" level="any"/></xsl:variable>
  <span style="text-decoration:none; background-color:#E0E0E0;">
    <xsl:text>[</xsl:text>
    <xsl:value-of select="$pos"/>
    <xsl:text>]</xsl:text>
  </span>
</xsl:template>

<!-- *************************************************************** -->
<!-- Tables                                                          -->
<!-- *************************************************************** -->

<xsl:template match="table | informaltable">
  <xsl:apply-templates select="title"/>
  <xsl:apply-templates select="*[not(self::title)]"/>
</xsl:template>

<xsl:template match="table/title">
  <h3><xsl:apply-templates select="." mode="numbered-text"/></h3>
</xsl:template>

<xsl:template match="tgroup">
  <table border="1" cellspacing="3pt" cellpadding="0">
    <xsl:apply-templates select="@align"/>
    <xsl:apply-templates select="colspec"/>
    <xsl:apply-templates select="thead"/>
    <xsl:apply-templates select="tfoot"/>
    <xsl:apply-templates select="tbody"/>
  </table>
</xsl:template>

<xsl:template match="colspec">
  <col>
    <xsl:apply-templates select="parent::*/@align"/>
    <xsl:apply-templates select="@colwidth|@align"/>
  </col>
</xsl:template>

<xsl:template match="thead">
  <thead bgcolor="silver" style="font-size: 110%; font-weight: bold">
    <xsl:apply-templates select="@valign"/>
    <xsl:apply-templates/>
  </thead>
</xsl:template>

<xsl:template match="tfoot">
  <tfoot background-color="silver" font-size="110%" font-weight="bold">
    <xsl:apply-templates select="@valign"/>
    <xsl:apply-templates/>
  </tfoot>
</xsl:template>

<xsl:template match="tbody">
  <tbody>
    <xsl:apply-templates select="@valign"/>
    <xsl:apply-templates/>
  </tbody>
</xsl:template>

<xsl:template match="row">
  <tr>
    <xsl:apply-templates select="@valign"/>
    <xsl:apply-templates/>
  </tr>
</xsl:template>

<xsl:template match="entry">
  <td style="padding: 3pt 6pt">
    <xsl:apply-templates select="@align|@valign|@morerows"/>

    <xsl:choose>
      <xsl:when test="@namest">
        <xsl:variable name="start-column">
          <xsl:call-template name="col-number-by-name">
            <xsl:with-param name="colname" select="@namest"/>
          </xsl:call-template>
        </xsl:variable>
        <!-- <xsl:attribute name="colnum"><xsl:value-of select="$start-column"/></xsl:attribute> -->

        <xsl:if test="@nameend">
          <xsl:variable name="end-column">
            <xsl:call-template name="col-number-by-name">
              <xsl:with-param name="colname" select="@nameend"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:attribute name="colspan"><xsl:value-of select="$end-column - $start-column + 1"/></xsl:attribute>
        </xsl:if>
      </xsl:when>
      <xsl:when test="@colname">
        <xsl:variable name="start-column">
          <xsl:call-template name="col-number-by-name">
            <xsl:with-param name="colname" select="@colname"/>
          </xsl:call-template>
        </xsl:variable>
        <!-- <xsl:attribute name="colnum"><xsl:value-of select="$start-column"/></xsl:attribute> -->
      </xsl:when>
    </xsl:choose>
    <xsl:apply-templates/>
  </td>
</xsl:template>

<xsl:template name="col-number-by-name">
  <xsl:param name="colname"/>
  <xsl:call-template name="col-number-by-name-recursive">
    <xsl:with-param name="csp" select="ancestor::tgroup[1]/colspec[@colname=$colname]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="col-number-by-name-recursive">
  <xsl:param name="csp"/>
  <xsl:choose>
    <xsl:when test="not($csp)">0</xsl:when>
    <xsl:otherwise>
      <xsl:variable name="prevnum">
        <xsl:call-template name="col-number-by-name-recursive">
          <xsl:with-param name="csp" select="$csp/preceding-sibling::colspec"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="$prevnum + 1"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="@colwidth">
  <xsl:attribute name="width"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="@align">
  <xsl:attribute name="align"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="@valign">
  <xsl:attribute name="valign"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="@morerows">
  <xsl:attribute name="rowspan"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>


<!-- =============================================================== -->
<!-- Footnote and FootnoteRef                                        -->
<!-- =============================================================== -->

<xsl:template match="footnote">
  <a href="#{generate-id()}"><xsl:apply-templates select="." mode="footnote-number"/></a>
</xsl:template>

<xsl:template match="footnote" mode="footnote-body">
  <tr valign="top">
    <td width="20">
      <p><b><xsl:apply-templates select="." mode="footnote-number"/></b></p>
    </td>
    <td>
      <a name="{generate-id()}"/>
      <xsl:apply-templates/>
    </td>
  </tr>
</xsl:template>

<xsl:template match="footnoteref">
  <xsl:variable name="linkend" select="@linkend"/>
  <xsl:apply-templates select="/descendant::footnote[@id=$linkend]"/>
</xsl:template>

<xsl:template match="footnote" mode="footnote-number">
  <xsl:text>[</xsl:text>
  <xsl:number count="footnote" level="any" format="1"/>
  <xsl:text>]</xsl:text>
</xsl:template>


<!-- *************************************************************** -->
<!-- Questions and Answers                                           -->
<!-- *************************************************************** -->


<xsl:template match="qandaentry">
  <div style="font-weight: bold; background-color: #E0E0E0; padding: 3pt; margin-top: 6pt">
    <xsl:apply-templates select="question"/>
  </div>
  <div style="margin-left: 24pt">
    <xsl:apply-templates select="answer"/>
  </div>
</xsl:template>


<!-- *************************************************************** -->
<!-- Table of Contents                                               -->
<!-- *************************************************************** -->

<xsl:template name="toc">
  <div style="margin-top: 18pt;">
    <h2>Table of Contents</h2>
    <xsl:apply-templates select="chapter | section | qandaset | qandadiv" mode="toc"/>
  </div>
</xsl:template>

<!-- *************************************************************** -->
<!-- Indexes: not yet implemented                                    -->
<!-- *************************************************************** -->
<xsl:template match="indexterm">
  <a name="{generate-id()}"/>
</xsl:template>

</xsl:stylesheet>

