<?xml version="1.0"?>
<!-- ============================================================ -->
<!--                                                              -->
<!-- This stylesheet renders XML documentation to PDF via XEP.    -->
<!-- It supports only those DocBook XML elements required to      -->
<!-- produce InterSystems product documentation.                  -->
<!--                                                              -->
<!--     Author: Susan Korgen                                     -->
<!--     copyright 2003-2004 InterSystems Corporation             -->
<!--                                                              -->
<!-- This InterSystems stylesheet was created through incremental -->
<!-- editing and testing of a sample XSL file provided by the     -->
<!-- makers of the XEP tool. Although this process replaced the   -->
<!-- original file over time, we acknowledge this source:         -->
<!--                                                              -->
<!--     Author: Nikolai Grigoriev                                -->
<!--     copyright RenderX, 2000-2003                             -->
<!--                                                              -->
<!-- ============================================================ -->
<!DOCTYPE xsl:stylesheet
          [
          <!ENTITY lowercase "'abcdefghijklmnopqrstuvwxyz'">
          <!ENTITY uppercase "'ABCDEFGHIJKLMNOPQRSTUVWXYZ'">
          <!ENTITY primary 'concat(primary/@sortas, primary[not(@sortas)])'>
          <!ENTITY secondary 'concat(secondary/@sortas, secondary[not(@sortas)])'>
          <!ENTITY tertiary 'concat(tertiary/@sortas, tertiary[not(@sortas)])'>
          
          <!-- ================================================================= -->
          <!-- GENERAL DESCRIPTION OF THE SECTION ENTITIES -->
          <!-- The following entity declarations have the effect of suppressing -->
          <!-- any page number "hits" after the first one, per container listed. -->
          <!-- Thus, if ancestor-or-self::chapter appears below -->
          <!-- (it does not, at present) -->
          <!-- and none of the containers that can be inside a chapter appear -->
          <!-- (for example para or figure) -->
          <!-- (para and figure do appear below, at present) -->
          <!-- then the additional page number references for a term that -->
          <!-- occurs multiple times within the container (that is, the chapter) -->
          <!-- are suppressed by the exisitence of this entity declaration and -->
          <!-- by the way this declaration interacts with the rest of the file. -->
          <!-- -->
          <!-- CURRENT EFFECTS OF THE SECTION ENTITIES -->
          <!-- The current design of this declaration is to provide the -->
          <!-- smallest possible granularity for this restriction. -->
          <!-- The items listed here are very small: para, figure, entry, -->
          <!-- refpurpose, table. This is the same as the list of containers in -->
          <!-- which our DocBook application permits us to put <indexterm>s. -->
          <!-- There are known bad effects in HTML if we stray from this list. -->
          <!-- -->
          <!-- The effect of the current design is intentionally not restrictive:-->
          <!-- If an index term appears multiple times within the same -->
          <!-- para, its first page number reference is output and additional -->
          <!-- page number references within that same para are suppressed. -->
          <!-- -->
          <!-- The reason for keeping the granularity very small is to -->
          <!-- ensure that writers get to see the full effects of such -->
          <!-- index terms as they put into the XML. If "hits" are -->
          <!-- suppressed from the index output, writers can't correct them. -->
          <!-- On the other hand, there is a practical limit: -->
          <!-- It's reasonable to avoid sending the reader to the same para -->
          <!-- for the same term more than once. -->
          <!-- -->
          <!-- FUTURE EFFECTS OF THE SECTION ENTITY -->
          <!-- It would be easy to make this more restrictive by deleting -->
          <!-- the smaller-granularity items such as para and replacing -->
          <!-- them with items such as sect1, refsect2, or even chapter. -->
          <!-- -->
          <!-- In general, this code is too tightly interwoven with the rest -->
          <!-- of the file to just rip it out, so customizing the granularity -->
          <!-- level seems to be the least of evils for now. -->
          <!-- ================================================================= -->
          <!ENTITY section '(ancestor-or-self::para |ancestor-or-self::refpurpose |ancestor-or-self::figure |ancestor-or-self::table |ancestor-or-self::entry)[1]'>
          <!ENTITY section.id 'generate-id(&section;)'>
          <!ENTITY sep '" "'>
          ]>

<!-- =============================================================== -->
<!-- The stylesheet begins                                           -->
<!-- =============================================================== -->

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format">
   <xsl:key name="letter"
            match="indexterm"
            use="translate(substring(&primary;, 1, 1),&lowercase;,&uppercase;)"/>
   <xsl:key name="primary"
            match="indexterm"
            use="&primary;"/>
   <xsl:key name="secondary"
            match="indexterm"
            use="concat(&primary;, &sep;, &secondary;)"/>
   <xsl:key name="tertiary"
            match="indexterm"
            use="concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;)"/>
   <xsl:key name="primary-section"
            match="indexterm[not(secondary) and not(see) and not(seealso)]"
            use="concat(&primary;, &sep;, &section.id;)"/>
   <xsl:key name="secondary-section"
            match="indexterm[not(tertiary) and not(see) and not(seealso)]"
            use="concat(&primary;, &sep;, &secondary;, &sep;, &section.id;)"/>
   <xsl:key name="tertiary-section"
            match="indexterm[not(see) and not(seealso)]"
            use="concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;, &sep;, &section.id;)"/>
   <xsl:key name="see-also"
            match="indexterm[seealso]"
            use="concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;, &sep;, seealso)"/>
   <xsl:key name="see"
            match="indexterm[see]"
            use="concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;, &sep;, see)"/>
   <xsl:key name="sections"
            match="*[@id]"
            use="@id"/>

   <!-- =============================================================== -->
   <!-- Attribute sets: index page numbers                              -->
   <!-- =============================================================== -->

   <xsl:attribute-set name="index-page-numbers">
      <xsl:attribute name="color">
         <xsl:value-of select="$href-color"/>
      </xsl:attribute>
   </xsl:attribute-set>

   <xsl:template name="generate-index">
      <fo:block text-align="left"
                span="all"
                space-after="6pt">
         <fo:inline xsl:use-attribute-sets="article-title-attrs">Index</fo:inline>
      </fo:block>
      <fo:block padding="2pt">
         <xsl:variable name="terms"
                       select="//indexterm[count(.|key('letter', translate(substring(&primary;, 1, 1),&lowercase;,&uppercase;))[1]) = 1]"/>
         <xsl:variable name="alphabetical"
                       select="$terms[contains(concat(&lowercase;, &uppercase;), substring(&primary;, 1, 1))]"/>
         <xsl:variable name="others"
                       select="$terms[not(contains(concat(&lowercase;, &uppercase;), substring(&primary;, 1, 1)))]"/>
         <xsl:if test="$others">
            <fo:block text-align="left"
                      space-before="9pt"
                      space-after="6pt"
                      keep-with-next.within-column="always">
               <fo:inline font-family="sans-serif"
                          font-size="12pt"
                          font-weight="bold">Symbols</fo:inline>
            </fo:block>
            <xsl:apply-templates select="$others[count(.|key('primary', &primary;)[1]) = 1]"
                                 mode="index-div">
               <xsl:sort select="&primary;"/>
            </xsl:apply-templates>
         </xsl:if>
         <xsl:apply-templates select="$alphabetical[count(.|key('letter', translate(substring(&primary;, 1, 1),&lowercase;,&uppercase;))[1]) = 1]"
                              mode="index-div">
            <xsl:sort select="&primary;"/>
         </xsl:apply-templates>
      </fo:block>
   </xsl:template>

   <xsl:template match="indexterm[primary]"
                 mode="index-div">
      <xsl:variable name="key"
                    select="translate(substring(&primary;, 1, 1),&lowercase;,&uppercase;)"/>
      <xsl:if test="contains(concat(&lowercase;, &uppercase;), $key)">
         <fo:block text-align="left"
                   space-before="9pt"
                   space-after="6pt"
                   keep-with-next.within-column="always">
            <fo:inline font-family="sans-serif"
                       font-size="12pt"
                       font-weight="bold">
               <xsl:value-of select="translate($key, &lowercase;, &uppercase;)"/>
            </fo:inline>
         </fo:block>
      </xsl:if>
      <xsl:apply-templates select="key('letter', $key)[count(.|key('primary', &primary;)[1]) = 1]"
                           mode="index-primary">
         <xsl:sort select="&primary;"/>
      </xsl:apply-templates>
   </xsl:template>

   <!-- =============================================== -->
   <!-- The template for primary index terms            -->
   <!-- =============================================== -->

   <xsl:template match="indexterm"
                 mode="index-primary">

      <!-- ========================================================== -->
      <!-- Generate a concatenated string of page numbers (I think)?  -->
      <!-- ========================================================== -->

      <xsl:variable name="key"
                    select="&primary;"/>
      <xsl:variable name="refs"
                    select="key('primary', $key)"/>
      <xsl:variable name="all-primary-sections"
                    select="$refs[generate-id() = generate-id(key('primary-section',concat($key, &sep;, &section.id;))[1])]"/>
      <xsl:variable name="page-number-citations">
         <xsl:for-each select="$all-primary-sections">
            <xsl:apply-templates select="."
                                 mode="reference">
               <xsl:with-param name="is-first-section"
                               select="generate-id() = generate-id($all-primary-sections[1])"/>
            </xsl:apply-templates>
         </xsl:for-each>
      </xsl:variable>

      <!-- =============================================== -->
      <!-- Output the primary entry                        -->
      <!-- =============================================== -->

      <fo:block font="10pt/1.1 serif"
                text-align="start">
         <xsl:value-of select="primary"/>
         <xsl:if test="count($all-primary-sections)!=0">,</xsl:if>

         <!-- =============================================== -->
         <!-- Output page numbers, if any                     -->
         <!-- =============================================== -->

         <fo:leader leader-pattern="space"
                    leader-length="6pt"/>
         <xsl:copy-of select="$page-number-citations"/>

         <!-- =============================================== -->
         <!-- Output secondary references, if any             -->
         <!-- =============================================== -->

         <xsl:if test="$refs/secondary">
            <fo:block margin-left="10pt"
                      keep-with-previous.within-column="always">
               <xsl:apply-templates select="$refs[secondary and count(.|key('secondary', concat($key, &sep;, &secondary;))[1]) = 1]"
                                    mode="index-secondary">
                  <xsl:sort select="&secondary;"/>
               </xsl:apply-templates>
            </fo:block>
         </xsl:if>

         <!-- =============================================== -->
         <!-- Output "see" and "see also" references, if any  -->
         <!-- =============================================== -->

         <xsl:if test="$refs[not(secondary)]/*[self::see or self::seealso]">
            <xsl:apply-templates select="$refs[generate-id() = generate-id(key('see', concat(&primary;, &sep;, &sep;, &sep;, see))[1])]"
                                 mode="index-see">
               <xsl:sort select="see"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="$refs[generate-id() = generate-id(key('see-also', concat(&primary;, &sep;, &sep;, &sep;, seealso))[1])]"
                                 mode="index-seealso">
               <xsl:sort select="seealso"/>
            </xsl:apply-templates>
         </xsl:if>

         <!-- =============================================== -->
         <!-- The primary entry is complete                   -->
         <!-- =============================================== -->

      </fo:block>
   </xsl:template>

   <!-- =============================================== -->
   <!-- The template for secondary index terms          -->
   <!-- =============================================== -->

   <xsl:template match="indexterm"
                 mode="index-secondary">

      <!-- ========================================================== -->
      <!-- Generate a concatenated string of page numbers (I think)?  -->
      <!-- ========================================================== -->

      <xsl:variable name="key"
                    select="concat(&primary;, &sep;, &secondary;)"/>
      <xsl:variable name="refs"
                    select="key('secondary', $key)"/>
      <xsl:variable name="all-secondary-sections"
                    select="$refs[generate-id() = generate-id(key('secondary-section', concat($key, &sep;, &section.id;))[1])]"/>
      <xsl:variable name="page-number-citations">
         <xsl:for-each select="$all-secondary-sections">
            <xsl:apply-templates select="."
                                 mode="reference">
               <xsl:with-param name="is-first-section"
                               select="generate-id() = generate-id($all-secondary-sections[1])"/>
            </xsl:apply-templates>
         </xsl:for-each>
      </xsl:variable>

      <!-- =============================================== -->
      <!-- Output the secondary entry                      -->
      <!-- =============================================== -->

      <fo:block>
         <xsl:value-of select="secondary"/>
         <xsl:if test="count($all-secondary-sections)!=0">,</xsl:if>

         <!-- =============================================== -->
         <!-- Output page numbers, if any                     -->
         <!-- =============================================== -->

         <fo:leader leader-pattern="space"
                    leader-length="6pt"/>
         <xsl:copy-of select="$page-number-citations"/>

         <!-- =============================================== -->
         <!-- Output tertiary references, if any              -->
         <!-- =============================================== -->

         <xsl:if test="$refs/tertiary">
            <fo:block margin-left="10pt"
                      keep-with-previous.within-column="always">
               <xsl:apply-templates select="$refs[tertiary and count(.|key('tertiary', concat($key, &sep;, &tertiary;))[1]) = 1]"
                                    mode="index-tertiary">
                  <xsl:sort select="&tertiary;"/>
               </xsl:apply-templates>
            </fo:block>
         </xsl:if>

         <!-- =============================================== -->
         <!-- Output "see" and "see also" references, if any  -->
         <!-- =============================================== -->

         <xsl:if test="$refs[not(tertiary)]/*[self::see or self::seealso]">
            <xsl:apply-templates select="$refs[generate-id() = generate-id(key('see', concat(&primary;, &sep;, &secondary;, &sep;, &sep;, see))[1])]"
                                 mode="index-see">
               <xsl:sort select="see"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="$refs[generate-id() = generate-id(key('see-also', concat(&primary;, &sep;, &secondary;, &sep;, &sep;, seealso))[1])]"
                                 mode="index-seealso">
               <xsl:sort select="seealso"/>
            </xsl:apply-templates>
         </xsl:if>

         <!-- =============================================== -->
         <!-- The secondary entry is complete                 -->
         <!-- =============================================== -->

      </fo:block>
   </xsl:template>

   <!-- =============================================== -->
   <!-- The template for tertiary index terms           -->
   <!-- =============================================== -->

   <xsl:template match="indexterm"
                 mode="index-tertiary">

      <!-- ========================================================== -->
      <!-- Generate a concatenated string of page numbers (I think)?  -->
      <!-- ========================================================== -->

      <xsl:variable name="key"
                    select="concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;)"/>
      <xsl:variable name="refs"
                    select="key('tertiary', $key)"/>
      <xsl:variable name="all-tertiary-sections"
                    select="$refs[generate-id() = generate-id(key('tertiary-section', concat($key, &sep;, &section.id;))[1])]"/>
      <xsl:variable name="page-number-citations">
         <xsl:for-each select="$all-tertiary-sections">
            <xsl:apply-templates select="."
                                 mode="reference">
               <xsl:with-param name="is-first-section"
                               select="generate-id() = generate-id($all-tertiary-sections[1])"/>
            </xsl:apply-templates>
         </xsl:for-each>
      </xsl:variable>

      <!-- =============================================== -->
      <!-- Output the tertiary entry                       -->
      <!-- =============================================== -->

      <fo:block keep-with-previous.within-column="always">
         <xsl:value-of select="tertiary"/>
         <xsl:if test="count($all-tertiary-sections)!=0">,</xsl:if>

         <!-- =============================================== -->
         <!-- Output page numbers, if any                     -->
         <!-- =============================================== -->

         <fo:leader leader-pattern="space"
                    leader-length="6pt"/>
         <xsl:copy-of select="$page-number-citations"/>

         <!-- =============================================== -->
         <!-- Output "see" and "see also" references, if any  -->
         <!-- =============================================== -->

         <xsl:variable name="see"
                       select="$refs/see | $refs/seealso"/>
         <xsl:if test="$see">
            <xsl:apply-templates select="$refs[generate-id() = generate-id(key('see', concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;, &sep;, see))[1])]"
                                 mode="index-see">
               <xsl:sort select="see"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="$refs[generate-id() = generate-id(key('see-also', concat(&primary;, &sep;, &secondary;, &sep;, &tertiary;, &sep;, seealso))[1])]"
                                 mode="index-seealso">
               <xsl:sort select="seealso"/>
            </xsl:apply-templates>
         </xsl:if>

         <!-- =============================================== -->
         <!-- The tertiary entry is complete                  -->
         <!-- =============================================== -->

      </fo:block>
   </xsl:template>

   <!-- =============================================== -->
   <!-- The template for multiple page references?      -->
   <!-- If so, it is not working.                       -->
   <!-- =============================================== -->

   <xsl:template match="indexterm"
                 mode="reference">
      <xsl:param name="is-first-section"/>
      <xsl:if test="not($is-first-section)">, </xsl:if>
      <xsl:choose>
         <xsl:when test="@zone and string(@zone)">
            <xsl:call-template name="reference">
               <xsl:with-param name="zones"
                               select="normalize-space(@zone)"/>
            </xsl:call-template>
         </xsl:when>
         <xsl:otherwise>
            <xsl:variable name="id">
               <xsl:call-template name="object.id"/>
            </xsl:variable>
            <fo:basic-link internal-destination="{$id}">
               <xsl:text/>
               <fo:page-number-citation xsl:use-attribute-sets="index-page-numbers"
                                        ref-id="{$id}">
                  <xsl:if test="@significance='preferred'">
                     <xsl:attribute name="font-weight">bold</xsl:attribute>
                  </xsl:if>
               </fo:page-number-citation>
               <xsl:text/>
            </fo:basic-link>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template name="reference">
      <xsl:param name="zones"/>
      <xsl:choose>
         <xsl:when test="contains($zones, ' ')">
            <xsl:variable name="zone"
                          select="substring-before($zones, ' ')"/>
            <xsl:variable name="target"
                          select="key('sections', $zone)"/>
            <xsl:variable name="id">
               <xsl:call-template name="object.id">
                  <xsl:with-param name="object"
                                  select="$target[1]"/>
               </xsl:call-template>
            </xsl:variable>
            <fo:basic-link internal-destination="{$id}">
               <xsl:text/>
               <fo:page-number-citation xsl:use-attribute-sets="index-page-numbers"
                                        ref-id="{$id}"/>
               <xsl:text/>
            </fo:basic-link>
            <xsl:text>, </xsl:text>
            <xsl:call-template name="reference">
               <xsl:with-param name="zones"
                               select="substring-after($zones, ' ')"/>
            </xsl:call-template>
         </xsl:when>
         <xsl:otherwise>
            <xsl:variable name="zone"
                          select="$zones"/>
            <xsl:variable name="target"
                          select="key('sections', $zone)"/>
            <xsl:variable name="id">
               <xsl:call-template name="object.id">
                  <xsl:with-param name="object"
                                  select="$target[1]"/>
               </xsl:call-template>
            </xsl:variable>
            <fo:basic-link internal-destination="{$id}">
               <xsl:text/>
               <fo:page-number-citation xsl:use-attribute-sets="index-page-numbers"
                                        ref-id="{$id}">
               </fo:page-number-citation>
               <xsl:text/>
            </fo:basic-link>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <!-- =============================================== -->
   <!-- The template for "see" references               -->
   <!-- =============================================== -->

   <xsl:template match="indexterm"
                 mode="index-see">
      <fo:block keep-with-previous.within-column="always"
                margin-left="10pt">
         <fo:inline font-style="italic">
            <xsl:text>see </xsl:text>
         </fo:inline>
         <xsl:value-of select="see"/>
      </fo:block>
   </xsl:template>

   <!-- =============================================== -->
   <!-- The template for "see also" references          -->
   <!-- =============================================== -->

   <xsl:template match="indexterm"
                 mode="index-seealso">
      <fo:block keep-with-previous.within-column="always"
                margin-left="10pt">
         <fo:inline font-style="italic">
            <xsl:text>see also </xsl:text>
         </fo:inline>
         <xsl:value-of select="seealso"/>
      </fo:block>
   </xsl:template>

   <xsl:template name="object.id">
      <xsl:param name="object"
                 select="."/>
      <xsl:choose>
         <xsl:when test="$object/@id">
            <xsl:value-of select="$object/@id"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="generate-id($object)"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="indexterm"
                 priority="1">
      <xsl:variable name="id">
         <xsl:call-template name="object.id">
            <xsl:with-param name="object"
                            select="."/>
         </xsl:call-template>
      </xsl:variable>
      <fo:inline id="{$id}"/>
   </xsl:template>
</xsl:stylesheet>
