
/***************************************************************************
*               Copyright (c), InterSystems Corporation. 1999 - 2003
*         File: ctrldemo.c                                                  
*         Date: 7 April 1999 
*	  Updated:  3/12/2003                                              
*  Description: An example of controlling a Cache configuration from a
*		user written application in "c".           
***************************************************************************/

/*****************  About this sample program.  **********************
*
*	This sample program is demonstrates how to control a
*	Windows NT or Windows 95 Cache configuration programatically.
*	It also demonstrates how to start up Cache processes in a 
*	variety of ways on Windows 32 system.
*
*	The sample program accepts the following command lines:
*
*	ctrldemo start <configname>
*	ctrldemo stop <configname>
*	ctrldemo force <configname>
*	ctrldemo terminal <configname> [<routine> [<namespace>]]
*	ctrldemo background <configname> [<routine> [<namespace>]]
*	ctrldemo tcpserver <configname> [<routine> [<namespace>]]
*	ctrldemo configlist
*	ctrldemo serverlist
*
*	The "start" command starts the configuration if it is not
*	already running.  This is equivalent to selecting "Start Cache"
*	from the Cache cube icon on the system tray.
*
*	The "stop" command shuts down the configuration if it is running.
*	This is equivalent to selecting "Stop Cache" from the 
*	Cache cube icon on the system tray.
*	
*	The "force" command performs the equivalent of a cforce on
*	Unix systems.
*
*	The "terminal" command creates a Cache terminal connection
*	to a Cache configuration.
*
*	The "background" command creates a process in a Cache 
*	configuration, using the NUL device as $Principal.  This
*	like creating a process using the JOB command in most
*	cases.
*
*	The "tcpserver" command creates a server process running
*	indicated program in the indicated namespace with a TCP
*	device (hard coded to port 7777 in the sample) used as
*	$Principal.  The example writes strings to a program
*	which simply echoes its input.  The Cache Object Script
*	program ^echo is included below.  The following command line 
*	will run the ^echo program in the USER namespace of the
*	configuration named "cache":
*
*		ctrldemo tcpserver cache ^echo USER
*
*
*	If the configuration is not running when the "terminal",
*	"background", or "tcpserver" command is issued, the sample program
*	asks the user if they want to start it, and if they affirm,
*	starts the configuration before creating the terminal,
*	background, or tcpserver process.
*
*	The configlist command lists installed configurations to
*	the standard output.  It demonstrates calling
*	CctrlGetConfigList() and CctrlGetConfigInfo()
*
*	The configlist command lists installed configurations to
*	the standard output.  It demonstrates calling
*	CctrlGetServerList() and CctrlGetServerInfo()
*
*	Error processing in the sample is rudimentary.
*
*	All interaction of the sample program with Cache is mediated
*	by the Cache Control Library, cctrl.dll which is loaded
*	dynamically.
*
************************************************************************/

/************** How to build the ctrldemo application. *****************
 *
 *	1) Using Microsoft Visual C++, create a new Windows 32 Console
 *	Application project.
 *
 *	2) Copy ctrldemo.c and ctrldemo.h into the project directory.
 *
 *	3) Using Project -> Add to Project -> Files..,  add ctrldemo.c
 *	and ctrldemo.h to the project.
 *
 *	4) Using Project -> Settings, select the Link tab and add wsock32
 *	to the Object/Library Modules list.
 *	
 *	5) Then select the Debug tab, and enter the path of the "bin"
 *	directory of your Cache configuration as the Working Directory.
 *	This is necessary so the LoadLibrary() call will find cctrl.dll.
 *
 *	6) Build the project.
 *
 *	NOTE:  When you create and install your own application, LoadLibrary()
 *	must be able to find cctrl.dll.  This can be done in several ways:
 *
 *		1) Your application's .exe can be in the same directory as
 *		cctrl.dll.
 *		2) Your application's working directory can be the binary
 *		directory of the configuration (example:  c:\cachesys\bin).
 *		3) The configuration binary directory can be in your path.
 *		4) Your application can supply the full path to cctrl.dll.
 *
 ************************************************************************/
 
/******************  The Cache Object Script Program, ^echo. **********
 *
 *	This is the Cache Object Stript program, ^echo, used by
 *	the example "tcpserver" command.  You can load it by
 *	logging into a Cache terminal, entering an argumentless
 *	zload command, cutting and pasting the text below
 *	into the terminal window, and entering a "zsave echo"
 *	command.
 *
 *
echo     ;
         k ^echo
         s $zt="err^echo"
         s ^echo("pid")=$j,^echo(0)=0
         s ^echo("$P")=$p
         u 0
LOOP     R X
         W X,!
         s (^echo(0),n)=^echo(0)+1,^echo(n)=X
         G LOOP
err      s $zt=""
         s ^echo("error")=$ze
         h



 *
 *
 ***********************************************************************/

/******************  Documentation of cctrl.dll functions.  *************
 *
 *	The sample program interacts with cctrl.dll, the Cache Control
 *	Library, which resides in the \bin directory under the
 *	configuration installation directory.  This library is used by
 *	Cache utilities like the Cache System Tray Manager to control
 *	Cache.  The library functions used by this sample program have
 *	function pointers typedeffed in ctrldemo.h, and are documented below:
 *
 ************************************************************************
 *
 *  FUNCTION NAME:      CctrlGetDirs(char *config, CCTRL_DIR_INFO *dirinfo)
 *
 *  PURPOSE:            Find configuration, binary, and manager directory 
 *			paths and service name given a configuration name.
 *
 *  RETURNED VALUE:     pointer to string containing configuration 
 *			directory path.
 *
 *  REFERENCED DATA:	Windows NT registry.
 *
 *  ERRORS:             returns (char *0) on ERROR.
 *			Call CctrlGetLastError() to get error string.
 *
 *
 ************************************************************************
 *	
 *  FUNCTION NAME:      CctrlConfigStatus(char *configname)
 *
 *  PURPOSE:            Return config status..
 *
 *  RETURNED VALUE:     0 - config is up
 *			1 - config is starting or stopping
 *			2 - config startup or shutdown aborted.
 *			3 - config is down
 *			4 - ERROR
 *
 ************************************************************************
 *
 *  FUNCTION NAME:      CctrlControl(char *command, char *configname)
 *
 *  PURPOSE:            Control a Cache configuration through the 
 *			Cache Control Service on Windows NT, or
 *			directly on Windows 95.  
 *
 *			Commands include:
 *			
 *			1) "start" - start a configuration.
 *			
 *			2) "stop" -  shut down a configuration gracefully.
 *			
 *			3) "stopnoshut" - shut down a configuration without
 *			   running the user supplied shutdown routine.
 *			
 *			4) "force" - force down a configuration.  This is
 *			    equivalent to cforce on unix systems.
 *
 *			5) "startstop" - shut down a configuration gracefully
 *			   and immediately restart it.
 *
 *			6) "csystraystart" - start up the csystray application.
 *			
 *
 *  RETURNED VALUE:     CCTRL_SUCCESS - the operation succeeded.
 *			CCTRL_ERROR - generic error
 *			CCTRL_INVALID_COMMAND - invalid command argument
 *			CCTRL_INVALID_CONFIGURATION - undefined configuration
 *			CCTRL_CONTROL_STU_ERROR	- ^STU failed.
 *
 *  ERRORS:             Following an error return, CctrlGetLastError()
 *			returns a pointer to an informational error
 *			string.
 *
 ***********************************************************************
 *
 *
 *  FUNCTION NAME:	CctrlRun(char *configuration, char *routine,
 *				 char *namespacename, char *IOtype)
 *
 *  PURPOSE:            Start a Cache process in the indicated configuration,
 *			and namespace, using the indicated principal I/O device
 *			and invoking the indicated routine.
 *
 *			I/O types values are:
 *				"cterminal" = Cache programmers terminal.
 *				"none"      = No I/O.  Process will be run
 *					      in the background with NUL:
 *					      used for $Principal.  Writes
 *					      to $P will be discarded.
 *					      Reads from $P produce an error.
 *
 *  RETURNED VALUE:    CCTRL_SUCCESS, or some error code.
 *
 *  WARNINGS:          On Windows NT, configuration MUST be running or
 *		       it is an error.  This is to avoid starting up
 *		       the configuration outside the context of the
 *		       control service.
 *
 ***********************************************************************
 *
 *  FUNCTION NAME:      CctrlRunIO(char *configuration,
 *	`			   char *routine,
 *				   char *namespacename,
 *				   char *IOtype,
 *				   HANDLE *hIOdevs,
 *				   char *cwd,
 *				   char *options,
 *				   HANDLE *child,
 *				   DWORD *childPID)
 *
 *  PURPOSE:            Start a Cache process in the indicated configuration,
 *			and namespace, using the indicated principal I/O device
 *			type and invoking the indicated routine.  The hIO
 *			argument is an array of three handles to be used as
 *			the standard input, output, and error devices of the
 *			Cache process.  The cwd argument points to path of a
 *			directory to become the working directory of the child
 *			process.  If the directory argument is zero, the working
 *			directory of current process is used.  The options argument
 *			are additional cache.exe command line options that will
 *			be appended to the generated command line.  A good use
 *			of the options argument is to declare a large symbol
 *			table size (-b 1024).  The child argument is a pointer
 *			to a variable of type HANDLE, where the handle to the
 *			child process will be returned.  If the value of handle
 *			is zero, the handle to the child process will be closed
 *			by this function.  The childPID argument is used to
 *			return the PID of the created cache.exe process.  This
 *			argument can be zero if the child's PID is not required.
 *
 *			I/O types values are:
 *				"TCP"	    = TCP socket.
 *
 *  RETURNED VALUE:     CCTRL_SUCCESS, or some CCTRL_... error code.
 *
 *  WARNINGS:		The handles in the hIO array must be inheritable.  Use
 *			DuplicateHandle() to make the handle inheritable by
 *			the child process.
 *
 *
 ***********************************************************************/

#include <windows.h>
#include <stdio.h>
#include "ctrldemo.h"

HINSTANCE		hCctrlDLL = NULL;  // handle to cctrl.dll
CCTRLCONFIGSTATUS	configstatus;
CCTRLCONTROL		control;
CCTRLRUN		run;
CCTRLRUNIO		runIO;
CCTRLGETDIRS		getdirs;
CCTRLGETLASTERROR	geterror;
CCTRLGETCONFIGLIST	getconfiglist;
CCTRLGETCONFIGINFO	getconfiginfo;
CCTRLGETSERVERLIST	getserverlist;
CCTRLGETSERVERINFO	getserverinfo;

char	myConfigDir[CCTRL_PATH_SIZE];
char 	myBinaryDir[CCTRL_PATH_SIZE];
char 	myMgrDir[CCTRL_PATH_SIZE];
char	myServiceName[CCTRL_SERVICENAME_SIZE];
	
CCTRL_DIR_INFO	cache_paths = 
	{myConfigDir, myBinaryDir, myMgrDir, myServiceName,
	 CCTRL_PATH_SIZE, CCTRL_PATH_SIZE, CCTRL_PATH_SIZE, CCTRL_SERVICENAME_SIZE};

#define	ERR	1
#define	OK	0
#define	COM_ERROR	0
#define COM_START	1
#define COM_STOP	2
#define COM_FORCE	3
#define COM_CTERM	4
#define COM_TCPSERVER	5
#define COM_BACKGROUND	6
#define	COM_CONFIGLIST	7
#define	COM_SERVERLIST	8
#define MBTITLE "Your Application Name - Error"


int cvtcmd(char *cmd) {
	/* convert alphabetic command to a token, -1 if undefined. */
	if (!cmd) return COM_ERROR;				/* no command given. */
	if (!stricmp(cmd,"start")) 
		return COM_START;	/* Start config. */
	if (!stricmp(cmd,"stop")) 
		return COM_STOP;	/* Stop config. */
	if (!stricmp(cmd,"force")) 
		return COM_FORCE;	/* Cforce config. */
	if (!stricmp(cmd,"terminal")) 
		return COM_CTERM;	/* Start terminal window.*/
	if (!stricmp(cmd,"tcpserver")) 
		return COM_TCPSERVER;	/* Start TCP server. */
	if (!stricmp(cmd,"background")) 
		return COM_BACKGROUND;	/* Start Cache process with NUL: I/O. */
	if (!stricmp(cmd,"configlist")) 
		return COM_CONFIGLIST;	/* Display config list. */
	if (!stricmp(cmd,"serverlist")) 
		return COM_SERVERLIST;	/* Display server list. */
	return COM_ERROR;
}

int wsastart; /* track whether this process issued a WSAStartup call */
int wsockstart()
{
	struct WSAData wsd;
	int	i;
	
	if  (!wsastart) {
	   i = WSAStartup(MAKEWORD(1,1),&wsd);
	   if (i) {
	      i = GetLastError();
	      return 0;
	   }
	   wsastart = 1;
	}
	return 1;	/* success */
}
#if	0
/* Duplicate a socket, the Microsoft way. */
SOCKET	dupsock(SOCKET sock) {
	WSAPROTOCOL_INFO pi;
	int rc;
	SOCKET tsock;

	rc = WSADuplicateSocket(sock,GetCurrentProcessId(),&pi);
	if (rc == SOCKET_ERROR) return INVALID_SOCKET;
	tsock = WSASocket(FROM_PROTOCOL_INFO,
			  FROM_PROTOCOL_INFO,
			  FROM_PROTOCOL_INFO,
			  &pi,0,0);
	closesocket(sock);
	return tsock;
}
#endif
/*
 * Run a Cache process, using TCP as its principal device.
 *
 * This function is intended to demonstrate starting up
 * and communicating with a TCP server on the local system.
 *
 * First, create both a server and a client socket, and make
 * a connection.
 * Next, call CctrlRunIO() to create a cache.exe process using
 * the server side of the socket for $Principal, running in
 * the user namespace, and running a routine named ^echo.
 * Then, send some messages and receive an echo.
 * Finally, close the connection.
 */
int runtcp(char *configname, char *routine, char *nspacename) {
	SOCKET		bsock;	     /* Socket for bind(). */
	SOCKET		asock;     /* Socket from accept(). */
	SOCKET		csock;	     /* Socket for connect(). */
	SOCKADDR_IN 	server_sin;  /* Server address info. */
	SOCKADDR_IN 	client_sin;  /* Client address info. */
	SOCKADDR_IN	accept_sin;  /* Returned address from accept() */
#ifdef	DEMO_DUPLICATE
	BOOL		tf;
#endif	/* DEMO_DUPLICATE */
	HANDLE		io[3];
	short		port = 7777;
	int		addrlen;
	int		rc;
	char		msg[128];
	int		i;
	int		len;

	if (!wsockstart()) return ERR;
	/* First, create, bind, and listen on a server side socket. */
	bsock = socket( AF_INET, SOCK_STREAM, 0);
	if (bsock == INVALID_SOCKET) {
	   WSACleanup();
	   return ERR;
	}
        /* Fill in the socket address structure. */
	memset(&server_sin, 0, sizeof(server_sin));
	server_sin.sin_family = AF_INET;
	server_sin.sin_addr.s_addr = htonl(INADDR_ANY);
	server_sin.sin_port = htons(port);
        /* Associate an address with the socket. (bind) */
	if (bind(bsock, (struct sockaddr *) &server_sin, 
        	 sizeof(server_sin)) == SOCKET_ERROR) {
           closesocket(bsock);
	   WSACleanup();
	   return ERR;
        }
	/* Listen on the socket. */
        if (listen( bsock, 1 ) < 0) {
           closesocket(bsock);
	   WSACleanup();
	   return ERR;
	}
	/* Create the client side socket. */
	csock = socket( AF_INET, SOCK_STREAM, 0);
	if (csock == INVALID_SOCKET) {
	   closesocket(bsock);
	   WSACleanup();
	   return ERR;
	}
#ifdef	DEMO_DUPLICATE
	/* children should not inherit handle we use for I/O. */
	tf = DuplicateHandle(GetCurrentProcess(), (HANDLE) csock, 
	   		     GetCurrentProcess(), (HANDLE *) &csock,
			     0, FALSE,
			     DUPLICATE_SAME_ACCESS | DUPLICATE_CLOSE_SOURCE);
	if (!tf) {
	   closesocket(bsock);
	   WSACleanup();
	   return ERR;
	}
#endif	/* DEMO_DUPLICATE */
        /* Fill in the socket address structure. */
	memset(&client_sin, 0, sizeof(client_sin));
	client_sin.sin_family = AF_INET;
	client_sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
	client_sin.sin_port = htons(port);
	if (connect(csock, 
		    (struct sockaddr *) &client_sin, 
		    sizeof(client_sin))) {
	   closesocket(bsock);
	   closesocket(csock);
	   WSACleanup();
	   return ERR;
	}
	addrlen = sizeof(accept_sin);
	asock = accept(bsock, (struct sockaddr *) &accept_sin,
			 (int *) &addrlen);
	if (asock == INVALID_SOCKET) {
	   closesocket(bsock);
	   closesocket(csock);
	   WSACleanup();
	   return ERR;
	}
	io[0] = io[1] = io[2] = (HANDLE) asock;
	closesocket(bsock); /* We won't do any more accepts. */
	rc = (runIO) (configname, routine, nspacename,"TCP",io,0,"",0,0);
	closesocket(asock); /* Close child's socket. */

	for (i=1 ; i < 10 ; i++) {
	   sprintf(msg,"This is message number %d.",i);
	   len = send(csock, msg, strlen(msg), 0);
	   msg[0] = 0;
	   len = recv(csock, msg, sizeof(msg), 0);
	   printf("%s\n",msg);
	}
	/* Be certain to call shutdown() for sending if you do not
	 * duplicate the socket used in the connect() call to make it not
	 * inheritable.  If you do not, the cache.exe process will not 
	 * recognize that the client process has disconnected because the
	 * inherited handle to the client socket is still open.  In this case
	 * the server socket does not receive a disconnect when the client
	 * process disconnects because the shared client side socket is
	 * still open.  It is usually a good idea to call shutdown() anyway.
	 */
	shutdown(csock,1);
	closesocket(csock);
#ifndef	DEMO_DUPLICATE
	/* If you choose to duplicate the connected socket to make it
	 * uninheritable, then WSACleanup will get an exception, invalid
	 * handle.  I don't know the impact of not calling WSACleanup.
	 * The client process exits OK, but perhaps there are
	 * some stranded resources.
	 */
	WSACleanup();
#endif	/* DEMO_DUPLICATE */
	return 0;

}

/****************************** PROLOGUE *******************************
*
*  FUNCTION NAME:       ShowList
*
*  PURPOSE:             Display config list in response to "ccontrol all".
*
*  RETURNED VALUE:      none
*
*  ERRORS:              none
*
***********************************************************************/
void ShowConfigList(void) {
	char	cfglist[2048];
	char	msg[4096];
	int	i;
	int	nconfigs;
	char	*cp = cfglist;
	char	*cp2;
	char	*ocp = msg;
	CCTRL_CONFIG_INFO configinfo;
	int	rc;

	if (getconfiglist == 0) {				/* RJW725+ */
	   printf("The CctrlGetConfigList() function is not available in the associated version of cctrl.dll.\n");
	   return;
	}							/* RJW725- */
	nconfigs = getconfiglist(sizeof(cfglist), cfglist);
	if (!nconfigs) sprintf(msg,"No configurations installed.");
	else for (i = 0 ; i < nconfigs ; i++ ) {
	   cp2 = cp;
	   while (*cp && *cp != '*' ) cp++;
	   *cp = (char) 0;
	   if (cp++ == cp2) break;
	   rc = getconfiginfo(cp2,&configinfo);
	   ocp += sprintf(ocp,"Configuration '%s'\t(%s installation)\n",
			  cp2,configinfo.InstallType);
	   ocp += sprintf(ocp,"\tdirectory: '%s'\n",configinfo.Directory);
	   ocp += sprintf(ocp,"\tversionid: '%s'\n",configinfo.Version);
	   ocp += sprintf(ocp,"\tconf file: '%s'  (default port = '%s')\n",
			configinfo.Properties,configinfo.Port);
	   ocp += sprintf(ocp,"\tstatus   : '%s'\n\n",configinfo.Status);
	}
	printf("\n%s\n",msg);
	return;
}

/****************************** PROLOGUE *******************************
*
*  FUNCTION NAME:       ShowServerList
*
*  PURPOSE:             Display server list in response to "ccontrol serverlist".
*
*  RETURNED VALUE:      none
*
*  ERRORS:              none
*
***********************************************************************/
void ShowServerList(void) {
	char	svrlist[2048];
	char	msg[4096];
	int	i;
	int	nservers;
	char	*cp = svrlist;
	char	*cp2;
	char	*ocp = msg;
	CCTRL_SERVER_INFO serverinfo;
	int	rc;

	if (getserverlist == 0) {				/* RJW725+ */
	   printf("The CctrlGetServerList() function is not available in the associated version of cctrl.dll.\n");
	   return;
	}							/* RJW725- */
	nservers = getserverlist(sizeof(svrlist), svrlist);
	if (!nservers) sprintf(msg,"No Servers defined.");
	else for (i = 0 ; i < nservers ; i++ ) {
	   cp2 = cp;
	   while (*cp && *cp != '*' ) cp++;
	   *cp = (char) 0;
	   if (cp++ == cp2) break;
	   rc = getserverinfo(cp2,&serverinfo);
	   ocp += sprintf(ocp,"Server: '%s'", cp2);
	   ocp += sprintf(ocp,"\tAddress: '%s'",serverinfo.Address);
	   ocp += sprintf(ocp,"\tPort: '%s'",serverinfo.Port);
	   ocp += sprintf(ocp,"\tTelnetPort: '%s'\n",serverinfo.Telnet);
	   ocp += sprintf(ocp,"Last Namespace: '%s'",serverinfo.LastNamespace);
	   ocp += sprintf(ocp,"\tLast Project: '%s'",serverinfo.LastProject);
	   ocp += sprintf(ocp,"\tComment: '%s'\n\n",serverinfo.Comment);
	}
	printf("\n%s\n",msg);
	return;
}


int main(int argc, char **argv)
{
	int	rc;		/* Return code. */
	unsigned int	i; /* loop index. */
	int	command = 0;
	char	message[256];	/* Storage for error messages. */
	char	*configname = "CACHE";
	char	*comstring = 0;
	char	*routine = 0;
	char	*nspacename = 0;
	char	*configdir = 0;
	char	*errtxt;

	/* Process command line. */
	if (argc > 1) comstring = argv[1];
	command = cvtcmd(comstring);		/* fetch command token. */
	if (command == COM_ERROR || argc > 5) {
		printf("Usage:  cctrl <command> [<config> [<routine> [<namespace>]]]\n");
		return ERR;
	}
	if (argc > 2) configname = argv[2];	/* fetch optional config name. */
	if (argc > 3) routine = argv[3];	/* fetch optional routine name. */
	if (argc > 4) nspacename = argv[4];	/* fetch optional namespace name. */
	/*  NOTE:  We need to use all capitals when passing the configuration name to
	 *         cctrl.dll because of an inconsistency in the way the telnet demon
	 *		   shutdown event is formatted.  This will be corrected in V3.2.
	 */
	for (i = 0 ; i < strlen(configname) ; i++) 
		if (islower(configname[i])) configname[i] = toupper(configname[i]); 
	/* Load cctrl.dll library and get function addresses.
	/* NOTE: Configuration binary directory (example:  c:\cachesys\bin) must
	 * be your current directory or your executable must reside there, or
	 * it must be in your path, or you must provide the path to cctrl.dll.
	 */
	hCctrlDLL = LoadLibrary("cctrl.dll");	/* load Cache control library. */
	if (hCctrlDLL == 0) {			/* on error, exit. */
		MessageBox(0, "cctrl.dll inaccessable", MBTITLE, 
			   MB_OK | MB_ICONERROR);
		return ERR;
	}
	/* Load Cache control function addresses. */
	getdirs = (CCTRLGETDIRS) GetProcAddress(hCctrlDLL,"CctrlGetDirs");
	configstatus = (CCTRLCONFIGSTATUS) GetProcAddress(hCctrlDLL,"CctrlConfigStatus");
	control = (CCTRLCONTROL) GetProcAddress(hCctrlDLL,"CctrlControl");
	run = (CCTRLRUN) GetProcAddress(hCctrlDLL,"CctrlRun");
	runIO = (CCTRLRUNIO) GetProcAddress(hCctrlDLL,"CctrlRunIO");
	geterror = (CCTRLGETLASTERROR) GetProcAddress(hCctrlDLL,"CctrlGetLastError");
	getconfiglist = (CCTRLGETCONFIGLIST) GetProcAddress(hCctrlDLL,"CctrlGetConfigList");
	getconfiginfo = (CCTRLGETCONFIGINFO) GetProcAddress(hCctrlDLL,"CctrlGetConfigInfo");
	getserverlist = (CCTRLGETSERVERLIST) GetProcAddress(hCctrlDLL,"CctrlGetServerList");
	getserverinfo = (CCTRLGETSERVERINFO) GetProcAddress(hCctrlDLL,"CctrlGetServerInfo");

	/* Get configuration paths. */
	configdir = (getdirs) (configname, &cache_paths);
	/* Process display requests. */
	switch (command) {
	case COM_CONFIGLIST:	/* Display config list. */
		ShowConfigList();
		return 0;
		break;
	case COM_SERVERLIST: /* Display server list. */
		ShowServerList();
		return 0;
		break;
	default:
		break;
	}
	/* Determine configuration state.
	 *  RETURNED VALUE:     0 config is up.
	 *			1 config is starting.
	 *			2 config startup failure.
	 *			3 config is down.
	 *			4 ERROR - configuration probably undefined.
	 */
getstatus:
	rc = (configstatus) (configname); /* Check status of configuration. */
chkstatus:
	switch(rc) {
	case 0: /* Configuration is running, and logins are enabled. */
		if (command == COM_STOP || command == COM_FORCE) {
			rc = (control) (comstring, configname);
			if (rc == CCTRL_SUCCESS) return OK;
			else return ERR;
		} else if (command == COM_START) return OK;
		/* Fall through and process run commands. */
		break;
	case 1:	/* Configuration is starting. */
		/* Ask user if they want to wait and continue. */
		sprintf(message,"Cache configuration %s is starting.\n"
			"Press OK to wait or CANCEL to quit.", configname);
		rc = MessageBox(0,message,MBTITLE, MB_OKCANCEL);
		if (rc == IDCANCEL) return ERR; /* don't start. */
		 /* Refetch status after user think time. */
		rc = (configstatus) ("cache");
		if (rc != 1) goto chkstatus; /* New state, reprocess status. */
		Sleep(10000);
		goto getstatus;	/* Try again. */
		break;
	case 2: /* Detected Cache startup failure.  Cannot proceed. */
		MessageBox(0,"Cache startup failed.\n"
			   "Please call customer support.",MBTITLE,
			   MB_OK | MB_ICONERROR);
		return ERR;
		break;
	case 3:	/* Configuration is down. */
		if (command == COM_STOP || command == COM_FORCE) return OK;
		if (command == COM_START) {
			rc = (control) (comstring, configname);
			if (rc == CCTRL_SUCCESS) return OK;
			/* Get cctrl.dll error text. You could display it. */
			errtxt = (geterror) ();
			MessageBox(0,errtxt,MBTITLE, MB_OK | MB_ICONERROR);
			return ERR;
		}
		/* Ask user if they want to start configuration. */
		sprintf(message,"Cache configuration %s is not running.\n"
			"Press OK to start it or CANCEL to quit.", configname);
		rc = MessageBox(0,message,MBTITLE, MB_OKCANCEL);
		if (rc == IDCANCEL) return ERR; /* don't start. */
		/* Start the configuration. */
		rc = (control) ("start", configname);
		if (rc != CCTRL_SUCCESS) {
			MessageBox(0,"Cache failed to start", MBTITLE, MB_OK);
			 /* Get cctrl.dll error text. You could display it. */
			errtxt = (geterror) ();
			return ERR;
		}
		/* Fall through and process run commands. */
		break;
	case 4: /* Some error, most likely undefined configuration. */
		/* Get cctrl.dll error text. You could display it like this. */
		errtxt = (geterror) ();
		MessageBox(0, errtxt, MBTITLE, MB_OK | MB_ICONERROR);
		sprintf(message,"Error accessing Cache configuration %s.\n"
			"Please call customer support.", configname);
		MessageBox(0, message, MBTITLE, MB_OK | MB_ICONERROR);
		return ERR;
		break;
	default:/* Will not happen, but code to catch future changes. */
		MessageBox(0, "Unknown Cache status, quitting.",
			   MBTITLE, MB_OK | MB_ICONERROR);
		return ERR;
		break;
	}
	/* Now, the configuration is running.  Process run commands. */
	switch (command) {
	case COM_CTERM:	/* Start a Cache terminal session. */
		rc = (run) (configname, routine, nspacename, "cterminal");
		break;
	case COM_TCPSERVER: /* Connect to a Cache server process using TCP. */
		rc = runtcp(configname, routine, nspacename);
		break;
	case COM_BACKGROUND: /* Create a Cache Background process with NUL
			      * device as $principal.
			      */
		rc = (run) (configname, routine, nspacename, "none");
		break;
	}
	if (rc != CCTRL_SUCCESS) {
		MessageBox(0,"Cache process startup failed", MBTITLE, MB_OK);
		rc = ERR;
	}

	return rc;
}
