
/***************************************************************************
*               Copyright (c), InterSystems Corporation. 1999 - 2002		    
*         File: ctrldemo.c                                                  
*         Date: 7 April 1999                                               
*	  Updated:  11/01/2002                                              
*  Description: An example of controlling a Cache configuration from a
*		user written application in "c".           
***************************************************************************/

/****************************** PROLOGUE *******************************
*
*	This sample header provides declarations used control a Cache
*	configuration programatically, and to start up Cache
*	processes in a variety of ways.
*
*	NOTE:  You should use this header.  We will keep it up to date.
*	
*
***********************************************************************/

/* Define extern for C or C++ */
#ifdef __cplusplus
#define CCTRL_EXPORT extern "C"
#else
#define CCTRL_EXPORT extern
#endif

#define CCTRL_SUCCESS			0
#define CCTRL_ERROR			1
#define	CCTRL_CONTROL_STU_ERROR		4
#define	CCTRL_INVALID_COMMAND		5
#define	CCTRL_INVALID_CONFIGURATION	6
#define CCTRL_CONFIGURATION_INACTIVE	7
#define CCTRL_INVALID_HANDLE		8

#define	CCTRL_MAX_ERROR_LEN		4096
#define	CCTRL_PATH_SIZE			261 /* MAX_PATH + 1 */
#define CCTRL_CMDLINE_SIZE		2048
#define CCTRL_SERVICENAME_SIZE		CCTRL_PATH_SIZE + 32

#ifdef __cplusplus
extern "C" {
#endif 

typedef struct 
	_CctrlDirInfo {
	   char		*ConfigDir;
	   char 	*BinaryDir;
	   char 	*MgrDir;
	   char		*ServiceName;
	   unsigned int ConfigDirSz;
	   unsigned int	BinaryDirSz;
	   unsigned int	MgrDirSz;
	   unsigned int	ServiceNameSz;
	   } CCTRL_DIR_INFO, *PCCTRL_DIR_INFO;

typedef struct
	_CctrlConfigInfo{
	   char		*Status;	/* "up", "dn", "??" */
	   char		*Version;	/* Version String. */
	   char		*Port;		/* Port String. */
	   char		*Directory;	/* Installation Directory. */
	   char		*InstallType;	/* "Standard" or "Client" */
	   char		*Properties;	/* Config (.cpf) file path. */
	   char		buf[2048];	/* Storage for the above. */
	} CCTRL_CONFIG_INFO, *PCCTRL_CONFIG_INFO;
typedef struct
	_CctrlServerInfo{
	   char		*Address;	/* Address. */
	   char		*Comment;	/* Comment. */
	   char		*LastNamespace;	/* Last Namespace in Studio. */
	   char		*LastProject;	/* Last Project in Studio. */
	   char		*Port;		/* Super Server Port. */
	   char		*Telnet;	/* Telnet port. */
	   char		buf[2048];	/* . */
	} CCTRL_SERVER_INFO, *PCCTRL_SERVER_INFO;


typedef char *(_stdcall *CCTRLGETDIRS) (char *configname, CCTRL_DIR_INFO *paths);
typedef int (_stdcall *CCTRLCONFIGSTATUS) (char *configname);
typedef int (_stdcall *CCTRLCONTROL) (char *command, char *configname);
typedef int (_stdcall *CCTRLRUN) (char *command, char *routine, 
			 char *nspacename, char *IOtype);
typedef int (_stdcall *CCTRLRUNIO) (char *command, char *routine, 
			   char *nspacename, char *IOtype,
			   HANDLE *hIO, char *cwd,
			   char *comopts, HANDLE hChild,
			   DWORD *childPID);
typedef char *(_stdcall *CCTRLGETLASTERROR) (void);

/****************************** PROLOGUE *******************************
*
*  FUNCTION NAME:	CctrlGetConfigList(int buflen, char *buffer)
*
*  PURPOSE:		Return a list of installed configurations in buffer.
*			The list is a zero terminated string.  The "*"
*			character delimits configuration names within
*			the string.
*
*  RETURNED VALUE:	Number of configurations in list.  0 means error.
*
*  ERRORS:		On error, check CctrlGetLastError() for reason text.
*
***********************************************************************/
CCTRL_EXPORT int _stdcall
	CctrlGetConfigList(int buflen, char *buffer);

typedef int (_stdcall *CCTRLGETCONFIGLIST) (int buflen, char *buffer);

/****************************** PROLOGUE *******************************
*
*  FUNCTION NAME:       CctrGetConfigInfo(char *config, PCCTRL_CONFIG_INFO configinfo);
*
*  PURPOSE:             Return information about the requested configuration.
*
*  RETURNED VALUE:	CCTRL_SUCCESS or CCTRL_ERROR
*
*  ERRORS:		On error, check CctrlGetLastError() for reason text.
*
***********************************************************************/
CCTRL_EXPORT int _stdcall
	CctrlGetConfigInfo(char *config, PCCTRL_CONFIG_INFO configinfo);

typedef int (_stdcall *CCTRLGETCONFIGINFO) (char *config, PCCTRL_CONFIG_INFO configinfo);

/****************************** PROLOGUE *******************************
*
*  FUNCTION NAME:	CctrlGetServerList(int buflen, char *buffer)
*
*  PURPOSE:		Return a list of defined servers in buffer.
*			The list is a zero terminated string.  The "*"
*			character delimits server names within
*			the string.
*
*  RETURNED VALUE:	Number of servers list.  0 means error.
*
*  ERRORS:		On error, check CctrlGetLastError() for reason text.
*
***********************************************************************/
CCTRL_EXPORT int _stdcall
	CctrlGetServerList(int buflen, char *buffer);

typedef int (_stdcall *CCTRLGETSERVERLIST) (int buflen, char *buffer);

/****************************** PROLOGUE *******************************
*
*  FUNCTION NAME:       CctrGetServerInfo(server, serverinfo);
*
*  PURPOSE:             Return information about the requested server.
*
*  RETURNED VALUE:	CCTRL_SUCCESS or CCTRL_ERROR
*
*  ERRORS:		On error, check CctrlGetLastError() for reason text.
*
***********************************************************************/
CCTRL_EXPORT int _stdcall
	CctrlGetServerInfo(char *server, PCCTRL_SERVER_INFO serverinfo);

typedef int (_stdcall *CCTRLGETSERVERINFO) (char *config, PCCTRL_SERVER_INFO serverinfo);


#ifdef __cplusplus
}
#endif 
