#ifndef CPPBIND_ABS_TYPES_H
#define CPPBIND_ABS_TYPES_H

#include "os_defs.h"
#include "t_iostream.h"

class Database;

DLL_DECL extern const_name_t empty_name;
DLL_DECL extern const_name_t empty_str;

DLL_DECL extern const_name_t t_ListOfObjects;
DLL_DECL extern const_name_t t_ListOfDataTypes;
DLL_DECL extern const_name_t t_ArrayOfObjects;
DLL_DECL extern const_name_t t_ArrayOfDataTypes;
DLL_DECL extern const_name_t t_GlobalCharStream;
DLL_DECL extern const_name_t t_GlobalBinStream;
DLL_DECL extern const_name_t t_AbstractStream;
DLL_DECL extern const_name_t t_FileBinStream;
DLL_DECL extern const_name_t t_FileCharStream;
DLL_DECL extern const_name_t t_CharStream;
DLL_DECL extern const_name_t t_BinaryStream;
DLL_DECL extern const_name_t t_RelationshipObj;
DLL_DECL extern const_name_t t_SQLProcContext;
DLL_DECL extern const_name_t m_SetAt;
DLL_DECL extern const_name_t m_GetAt;
DLL_DECL extern const_name_t m_InsertAt;
DLL_DECL extern const_name_t m_RemoveAt;
DLL_DECL extern const_name_t m_GetNext;
DLL_DECL extern const_name_t m_GetPrevious;
DLL_DECL extern const_name_t m_Insert;
DLL_DECL extern const_name_t m_Remove;
DLL_DECL extern const_name_t m_GetStreamId;
DLL_DECL extern const_name_t m_SaveStream;
DLL_DECL extern const_name_t m_Clear;
DLL_DECL extern const_name_t m_Rewind;
DLL_DECL extern const_name_t m_MoveToEnd;
DLL_DECL extern const_name_t m_SizeGet;
DLL_DECL extern const_name_t m_FileName;
DLL_DECL extern const_name_t m_LastModifiedGet;
DLL_DECL extern const_name_t m_IsFileUnicode;
DLL_DECL extern const_name_t m_Read;
DLL_DECL extern const_name_t m_ReadLine;
DLL_DECL extern const_name_t m_Write;
DLL_DECL extern const_name_t m_CopyFrom;
DLL_DECL extern const_name_t m_FilenameSet;
DLL_DECL extern const_name_t m_LinkToFile;
DLL_DECL extern const_name_t m_GetAttribute;
DLL_DECL extern const_name_t m_SetAttribute;
DLL_DECL extern const_name_t m_IsDefinedAttribute;
DLL_DECL extern const_name_t m_NextAttribute;
DLL_DECL extern const_name_t m_DeleteAttribute;
DLL_DECL extern const_name_t m_Count;
DLL_DECL extern const_name_t m_New;
DLL_DECL extern const_name_t m_Oid;
DLL_DECL extern const_name_t m_Save;

DLL_DECL extern const_name_t p_RowCount;
DLL_DECL extern const_name_t p_SQLCode;

//==============

typedef std::string string;

class bin_string : public string {
public:
   bin_string(const char* p, int l)
      : string(p, l)
      {}
   bin_string(const std::string& s)
      : string(s)
      {}
   bin_string()
      {}

   bool operator==(const bin_string& s) const
      {
         return
            static_cast<const std::string&>(*this) ==
            static_cast<const std::string&>(s);
      }
   bool operator!=(const bin_string& s) const
      { return !(*this == s); }

   const char* get_buf() const
      { return c_str(); }
   long get_size() const
      { return length(); }
};

inline t_istream& operator>>(t_istream& in, bin_string& val)
{ bool uni; in.get(&uni, &val); return in; }

//==============

class D_type {
private:
   bool null_val;

public:
   enum t_id { D_VOID = -1,
               D_OBJ_ID = 0,
               D_INT_ID = 1,
               D_DOUBLE_ID = 2,
               D_BINARY_ID = 3,
               D_STRING_ID = 4,
               D_STATUS_ID = 5,
               D_TIME_ID = 6,
               D_DATE_ID = 7,
               D_TIMESTAMP_ID = 8,
               D_BOOL_ID = 9,
               D_CURRENCY_ID = 10 };

   static t_id get_t_id(int id)
      { return static_cast<t_id>(id); }

   D_type()
      { null_val = false; }

   virtual ~D_type() {}

   bool is_null() const
      { return null_val; }
   void make_not_null()
      { null_val = false; }
   virtual void make_null()
      { null_val = true; }
   // A subclass may have to do some clean up, for example close its oref.
   // Call make_null() of the super class(es) for complete clean up.
   // set the object value so that it will give alert the user if he tries
   // to use a null object.

   // t_io

   virtual void get(t_istream& in, Database* db = 0) = 0;
   virtual void put(t_ostream& out, Database* db = 0) const = 0;
};

inline t_istream& operator>>(t_istream& in, D_type& val)
{ val.get(in); return in; }

inline t_ostream& operator<<(t_ostream& out,const D_type& val)
{ val.put(out); return out; }

//==============

typedef enum {
   ERR_INTERNAL = -1,
   ERR_NULL_VALUE = -2,
   ERR_NOT_FOUND = -3,
   ERR_WRONG_TYPE = -4,
   ERR_OUT_OF_RANGE = -5,
   ERR_BAD_DSN_STR = -6,
   ERR_CLOSE_OBJ = -7,
   ERR_CLOSE_OBJS = -8,
   ERR_OPEN_OBJ = -9,
   ERR_USER = -1000
} client_err;

class Db_err {
private:
   int err_code;
   std::string msg;

protected:
   bin_string data;

public:
   Db_err()
      : err_code(0)
      {}
   Db_err(int c)
      : err_code(c)
      {}
   Db_err(int c, const char* m, int l)
      : err_code(c), msg(m, l)
      {}
   Db_err(int c, const std::string& m)
      : err_code(c), msg(m)
      {}

   void clear()
      { err_code = 0; msg.erase(); }

   void set_code(int code)
      { err_code = code; }

   void set_msg(const char* m, int l)
      { msg.assign(m, l); }
   void set_msg(const char* m)
      { msg.assign(m); }

   operator bool() const
      { return err_code != 0; }

   int get_code() const
      { return err_code; }

   // Gen_err interface

   const std::string& get_msg() const
      { return msg; }

   // t_io

   DLL_DECL void get(t_istream& in);
};

inline std::ostream& operator<<(std::ostream& out, const Db_err& err)
{
   return out << "Error: code = " << err.get_code() << ", "
              << "msg = " << err.get_msg();
}

inline t_istream& operator>>(t_istream& in, Db_err& val)
{ val.get(in); return in; }

//==============

template<class T>
class Handle {
   T* rep;
   int* pcount;

public:
   T* operator->()
      { return rep; }
   const T* operator->() const
      { return rep; }

   Handle()
      : rep(new T()), pcount(new int(1))
      {}
   Handle(T* pp)
      : rep(pp), pcount(new int(1))
      {}
   Handle(const Handle& r)
      : rep(r.rep), pcount(r.pcount)
      { ++(*pcount); }

   Handle& operator=(const Handle& r)
      {
         if (rep == r.rep)
            return *this;

         if (--(*pcount) == 0)
         {
            delete rep;
            delete pcount;
         }

         rep = r.rep;
         pcount = r.pcount;
         ++(*pcount);
         return *this;
      }

   ~Handle()
      { if (--(*pcount) == 0) { delete rep; delete pcount; } }
};

//==============

class db_output_callback {
public:
   virtual void output(bool is_uni, const char* buf, byte_size_t size)
      {
         if (is_uni)
            std::wcout.write((const wchar_t*) buf, size / sizeof(wchar_t)).flush();
         else
            std::cout.write(buf, size).flush();
      }

   virtual ~db_output_callback() {}
};

#endif
