#ifndef CPPBIND_CACHE_H
#define CPPBIND_CACHE_H

#include "os_defs.h"

#include <vector>
#include <map>

#include "prim_types.h"
#include "t_iostream.h"

typedef enum {NONE = 0, ANT = 1, LAZY = 2} Cache_type;
typedef enum {FOUND, IS_NULL, NOT_FOUND} Cache_ret_type;

class Database;

class Cache {
protected:
   Database* db;

public:
   Cache(Database* _db)
      : db(_db)
      {}
   virtual ~Cache() {}

   virtual Cache_type get_type() = 0;

   virtual void get(t_istream& in) = 0;
   virtual void put(t_ostream& out) = 0;

   virtual void remove(int oref) = 0;
   virtual bool get_property(int oref,
                             int ii,
                             int jj,
                             bool is_obj,
                             t_istrstream& in) = 0;
   virtual bool set_property(int oref,
                             int ii,
                             int jj,
                             int mod_flag,
                             bool is_obj,
                             const_name_t name,
                             D_type* val) = 0;
   virtual void enqueue_close(int oref) = 0;
};

//==============

class key_2i {
   int ii, jj;

public:
   key_2i(int i, int j)
      : ii(i), jj(j)
      {}
   bool operator<(const key_2i& key) const
      {
         return
            (ii < key.ii) ||
            (jj < key.jj);
      }
   bool equals(int i, int j) const
      { return (ii == i) && (jj == j); }
   int get_ii() const
      { return ii; }
   int get_jj() const
      { return jj; }
};

//==============

class key_3i {
   int ii, jj, kk;

public:
   key_3i(int i, int j, int k)
      : ii(i), jj(j), kk(k)
      {}
   bool operator<(const key_3i& key) const
      {
         return
            (ii < key.ii) ||
            (jj < key.jj) ||
            (kk < key.kk);
      }
   bool equals(int i, int j, int k) const
      { return (ii == i) && (jj == j) && (kk == k); }
   int get_ii() const
      { return ii; }
   int get_jj() const
      { return jj; }
   int get_kk() const
      { return kk; }
};

//==============

class ZObjVal_rep {
private:
   std::map< int, std::pair<bool, d_string> > nodes;
   typedef std::map< int, std::pair<bool, d_string> >::iterator It;

public:
   ZObjVal_rep(t_istream& in);

   void get_property(int ii, int jj, bool unicode_db, t_istrstream& in);

   bool is_valid_node(int ii)
      {
         It itor = nodes.find(ii);
         return (itor != nodes.end()) && ((itor->second).first != false);
      }

   void make_node_invalid(int ii)
      {
         It itor = nodes.find(ii);
         if (itor != nodes.end())
            (itor->second).first = false;
      }
};

//==============

class Cache_map {
public:
   std::map<int, ZObjVal_rep*> cache_map;
   // the map owns the pointers

   ~Cache_map();

   ZObjVal_rep* get(int oref)
      {
         std::map<int, ZObjVal_rep*>::iterator itor = cache_map.find(oref);
         return (itor != cache_map.end()) ? itor->second : 0;
      }
   void remove(int oref);
   void insert(int oref, t_istream& in);
};

//==============

class Unset_props : public is_uni_srv_info {
private:
   std::map<key_3i, std::pair<int, t_ostrstream> > lit_props;
   std::map<key_3i, std::pair<db_name, t_ostrstream> > obj_props;

   typedef std::map<key_3i, std::pair<db_name, t_ostrstream> >::iterator obj_itor_t;
   typedef std::map<key_3i, std::pair<int, t_ostrstream> >::iterator lit_itor_t;

public:
   Unset_props(bool uni_srv)
      : is_uni_srv_info(uni_srv)
      {}

   void set_property(int oref,
                     int ii,
                     int jj,
                     int mod_flag,
                     bool is_obj,
                     const_name_t name,
                     const D_type* val);
   bool get_property(int oref, int ii, int jj, bool is_obj, t_istrstream& in);
   void put(t_ostream& out, Cache* cache);
   // make set list and remove $zobjval for invalid orefs from cache
};

//==============

class Close_queue {
private:
   int num_elems;
   t_ostrstream queue;

public:
   Close_queue()
      : num_elems(0)
      { queue.reset(); }

   void enqueue(int oref)
      { ++num_elems; queue << oref; }
   void put(t_ostream& out)
      {
         out << num_elems;
         if (num_elems > 0) // bug in usparc
            out.write(queue.get_buf(), queue.get_size());
         num_elems = 0; queue.reset();
      }
};

#endif
