#ifndef CPPBIND_CLASS_DEF_H
#define CPPBIND_CLASS_DEF_H

#include "os_defs.h"

#include <string>
#include <vector>
#include <map>
#include <set>

#include "prim_types.h"

//==============

template<typename T>
class db_name_dict {
private:
   std::map<db_name, const T*> dict;

public:
   ~db_name_dict()
      {
         typename std::map<db_name, const T*>::iterator itor = dict.begin();
         while (itor != dict.end())
         {
            delete itor->second;
            ++itor;
         }
      }

   void add(const T* val)
      {
         dict.insert(std::make_pair(val->get_name(), val));
      }

   bool find(const_name_t _name, const T** res) const
      {
         db_name& name = db_name::cmn; name.assign(_name);
         typename std::map<db_name, const T*>::const_iterator itor = dict.find(name);
         if (itor != dict.end())
         {
            *res = itor->second;
            return true;
         }
         else
            return false;
      }
};

//==============

class Prop_def {
private:
   db_name cache_type;
   d_int type_id;
   bool lit;
   db_name name;
   int ii, jj, mod_flag;
   db_name coln_elem_type;

public:
   Prop_def(t_istream& in);

   const db_name& get_cache_type() const
      { return cache_type; }
   D_type::t_id get_type_id() const
      { return D_type::get_t_id(type_id); }

   bool is_literal() const
      { return lit; }

   const db_name& get_name() const
      { return name; }

   int get_ii() const
      { return ii; }
   int get_jj() const
      { return jj; }
   int get_mod_flag() const
      { return mod_flag; }
};

//==============

class Arg_def {
private:
   db_name cache_type;
   d_int type_id;
   bool lit;
   bool by_ref;
   db_name name;
   bool def;
   bin_string def_val;

public:
   Arg_def(t_istream& in);

   const db_name& get_cache_type() const
      { return cache_type; }
   D_type::t_id get_type_id() const
      { return D_type::get_t_id(type_id); }

   bool is_literal() const
      { return lit; }

   bool is_by_ref() const
      { return by_ref; }

   const db_name& get_name() const
      { return name; }

   bool is_default() const
      { return def; }
   const bin_string& get_def_val() const
      { return def_val; }
};

//==============

class Args;

class Method_def {
private:
   bool cls_mtd;

   db_name cache_type;
   d_int type_id;
   bool lit;
   bool srv_only;

   db_name name;

   std::vector<const Arg_def*> args;
   mutable int idx;

   friend class Args;

public:
   Method_def(t_istream& in);
   ~Method_def()
      { for (size_t  ii = 0; ii < args.size(); ++ii) delete args[ii]; }

   bool is_cls_mtd() const
      { return cls_mtd; }
   bool is_func() const
      { return D_type::get_t_id(type_id) != D_type::D_VOID; }

   const db_name& get_cache_type() const
      { return cache_type; }
   D_type::t_id get_type_id() const
      { return D_type::get_t_id(type_id); }

   bool is_literal() const
      { return lit; }

   const db_name& get_name() const
      { return name; }

   int get_num_args() const
      { return args.size(); }
   void args_rewind() const
      { idx = 0; }
   const Arg_def& arg_get() const
      { return *args[idx]; }
   void arg_next() const
      { ++idx; }
};

//==============

class Query_def {
private:
   db_name name;
   db_name stored_proc_name;
   int num_pars;

public:
   Query_def(t_istream& in)
      {
         in >> name
            >> stored_proc_name
            >> num_pars;
      }

   const db_name& get_name() const
      { return name; }
   const db_name& get_proc_name() const
      { return stored_proc_name; }
   int get_num_pars() const
      { return num_pars; }
};

//==============

class Database;
class Properties;
class Methods;
class Queries;
class Com_class_def;

class Class_def {
private:
   db_name tname;

   db_name_dict<Prop_def> properties;
   db_name_dict<Method_def> methods;
   db_name_dict<Query_def> queries;

   void get_props(Database* db, const_name_t name);
   void get_methods(Database* db, const_name_t name);
   void get_queries(Database* db, const_name_t name);

   friend class Properties;
   friend class Methods;
   friend class Queries;
   friend class Com_class_def;

public:
   Class_def(Database* db, const_name_t type)
      : tname(type)
      {
         get_props(db, type);
         get_methods(db, type);
         get_queries(db, type);
      }

   const db_name& get_tname() const
      { return tname; }

   DLL_DECL bool prop_get(const_name_t name, const Prop_def** res) const;
   DLL_DECL bool mtd_get(const_name_t name, const Method_def** res) const;
   DLL_DECL bool query_get(const_name_t name, const Query_def** res) const;
};

#endif
