#ifndef CPPBIND_COLLECTIONS_H
#define CPPBIND_COLLECTIONS_H

#include "dyn_obj.h"

#include <vector>
#include <map>
#include <iterator>

class d_collection : protected Dyn_obj {
protected:
   int get_num_elems()
      { return d_int(run_obj_method(m_Count, 0, 0)); }

   d_collection() {}

   d_collection(Database* db, int ref, const_name_t tname)
      : Dyn_obj(db, ref, tname)
      {}

   void clear()
      { run_obj_method(m_Clear, 0, 0); }

public:
   Database* get_db()
      { return Dyn_obj::get_db(); }
   int get_ref()
      { return Dyn_obj::get_ref(); }

   DLL_DECL void set(const D_type& val, const D_type& key);
   DLL_DECL virtual void ins(const D_type& val, const D_type& key); // map ins to set for d_map
   DLL_DECL t_istream& get(const D_type& key);
   DLL_DECL void del(const D_type& key);
};

//==============

template<typename T, typename K>
class d_obj_coln_type : public d_ref<T> {
private:
   d_collection* owner;
   K key;
   bool empty;

public:
   d_obj_coln_type(d_collection* my_owner, K my_key)
      : owner(my_owner), key(my_key), empty(true)
      {}
   d_obj_coln_type(const d_obj_coln_type& val)
      : d_ref<T>(val), owner(val.owner), key(val.key), empty(val.empty)
      {}
   d_obj_coln_type(const d_ref<T>& val)
      : d_ref<T>(val), owner(0), empty(false)
      {}
   d_obj_coln_type()
      : owner(0), empty(true)
      {}

   void reset_coln_info(d_collection* my_owner, K my_key)
      { owner = my_owner; key = my_key; }

   K get_key() const
      { return key; }

   d_obj_coln_type& validate()
      {
         if (!empty) return *this;
         static_cast< d_ref<T>& >(*this) = T::openref(owner->get(key), owner->get_db());
         empty = false;
         return *this;
      }

   d_obj_coln_type& operator=(const d_obj_coln_type& r)
      {
         if (!empty && !r.empty && (d_ref<T>::operator->()->get_ref() == r->get_ref()))
            return *this;

         if (!r.empty)
         {
            *static_cast< d_ref<T>* >(this) = r;
            owner->set(*this, key);
         }
         empty = r.empty;

         return *this;
      }
};

//=====

template<typename T, typename K>
class d_prim_coln_type : public T {
private:
   d_collection* owner;
   K key;

public:
   d_prim_coln_type(d_collection* my_owner, const T& val, K my_key)
      : T(val), owner(my_owner), key(my_key)
      {}
   d_prim_coln_type(const d_prim_coln_type& val)
      : T(val), owner(val.owner), key(val.key)
      {}
   d_prim_coln_type(const T& val)
      : T(val), owner(0)
      {}
   d_prim_coln_type()
      : owner(0)
      {}

   void reset_coln_info(d_collection* my_owner, K my_key)
      { owner = my_owner; key = my_key; }

   K get_key() const
      { return key; }

   d_prim_coln_type& operator=(const d_prim_coln_type& r)
      {
         static_cast<T&>(*this) = r;
         owner->set(*this, key);
         return *this;
      }
};

//==============

template<typename T, typename K>
class Validate_obj_coln_type {
private:
   typedef d_obj_coln_type<T, K> val_t;

public:
   val_t& operator()(val_t& val)
      { return val.validate(); }
};

//=====

template<typename T, typename K>
class Validate_prim_coln_type {
private:
   typedef d_prim_coln_type<T, K> val_t;

public:
   val_t& operator()(val_t& val)
      { return val; }
   const val_t& operator()(const val_t& val) const
      { return val; }
};

//==============

template<typename I, typename V, typename F>
class d_coln_vec_iterator : public std::iterator<std::random_access_iterator_tag, V> {
private:
   int idx;

protected:
   I itor;
   F validate;

public:
   // should only used by d_vector
   I get_itor() const
      { return itor; }
   int get_idx() const
      { return idx; }
   const V& get_elem() const
      { return *itor; }
   V& get_elem()
      { return *itor; }

   d_coln_vec_iterator(const I& it, int i)
      : itor(it), idx(i)
      {}
   d_coln_vec_iterator()
      : idx(-1)
      {}

   V* operator->()
      { return &validate(*itor); }
   V& operator*()
      { return validate(*itor); }

   d_coln_vec_iterator& operator++()
      { ++itor; ++idx; return *this; }
   d_coln_vec_iterator& operator--()
      { --itor; --idx; return *this; }

   d_coln_vec_iterator operator++(int)
      { d_coln_vec_iterator res(itor);  ++itor; return res; }
   d_coln_vec_iterator operator--(int)
      { d_coln_vec_iterator res(itor);  --itor; return res; }

   d_coln_vec_iterator& operator+=(int n)
      { itor += n; idx += n; return *this; }
   d_coln_vec_iterator& operator-=(int n)
      { itor -= n; idx -= n; return *this; }

   d_coln_vec_iterator operator+(int n)
      { return d_coln_vec_iterator(itor+n, idx+n); }
   d_coln_vec_iterator operator-(int n)
      { return d_coln_vec_iterator(itor-n, idx-n); }

   int operator-(const d_coln_vec_iterator& it)
      { return itor - it.itor; }

   bool operator==(const d_coln_vec_iterator& it) const
      { return itor == it.itor; }
   bool operator!=(const d_coln_vec_iterator& it) const
      { return itor != it.itor; }
   bool operator>=(const d_coln_vec_iterator& it) const
      { return itor >= it.itor; }
   bool operator<=(const d_coln_vec_iterator& it) const
      { return itor <= it.itor; }
   bool operator>(const d_coln_vec_iterator& it) const
      { return itor > it.itor; }
   bool operator<(const d_coln_vec_iterator& it) const
      { return itor < it.itor; }
};

//=====

template<typename S>
class d_vector : public d_collection {
   friend class d_ref<d_vector>;
   friend class Database;

public:
   typedef typename S::value_type value_type;

   static const char* init_msg()
      { return S::init_msg(); }

private:
   typedef typename S::vec_t vec_t;
   vec_t vec;

   typename S::validate_f_type validate;

   void init(t_istream& in)
      { S::init_vec(this, vec, in); }

protected:
   d_vector() {}

   d_vector(Database* db, int oref, const_name_t tname)
      : d_collection(db, oref, tname)
      { get_db()->init_coln(get_ref(), *this); }

public:
   // Vector interface

   // types

   typedef typename vec_t::size_type size_type;

   typedef typename S::iterator iterator;
   typedef typename S::reverse_iterator reverse_iterator;
   typedef typename vec_t::reference reference;
   typedef typename vec_t::const_reference const_reference;

   // iterators

   iterator begin()
      { return iterator(vec.begin(), 0); }
   iterator end()
      { return iterator(vec.end(), vec.size()); }

   reverse_iterator rbegin()
      { return reverse_iterator(vec.rbegin(), vec.size() - 1); }
   reverse_iterator rend()
      { return reverse_iterator(vec.rend(), -1); }

   // element access

   reference operator[](size_type idx)
      { return validate(vec[idx]); }
   const_reference operator[](size_type idx) const
      { return validate(vec[idx]); }

   reference at(size_type idx)
      { return validate(vec.at(idx)); }
   const_reference at(size_type idx) const
      { return validate(vec.at(idx)); }

   // stack operations

   void push_back(const value_type& val)
      {
         d_int key = vec.size() + 1;
         ins(val, key);
         vec.push_back(val);
         vec.back().reset_coln_info(this, key);
      }
   void pop_back()
      {
         d_int key = vec.size();
         del(key);
         vec.pop_back();
      }

   //  list operations

   iterator insert(iterator pos, const value_type& val);
   iterator erase(iterator pos);
   void clear()
      { d_collection::clear(); vec.clear(); }

   // size and capacity

   size_type size()
      { return vec.size(); }
   bool empty() const
      { return vec.empty(); }
   size_type max_size() const
      { return vec.max_size(); }
   size_type capacity() const
      { return vec.capacity(); }
   void reserve(size_type n)
      { vec.reserve(n); }
};

template<typename T>
typename d_vector<T>::iterator d_vector<T>::insert(iterator pos, const value_type& val)
{
   d_int key = pos.get_idx() + 1;
   ins(val, key);
   iterator res(vec.insert(pos.get_itor(), val), pos.get_idx());
   res.get_elem().reset_coln_info(this, key);

   iterator itor = ++res;
   while (itor.get_itor() != vec.end())
   {
      itor.get_elem().reset_coln_info(this, ++key);
      ++itor;
   }

   return res;
}

template<typename T>
typename d_vector<T>::iterator d_vector<T>::erase(iterator pos)
{
   d_int key = pos.get_idx() + 1;
   del(key);
   iterator res(vec.erase(pos.get_itor()), pos.get_idx());

   iterator itor = res;
   while (itor.get_itor() != vec.end())
   {
      itor.get_elem().reset_coln_info(this, key);
      ++itor;
      ++key;
   }

   return res;
}

//=====

template<typename T>
class d_obj_vector_traits {
public:
   typedef Validate_obj_coln_type<T, d_int> validate_f_type;

   static const char* init_msg()
      { return INIT_OBJ_LST; }

   typedef d_obj_coln_type<T, d_int> value_type;
   typedef std::vector<value_type> vec_t;

   typedef d_coln_vec_iterator<typename vec_t::iterator, value_type, validate_f_type> iterator;
   typedef d_coln_vec_iterator<typename vec_t::reverse_iterator, value_type, validate_f_type> reverse_iterator;

   static void init_vec(d_vector<d_obj_vector_traits>* pvec, vec_t& v, t_istream& in);
};

template<typename T>
void d_obj_vector_traits<T>::init_vec(d_vector<d_obj_vector_traits>* pvec,
                                      vec_t& v,
                                      t_istream& in)
{
   int num_elems;
   in >> num_elems;
   v.reserve(num_elems);
   for (int ii = 0; ii < num_elems; ++ii)
      v.push_back(value_type(pvec, ii+1));
}

//=====

template<typename T>
class d_prim_vector_traits {
public:
   typedef Validate_prim_coln_type<T, d_int> validate_f_type;

   static const char* init_msg()
      { return INIT_PRIM_LST; }

   typedef d_prim_coln_type<T, d_int> value_type;
   typedef std::vector<value_type> vec_t;

   typedef d_coln_vec_iterator<typename vec_t::iterator, value_type, validate_f_type> iterator;
   typedef d_coln_vec_iterator<typename vec_t::reverse_iterator, value_type, validate_f_type> reverse_iterator;

   static void init_vec(d_vector<d_prim_vector_traits>* pvec, vec_t& v, t_istream& in);
};

template<typename T>
void d_prim_vector_traits<T>::init_vec(d_vector<d_prim_vector_traits>* pvec,
                                       vec_t& v,
                                       t_istream& in)
{
   int num_elems;
   in >> num_elems;
   v.reserve(num_elems);
   T val;
   for (int ii = 0; ii < num_elems; ++ii)
   {
      in >> val;
      v.push_back(value_type(pvec, val, ii+1));
   }
}

//=====

template<typename T>
class d_obj_vector : public d_vector< d_obj_vector_traits<T> > {
   friend class d_ref< d_obj_vector<T> >;

private:
   typedef d_obj_vector_traits<T> S;

protected:
   d_obj_vector() {}

   d_obj_vector(Database* db, int ref, const_name_t tname)
      : d_vector<S>(db, ref, tname)
      {}

public:
   static d_ref< d_obj_vector<T> > openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_obj_vector<T>), &obj, &num_refs))
            return d_ref< d_obj_vector<T> >(dynamic_cast< d_obj_vector<T>*>(obj), num_refs);
         else
            return d_ref< d_obj_vector<T> >(new d_obj_vector<T>(db, ref, t_ListOfObjects));
      }

   static d_ref< d_obj_vector<T> > create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_ListOfObjects))); }
};

//=====

template<typename T>
class d_prim_vector: public d_vector< d_prim_vector_traits<T> > {
   friend class d_ref< d_prim_vector<T> >;

private:
   typedef d_prim_vector_traits<T> S;

protected:
   d_prim_vector() {}

   d_prim_vector(Database* db, int ref, const_name_t tname)
      : d_vector<S>(db, ref, tname)
      {}

public:
   static d_ref< d_prim_vector<T> > openref(Database* db, int oref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(oref, typeid(d_prim_vector<T>), &obj, &num_refs))
            return d_ref< d_prim_vector<T> >(dynamic_cast< d_prim_vector<T>*>(obj), num_refs);
         else
            return d_ref< d_prim_vector<T> >(new d_prim_vector<T>(db, oref, t_ListOfDataTypes));
      }

   static d_ref< d_prim_vector<T> > create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_ListOfDataTypes))); }
};

//==============

template<typename I, typename V, typename F>
class d_coln_map_iterator : public std::iterator<std::bidirectional_iterator_tag, V> {
private:
   I itor;
   F validate;

public:
   // should only used by d_map
   I get_itor() const
      { return itor; }
   const typename V::second_type& get_elem() const
      { return itor->second; }
   typename V::second_type& get_elem()
      { return itor->second; }

   d_coln_map_iterator(const I& it)
      : itor(it)
      {}
   d_coln_map_iterator()
      {}

   V* operator->()
      { validate(itor->second); return &(*itor); }
   V& operator*()
      { validate(itor->second); return *itor; }

   const typename V::first_type& get_key() const
      { return itor->first; }

   d_coln_map_iterator& operator++()
      { ++itor; return *this; }
   d_coln_map_iterator& operator--()
      { --itor; return *this; }

   d_coln_map_iterator operator++(int)
      { d_coln_map_iterator res(itor++); return res; }
   d_coln_map_iterator operator--(int)
      { d_coln_map_iterator res(itor--); return res; }

   d_coln_map_iterator& operator=(const d_coln_map_iterator& it)
      { itor = it.itor; return *this; }

   bool operator==(const d_coln_map_iterator& it) const
      { return itor == it.itor; }
   bool operator!=(const d_coln_map_iterator& it) const
      { return itor != it.itor; }
};

//=====

template<typename S>
class d_map : public d_collection {
   friend class d_ref<d_map>;
   friend class Database;

public:
   typedef typename S::key_type key_type;
   typedef typename S::mapped_type mapped_type;
   typedef typename S::value_type value_type;

   static const char* init_msg()
      { return S::init_msg(); }

private:
   typedef typename S::map_t map_t;
   map_t map;

   typename S::validate_f_type validate;

   void init(t_istream& in)
      { S::init_map(this, map, in); }

protected:
   d_map() {}

   d_map(Database* db, int ref, const_name_t tname)
      : d_collection(db, ref, tname)
      { get_db()->init_coln(get_ref(), *this); }

public:
   void ins(const D_type& val, const D_type& key)
      { d_collection::set(val, key); }

   // Map interface

   // types

   typedef typename map_t::size_type size_type;

   typedef typename S::iterator iterator;
   typedef typename S::reverse_iterator reverse_iterator;
   typedef typename map_t::reference reference;
   typedef typename map_t::const_reference const_reference;

   // iterators

   iterator begin()
      { return map.begin(); }
   iterator end()
      { return map.end(); }

   reverse_iterator rbegin()
      { return map.rbegin(); }
   reverse_iterator rend()
      { return map.rend(); }

   // element access

   mapped_type& operator[](const key_type& key)
      { return validate(map[key]); }
   const mapped_type& operator[](const key_type& key) const
      { return validate(map[key]); }

   // map operations

   iterator find(const key_type& key)
      { return map.find(key); }

   //  list operations

   std::pair<iterator, bool> insert(const value_type& val)
      {
         ins(val.second, val.first);
         std::pair<iterator, bool> res = map.insert(val);
         if (res.second)
            (res.first).get_elem().reset_coln_info(this, val.first);
         return res;
      }
   iterator insert(iterator pos, const value_type& val)
      {
         ins(val.second, val.first);
         std::pair<iterator, bool> res = map.insert(pos.get_itor(), val);
         if (res.second)
            (res.first).get_elem().reset_coln_info(this, val.first);
         return res;
      }
   void erase(iterator pos)
      {
         del(pos.get_key());
         map.erase(pos.get_itor());
      }
   size_type erase(const key_type& key)
      {
         iterator pos = find(key);
         if (pos.get_itor() != map.end())
         {
            del(key);
            map.erase(pos.get_itor());
            return 1;
         }
         else
            return 0;
      }
   void clear()
      { d_collection::clear(); map.clear(); }

   // size and capacity

   size_type size()
      { return map.size(); }
   bool empty() const
      { return map.empty(); }
   size_type max_size() const
      { return map.max_size(); }
   size_type capacity() const
      { return map.capacity(); }
};

//=====

template<typename T>
class d_obj_map_traits {
public:
   typedef Validate_obj_coln_type<T, d_string> validate_f_type;

   static const char* init_msg()
      { return INIT_OBJ_ARR; }

   typedef d_string key_type;
   typedef d_obj_coln_type<T, key_type> mapped_type;
   typedef std::pair<const key_type, mapped_type> value_type;
   typedef std::map<key_type, mapped_type> map_t;

   typedef d_coln_map_iterator<typename map_t::iterator, value_type, validate_f_type> iterator;
   typedef d_coln_map_iterator<typename map_t::reverse_iterator, value_type, validate_f_type> reverse_iterator;

   static void init_map(d_map<d_obj_map_traits>* pmap, map_t& m, t_istream& in);
};

template<typename T>
void d_obj_map_traits<T>::init_map(d_map<d_obj_map_traits>* pmap,
                                   map_t& m,
                                   t_istream& in)
{
   int num_elems;
   in >> num_elems;
   key_type key;
   for (int ii = 0; ii < num_elems; ++ii)
   {
      in >> key;
      m.insert(std::make_pair(key, mapped_type(pmap, key)));
   }
}

//=====

template<typename T>
class d_prim_map_traits {
public:
   typedef Validate_prim_coln_type<T, d_string> validate_f_type;

   static const char* init_msg()
      { return INIT_PRIM_ARR; }

   typedef d_string key_type;
   typedef d_prim_coln_type<T, key_type> mapped_type;
   typedef std::pair<const key_type, mapped_type> value_type;
   typedef std::map<key_type, mapped_type> map_t;

   typedef d_coln_map_iterator<typename map_t::iterator, value_type, validate_f_type> iterator;
   typedef d_coln_map_iterator<typename map_t::reverse_iterator, value_type, validate_f_type> reverse_iterator;

   static void init_map(d_map<d_prim_map_traits>* pmap, map_t& m, t_istream& in);
};

template<typename T>
void d_prim_map_traits<T>::init_map(d_map<d_prim_map_traits>* pmap,
                                    map_t& m,
                                    t_istream& in)
{
   int num_elems;
   in >> num_elems;
   key_type key;
   T val;
   for (int ii = 0; ii < num_elems; ++ii)
   {
      in >> key >> val;
      m.insert(std::make_pair(key, mapped_type(pmap, val, key)));
   }
}

//=====

template<typename T>
class d_obj_map : public d_map< d_obj_map_traits<T> > {
   friend class d_ref< d_obj_map<T> >;

private:
   typedef d_obj_map_traits<T> S;

protected:
   d_obj_map() {}

   d_obj_map(Database* db, int ref, const_name_t tname)
      : d_map<S>(db, ref, tname)
      {}

public:
   static d_ref< d_obj_map<T> > openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_obj_map<T>), &obj, &num_refs))
            return d_ref< d_obj_map<T> >(dynamic_cast< d_obj_map<T>*>(obj), num_refs);
         else
            return d_ref< d_obj_map<T> >(new d_obj_map<T>(db, ref, t_ArrayOfObjects));
      }

   static d_ref< d_obj_map<T> > create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_ArrayOfObjects))); }
};

//=====

template<typename T>
class d_prim_map: public d_map< d_prim_map_traits<T> > {
   friend class d_ref< d_prim_map<T> >;

private:
   typedef d_prim_map_traits<T> S;

protected:
   d_prim_map() {}

   d_prim_map(Database* db, int ref, const_name_t tname)
      : d_map<S>(db, ref, tname)
      {}

public:
   static d_ref< d_prim_map<T> > openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_prim_map<T>), &obj, &num_refs))
            return d_ref< d_prim_map<T> >(dynamic_cast< d_prim_map<T>*>(obj), num_refs);
         else
            return d_ref< d_prim_map<T> >(new d_prim_map<T>(db, ref, t_ArrayOfDataTypes));
      }

   static d_ref< d_prim_map<T> > create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_ArrayOfDataTypes))); }
};

//==============

#endif
