#ifndef CPPBIND_COMMON_H
#define CPPBIND_COMMON_H

#include "os_defs.h"

//==============

class is_uni_srv_info {
private:
   bool uni_srv;

public:
   is_uni_srv_info(bool val = false)
      : uni_srv(val)
      {}

   bool is_uni_srv() const
      { return uni_srv; }
   void set_uni_srv(bool val)
      { uni_srv = val; }
};

//==============

typedef unsigned short byte_size_t;
typedef unsigned short char_size_t;
typedef unsigned char uchr;

typedef unsigned int UINT4;
// from CacheODBC

typedef const wchar_t* const_name_t;
typedef const wchar_t* const_str_t;

typedef enum { multibyte, cache_uni, cpp_uni } uni_t;

inline int num(const wchar_t* p)
{ return *p - L'0'; }
inline int num(const char* p)
{ return *p - '0'; }

template<typename T>
inline bool is_num(const T* p)
{ int n = num(p); return (-1 < n) && (n < 10); }

inline bool is_plus(const char* p)
{ return *p == '+'; }
inline bool is_plus(const wchar_t* p)
{ return *p == '+'; }

inline bool is_minus(const char* p)
{ return *p == L'-'; }
inline bool is_minus(const wchar_t* p)
{ return *p == L'-'; }

template<typename T>
__int64 stoi64(const T* p)
{
   __int64 res = 0;

   int sign = 1;
   if (*p == 0) return 0;
   else if (is_plus(p)) ++p;
   else if (is_minus(p)) { sign = -1; ++p; }

   while (is_num(p)) { res = res * 10 + num(p); ++p; }
   return res * sign;
}

#ifndef WIN32

inline __int64 __atoi64(const char* p)
{ return stoi64(p); }
inline __int64 __wtoi64(const wchar_t* p)
{ return stoi64(p); }

void __i64toa(__int64 val, char* p, int radix);

#else

#define __atoi64 _atoi64
#define __wtoi64 _wtoi64
#define __i64toa _i64toa

#endif

#endif
