#ifndef CPPBIND_DLIST_H
#define CPPBIND_DLIST_H

#include "prim_types.h"

//==============

const char null_lst_elem = 1;

class d_list : public d_binary, public is_uni_srv_info {
private:
   mutable int idx;
   mutable int str_idx;

   DLL_DECL static t_ostrstream out;

   template<typename T>
   static void _get_elem(const char* buf, T& res)
      {
         byte_size_t size = dlist_fmt::get_elem_size(buf);
         t_istrstream in(buf, size);
         in >> res;
      }

   template<typename T>
   void create_elem(const T& val, const char** p, byte_size_t* size)
      {
         out.set_uni_srv(is_uni_srv());
         out.reset();
         out << val;
         *p = out.get_buf();
         *size = out.get_size();
      }
   void create_elem(const char* v_buf, char_size_t v_size,
                    const char** p, byte_size_t* size)
      {
         out.reset();
         out.put(multibyte, v_buf, v_size);
         *p = out.get_buf();
         *size = out.get_size();
      }
   void create_elem(const wchar_t* v_buf, char_size_t v_size,
                    const char** p, byte_size_t* size);

   template<typename T>
   void _set_elem(const T& val)
      {
         const char* p; byte_size_t size;
         create_elem(val, &p, &size);
         set_elem_bin(p, size);
      }
   template<typename T>
   void _set_elem(const T* v_buf, char_size_t v_size)
      {
         const char* p; byte_size_t size;
         create_elem(v_buf, v_size, &p, &size);
         set_elem_bin(p, size);
      }

   template<typename T>
   void _ins_elem(const T& val)
      {
         const char* p; byte_size_t size;
         create_elem(val, &p, &size);
         ins_elem_bin(p, size);
      }
   template<typename T>
   void _ins_elem(const T* v_buf, char_size_t v_size)
      {
         const char* p; byte_size_t size;
         create_elem(v_buf, v_size, &p, &size);
         ins_elem_bin(p, size);
      }

   template<typename T>
   void _append_elem(const T& val)
      {
         const char* p; byte_size_t size;
         create_elem(val, &p, &size);
         append_elem_bin(p, size);
      }
   template<typename T>
   void _append_elem(const T* v_buf, char_size_t v_size)
      {
         const char* p; byte_size_t size;
         create_elem(v_buf, v_size, &p, &size);
         append_elem_bin(p, size);
      }

   void reset_idx() const
      { idx = str_idx = 0; }

public:
   // C interface

   DLL_DECL static void get_elem(const char* buf, __int64* val);
   DLL_DECL static void get_elem(const char* buf, double* val);
   DLL_DECL static void get_elem(const char* buf, d_string& val);
   DLL_DECL static void get_elem(const char* buf, d_wstring& val);
   DLL_DECL static void get_elem(const char* buf, d_binary& val);
   static void get_elem(const char* buf,
                        bool* is_uni, const char** p_buf, byte_size_t* p_size)
      {
         byte_size_t hsize = dlist_fmt::get_elem_header_size(buf);
         *is_uni = dlist_fmt::is_uni(buf);
         *p_buf = buf + hsize;
         *p_size = dlist_fmt::get_elem_size(buf) - hsize;
      }

   static bool is_elem_int(const char* buf)
      { return dlist_fmt::is_int(buf); }
   static bool is_elem_double(const char* buf)
      { return dlist_fmt::is_int(buf); }
   static bool is_elem_str(const char* buf)
      { return dlist_fmt::is_str(buf); }
   static bool is_elem_null(const char* buf)
      { return dlist_fmt::is_null(buf); }

   static char get_elem_type(const char* buf)
      { return dlist_fmt::get_elem_type(buf); }
   static byte_size_t get_elem_size(const char* buf)
      { return dlist_fmt::get_elem_size(buf); }

   // C++ interface

   d_list()
      : d_binary(), idx(0), str_idx(0)
      {}
   d_list(const char* buf, byte_size_t size)
      : d_binary(buf, size), idx(0), str_idx(0)
      {}

   DLL_DECL void move_to(int idx) const;
   void move_to_front() const
      { reset_idx(); }
   void next() const
      { ++idx; str_idx += dlist_fmt::get_elem_size(get_ptr()); }
   bool at_end() const
      { return str_idx == get_size(); }

   int get_elem_idx() const
      { return idx; }
   const char* get_ptr() const
      { return get_buf() + str_idx; }

   bool is_elem_int() const
      { return is_elem_int(get_ptr()); }
   bool is_elem_double() const
      { return is_elem_double(get_ptr()); }
   bool is_elem_str() const
      { return is_elem_str(get_ptr()); }
   bool is_elem_null() const
      { return is_elem_null(get_ptr()); }
   char get_elem_type() const
      { return get_elem_type(get_ptr()); }

   void get_elem(__int64* val) const
      { get_elem(get_ptr(), val); }
   void get_elem(double* val) const
      { get_elem(get_ptr(), val); }
   void get_elem(d_string& val) const
      { get_elem(get_ptr(), val); }
   void get_elem(d_wstring& val) const
      { get_elem(get_ptr(), val); }
   void get_elem(d_binary& val) const
      { get_elem(get_ptr(), val); }
   void get_elem(bool* is_uni, const char** p_buf, byte_size_t* p_size) const
      { get_elem(get_ptr(), is_uni, p_buf, p_size); }
   void get_elem_bin(const char** pbuf, byte_size_t* psize) const
      { *pbuf = get_ptr(); *psize = dlist_fmt::get_elem_size(*pbuf); }

   DLL_DECL void set_elem(__int64 val);
   DLL_DECL void set_elem(double val);
   DLL_DECL void set_elem(const d_string& val);
   DLL_DECL void set_elem(const d_wstring& val);
   DLL_DECL void set_elem(const d_binary& val);
   DLL_DECL void set_elem(const wchar_t* p, char_size_t size);
   DLL_DECL void set_elem(const char* p, char_size_t size);
   void set_elem_bin(const char* p, byte_size_t size)
      { val.replace(str_idx, dlist_fmt::get_elem_size(get_ptr()), p, size); }
   void set_elem_null()
      { set_elem_bin(&null_lst_elem, 1); }

   DLL_DECL void ins_elem(__int64 val);
   DLL_DECL void ins_elem(double val);
   DLL_DECL void ins_elem(const d_string& val);
   DLL_DECL void ins_elem(const d_wstring& val);
   DLL_DECL void ins_elem(const d_binary& val);
   DLL_DECL void ins_elem(const wchar_t* p, char_size_t size);
   DLL_DECL void ins_elem(const char* p, char_size_t size);
   void ins_elem_bin(const char* p, byte_size_t size)
      { val.insert(str_idx, p, size); }
   void ins_elem_null()
      { ins_elem_bin(&null_lst_elem, 1); }

   DLL_DECL void append_elem(__int64 val);
   DLL_DECL void append_elem(double val);
   DLL_DECL void append_elem(const d_string& val);
   DLL_DECL void append_elem(const d_wstring& val);
   DLL_DECL void append_elem(const d_binary& val);
   DLL_DECL void append_elem(const wchar_t* p, char_size_t size);
   DLL_DECL void append_elem(const char* p, char_size_t size);
   void append_elem_bin(const char* p, byte_size_t size)
      { val.append(p, size); }
   void append_elem_null()
      { append_elem_bin(&null_lst_elem, 1); }

   void del_elem()
      { val.erase(str_idx, dlist_fmt::get_elem_size(get_ptr())); }

   void clear()
      { val.erase(); reset_idx(); }
   void reset(const char* buf, byte_size_t size)
      { val.assign(buf, size); reset_idx(); }

   int count()
      { while (!at_end()) next(); return get_elem_idx(); }

   // t_io

   d_list(t_istream& in)
      : d_binary(in), idx(0), str_idx(0)
      {}
};

#endif
