#ifndef CPPBIND_DLIST_FMT_H
#define CPPBIND_DLIST_FMT_H

#define VT_MARKER    0
#define VT_MSTRING   1
#define VT_USTRING   2
#define VT_POS_MINT  4
#define VT_NEG_MINT  5
#define VT_POS_MREAL 6
#define VT_NEG_MREAL 7

#include <iostream>

#include "common.h"

//==============

class dlist_fmt {
public:
   // low level I/O

   // header

   static void get_meta_info(std::istream& in, char* ptype, byte_size_t* psize);
   static void put_meta_info(std::ostream& out, char type, byte_size_t size);

   static byte_size_t get_elem_size(const char* lst)
      { return (*lst) ? uchr(*lst) : uchr(*(lst + 1)) + (uchr(*(lst + 2)) << 8) + 3; }
   static char get_elem_type(const char* lst)
      { return (*lst) ? ((*lst == 1) ? VT_MSTRING : *(lst + 1)) : *(lst + 3); }
   static void get_elem_header(const char* lst, char* ptype, byte_size_t* psize)
      {
         *psize = get_elem_size(lst);
         *ptype = get_elem_type(lst);
      }
   static byte_size_t get_elem_header_size(const char* lst)
      { return (*lst) ? ((*lst == 1) ? 1 : 2) : 4; }

   static bool is_int(const char* lst)
      {
         char t = get_elem_type(lst);
         return (t == VT_POS_MINT) || (t == VT_NEG_MINT);
      }
   static bool is_double(const char* lst)
      {
         char t = get_elem_type(lst);
         return (t == VT_POS_MREAL) || (t == VT_NEG_MREAL);
      }
   static bool is_str(const char* lst)
      {
         char t = get_elem_type(lst);
         return (t == VT_MSTRING) || (t == VT_USTRING);
      }
   static bool is_uni(const char* lst)
      {
         char t = get_elem_type(lst);
         return t == VT_USTRING;
      }
   static bool is_null(const char* lst)
      { return get_elem_size(lst) == 1; }

   // data

   static __int64 get_int(std::istream& in, char type, byte_size_t size);
   static double get_double(std::istream& in, char type, byte_size_t size);

   static void get_str(std::istream& in, byte_size_t size, std::string* s)
      {
         s->erase(); s->reserve(size);
         for (byte_size_t ii = 0; ii < size; s->append(1, char(in.get())), ++ii);
      }
   static void get_str(std::istream& in, byte_size_t size, char* buf)
      { in.read(buf, size); }

   // high level I/O

   static void skip_elem(std::istream& in)
      {
         char type; byte_size_t size;
         get_meta_info(in, &type, &size);
         in.ignore(size);
      }

   static void get(std::istream& in, __int64* pval, bool* null_val);
   static void get(std::istream& in, double* pval, bool* null_val);

   static void put(std::ostream& out, __int64 val);
   static void put(std::ostream& out, double val);

   static void get(std::istream& in,
                   bool* is_uni,
                   char* buf, byte_size_t cap,
                   byte_size_t* psize);
   static void get(std::istream& in, bool* is_uni, std::string* str);

   static void put(std::ostream& out, uni_t uni, const char* buf, byte_size_t len);

   static void put_null(std::ostream& out)
      { put_meta_info(out, VT_MSTRING, 0); }
};

#endif
