#ifndef CPPBIND_DYN_OBJ_H
#define CPPBIND_DYN_OBJ_H

#include "obj_types.h"
#include "query.h"

class Dyn_obj : public Obj_t {
   friend class d_ref<Dyn_obj>;
   friend class Com_dyn_obj;
   friend class Com_stream_obj;

public:
   static const Prop_def& prop_get(const Class_def* def, const_name_t name)
      {
         const Prop_def* prop_def;
         if (!def->prop_get(name, &prop_def))
            throw Db_err(ERR_INTERNAL, "property not found");
         return *prop_def;
      }
   static const Method_def& mtd_get(const Class_def* def, const_name_t name)
      {
         const Method_def* mtd_def;
         if (!def->mtd_get(name, &mtd_def))
            throw Db_err(ERR_INTERNAL, "method not found");
         return *mtd_def;
      }
   static const Query_def& query_get(const Class_def* def, const_name_t name)
      {
         const Query_def* query_def;
         if (!def->query_get(name, &query_def))
            throw Db_err(ERR_INTERNAL, "query not found");
         return *query_def;
      }

   DLL_DECL t_istream& get_property(const Prop_def& prop_def) const;
   DLL_DECL void set_property(const Prop_def& prop_def, D_type* val);

   t_istream& run_obj_method(const Method_def& mtd_def,
                             D_type** args,
                             int num_args)
      { return run_method(get_db(), def, mtd_def, get_ref(), args, num_args); }
   static t_istream& run_class_method(Database* db,
                                      const Class_def* cl_def,
                                      const Method_def& mtd_def,
                                      D_type** args,
                                      int num_args)
      { return run_method(db, cl_def, mtd_def, -1, args, num_args); }

   DLL_DECL static t_istream& run_method(Database* db,
                                         const Class_def* cl_def,
                                         const Method_def& mtd_def,
                                         int obj_ref,
                                         D_type** args,
                                         int num_args);

protected:
   const Class_def* def;

   Dyn_obj()
      : def(0)
      {}

   Dyn_obj(Database* db, int ref, const_name_t type)
      {
         init_openref(db, ref);
         def = db->open_cl_def(type);
      }

public:
   static d_ref<Dyn_obj> openref(t_istream& in, Database* db)
      {
         int oref; const_name_t tname;
         abs_d_ref::get_oref_n_name(in, &oref, &tname);
         return openref(db, oref, tname);
      }

   static d_ref<Dyn_obj> openref(Database* db, int ref, const_name_t type)
      {
         if (ref == 0)
            return d_ref<Dyn_obj>();

         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(Dyn_obj), &obj, &num_refs))
            return d_ref<Dyn_obj>(dynamic_cast<Dyn_obj*>(obj), num_refs);
         else
            return d_ref<Dyn_obj>(new Dyn_obj(db, ref, type));
      }

   static d_ref<Dyn_obj> openid(Database* db,
                                const_name_t type,
                                const_str_t ident,
                                int concurrency = -1,
                                int timeout = -1,
                                Db_err* err = 0)
      {
         t_istream& in = db->openid_obj(type, ident, concurrency, timeout, err);
         return openref(in, db);
      }

   static d_ref<Dyn_obj> open(Database* db,
                              const d_binary& oid,
                              int concurrency = -1,
                              int timeout = -1,
                              Db_err* err = 0)
      {
         t_istream& in = db->open_obj(oid, concurrency, timeout, err);
         return openref(in, db);
      }

   static d_ref<Dyn_obj> create_new(Database* db,
                                    const_name_t type,
                                    const_str_t init_val = 0,
                                    Db_err* err = 0)
      {
         t_istream& in = db->make_obj(type, init_val, err);
         return openref(in, db);
      }

   ~Dyn_obj()
      {
         if ((get_db() != 0) && (def != 0))
            get_db()->close_cl_def(def->get_tname());
      }

   const Class_def* get_def() const
      { return def; }

   t_istream& get_property(const_name_t prop_name) const
      { return get_property(prop_get(def, prop_name)); }
   void set_property(const_name_t prop_name, D_type* val)
      { set_property(prop_get(def, prop_name), val); }

   DLL_DECL void get_query(const_name_t query_name, d_query& query) const;

   t_istream& run_obj_method(const_name_t mtd_name,
                             D_type** args,
                             int num_args)
      { return run_obj_method(mtd_get(def, mtd_name), args, num_args); }
   static t_istream& run_class_method(Database* db,
                                      const_name_t cl_name,
                                      const_name_t mtd,
                                      D_type** args,
                                      int num_args)
      {
         const Class_def* cl_def = db->open_cl_def(cl_name);
         return run_class_method(db, cl_def, mtd_get(cl_def, mtd), args, num_args);
      }
};

typedef d_ref<Dyn_obj> dyn_obj;

#endif
