#ifndef CPPBIND_EXT_ABS_TYPES_H
#define CPPBIND_EXT_ABS_TYPES_H

#include "dyn_obj.h"

//==============

template<typename V>
class Variant_d_base {
public:
   virtual ~Variant_d_base() {}

   virtual void set_variant(V* var) const = 0;
   virtual void set_val(V* var) = 0;
};

//==============

template<typename V, typename T, typename S>
class Variant_d_type : public Variant_d_base<V>, public T {
public:
   void set_variant(V* var) const
      {
         if (is_null())
            S::make_null(var);
         else
            S::assign(var, *this);
      }

   void set_val(V* var);
};

template<typename V, typename T, typename S>
void Variant_d_type<V, T, S>::set_val(V* var)
{
   if (S::is_null(var))
      make_null();
   else if (S::enable_type_checking())
      S::assign(*this, var);
   else
   {
      try {
         S::assign(*this, var);
      }
      catch(...) {
         make_null();
      }
   }
}

//==============

const int num_types = 11;

template<typename T1, int id1,
         typename T2, int id2,
         typename T3, int id3,
         typename T4, int id4,
         typename T5, int id5,
         typename T6, int id6,
         typename T7, int id7,
         typename T8, int id8,
         typename T9, int id9,
         typename T10, int id10,
         typename T11, int id11>

class Variant_d_type_factory {
private:
   T1 val1[max_num_obj_args + 1];
   T2 val2[max_num_obj_args + 1];
   T3 val3[max_num_obj_args + 1];
   T4 val4[max_num_obj_args + 1];
   T5 val5[max_num_obj_args + 1];
   T6 val6[max_num_obj_args + 1];
   T7 val7[max_num_obj_args + 1];
   T8 val8[max_num_obj_args + 1];
   T9 val9[max_num_obj_args + 1];
   T10 val10[max_num_obj_args + 1];
   T11 val11[max_num_obj_args + 1];

   int unused_elem[num_types];

public:
   Variant_d_type_factory()
      {}

   void reset()
      { for (int ii = 0; ii < num_types; unused_elem[ii] = 0, ++ii); }
   D_type* get(int id)
      {
         switch (id) {
         case id1: return get_val1();
         case id2: return get_val2();
         case id3: return get_val3();
         case id4: return get_val4();
         case id5: return get_val5();
         case id6: return get_val6();
         case id7: return get_val7();
         case id8: return get_val8();
         case id9: return get_val9();
         case id10: return get_val10();
         case id11: return get_val11();
         default: throw Db_err(ERR_INTERNAL, "assertion failed in Variant_d_type_holder");
         }
      }

   T1* get_val1() { return &val1[unused_elem[id1]++]; }
   T2* get_val2() { return &val2[unused_elem[id2]++]; }
   T3* get_val3() { return &val3[unused_elem[id3]++]; }
   T4* get_val4() { return &val4[unused_elem[id4]++]; }
   T5* get_val5() { return &val5[unused_elem[id5]++]; }
   T6* get_val6() { return &val6[unused_elem[id6]++]; }
   T7* get_val7() { return &val7[unused_elem[id7]++]; }
   T8* get_val8() { return &val8[unused_elem[id8]++]; }
   T9* get_val9() { return &val9[unused_elem[id9]++]; }
   T10* get_val10() { return &val10[unused_elem[id10]++]; }
   T11* get_val11() { return &val11[unused_elem[id11]++]; }
};

//==============

template<typename F, typename S>
class Ext_database : public Database {
private:
   F d_type_factory;

public:
   typedef S Traits;
   typedef F Factory;

   Ext_database(const d_connection& conn, bool use_cache = true)
      : Database(conn, use_cache)
      { set_term_output_callback(0); }

   F& get_d_type_factory()
      {
         d_type_factory.reset();
         return d_type_factory;
      }
};

//==============

template<typename D, typename V>
inline void ext_get_property(D* db,
                             Dyn_obj* obj,
                             const Prop_def& prop_def,
                             V* arg)
{
   typename D::Factory& d_type_factory = db->get_d_type_factory();
   D_type* cpp_arg = d_type_factory.get(prop_def.get_type_id());
   cpp_arg->get(obj->get_property(prop_def), db);
   dynamic_cast<Variant_d_base<V>*>(cpp_arg)->set_variant(arg);
}

template<typename D, typename V>
inline void ext_set_property(D* db,
                             Dyn_obj* obj,
                             const Prop_def& prop_def,
                             V* arg)
{
   typename D::Factory& d_type_factory = db->get_d_type_factory();
   D_type* cpp_arg = d_type_factory.get(prop_def.get_type_id());
   dynamic_cast<Variant_d_base<V>*>(cpp_arg)->set_val(arg);
   obj->set_property(prop_def, cpp_arg);
}

template<typename D, typename V>
void ext_run_method(D* db,
                    int oref,
                    const Class_def* cl_def,
                    const Method_def& mtd_def,
                    V* args_arr,
                    int num_args,
                    V* res)
{
   int ii;
   int num_by_ref;
   int arr_idx;

   typedef typename D::Traits S;

   D_type** args = (D_type**) db->get_arg_ptrs_buf();
   static int by_ref_idx[max_num_obj_args];

   typename D::Factory& d_type_factory = db->get_d_type_factory();
   const db_name& cl_name = cl_def->get_tname();

   mtd_def.args_rewind();
   num_by_ref = 0;
   for (ii = 0; ii < num_args; ++ii, mtd_def.arg_next())
   {
      D_type* cpp_arg = d_type_factory.get(mtd_def.arg_get().get_type_id());
      arr_idx = S::arg_idx(ii, num_args);
      dynamic_cast<Variant_d_base<V>*>(cpp_arg)->set_val(&args_arr[arr_idx]);

      if ((mtd_def.arg_get().is_by_ref()) && S::is_by_ref(&args_arr[arr_idx]))
      {
         by_ref_idx[num_by_ref] = ii;
         ++num_by_ref;
      }

      args[ii] = cpp_arg;
   }

   if (mtd_def.is_func())
   {
      D_type* cpp_arg = d_type_factory.get(mtd_def.get_type_id());

      cpp_arg->get(Dyn_obj::run_method(db, cl_def, mtd_def, oref, args, num_args), db);

      d_status* status = dynamic_cast<d_status*>(cpp_arg);
      if ((status != 0) && (status->get_code() != 0))
         status->throw_err();

      if (S::can_set_mtd_ret_val(res))
         dynamic_cast<Variant_d_base<V>*>(cpp_arg)->set_variant(res);
   }
   else
   {
      Dyn_obj::run_method(db, cl_def, mtd_def, oref, args, num_args);
      if (S::can_set_mtd_ret_val(res))
         S::make_null(res);
   }

   int idx;
   for (ii = 0; ii < num_by_ref; ++ii)
   {
      idx = by_ref_idx[ii];
      arr_idx = S::arg_idx(idx, num_args);
      dynamic_cast<Variant_d_base<V>*>(args[idx])->set_variant(&args_arr[arr_idx]);
   }
}

#endif
