#ifndef CPPBIND_LE_H
#define CPPBIND_LE_H

// include to take care of little-endian issues
typedef unsigned short U16;
typedef unsigned int U32;
#ifndef WIN32
typedef unsigned long long U64;
#else
typedef unsigned __int64 U64;
#endif

#define C64(x) x ## LL

// C16TOLE - host to little-endian format short

inline short C16TOLE(short x)
{
#ifndef MY_BIG_ENDIAN
    return x;
#else
    return (((U16)x & (U16)0x00ffU) << 8) | (((U16)x & (U16)0xff00U) >> 8);
#endif
}

//C32TOLE - host to little-endian format long
inline long C32TOLE(long l)
{
#ifndef MY_BIG_ENDIAN
    return l;
#else
        int x = l;
    return (((U32) (x) & (U32) 0x000000ffU) << 24) |
    (((U32) (x) & (U32) 0x0000ff00U) <<  8) |
    (((U32) (x) & (U32) 0x00ff0000U) >>  8) |
    (((U32) (x) & (U32) 0xff000000U) >> 24);


#endif

}
//C64TOLE - host to little-endian format 64 bit long
inline __int64 C64TOLE(__int64 x)
{
#ifndef MY_BIG_ENDIAN
    return x;
#else
    return (((U64) (x) &
        (U64) C64(0x00000000000000ffU)) << 56) |
      (((U64) (x) &
        (U64) C64(0x000000000000ff00U)) << 40) |
      (((U64) (x) &
        (U64) C64(0x0000000000ff0000U)) << 24) |
      (((U64) (x) &
        (U64) C64(0x00000000ff000000U)) <<  8) |
      (((U64) (x) &
        (U64) C64(0x000000ff00000000U)) >>  8) |
      (((U64) (x) &
        (U64) C64(0x0000ff0000000000U)) >> 24) |
      (((U64) (x) &
        (U64) C64(0x00ff000000000000U)) >> 40) |
      (((U64) (x) &
        (U64) C64(0xff00000000000000U)) >> 56);
#endif

}

//LETO16 - little-endian to host short
inline short LETO16(short x)
{
#ifndef MY_BIG_ENDIAN
    return x;
#else
    return C16TOLE(x);
#endif

}
// LETO32 - little-endian to host long
inline long LETO32(long x)
{
#ifndef MY_BIG_ENDIAN
    return x;
#else
    return C32TOLE(x);
#endif

}
//ltohll - little-endian to host 64 bit long
inline __int64 LETO64(__int64 x)
{
#ifndef MY_BIG_ENDIAN
    return x;
#else
    return C64TOLE(x);
#endif

}

#endif
