#ifndef CPPBIND_RELATIONSHIP_H
#define CPPBIND_RELATIONSHIP_H

#include "dyn_obj.h"

#include <iterator>

//==============

template<typename R>
class d_relationship_iterator :
   public std::iterator<std::input_iterator_tag, typename R::value_type> {
private:
   R* r;
   int dir;

   d_string key;

   typedef typename R::value_type value_type;
   value_type val;

   void read_next()
      { dir == 1 ? r->next(key, val) : r->prev(key, val); }

public:
   d_relationship_iterator(R* reln, int d = 1)
      : r(reln), dir(d), key("")
      {}

   void switch_dir()
      { dir = !dir; }

   d_string get_key() const
      { return key; }

   bool operator==(const d_relationship_iterator& itor) const
      { return key == itor.key; }
   bool operator!=(const d_relationship_iterator& itor) const
      { return !(*this == itor); }

   const value_type* operator->() const
      { return &val; }
   const value_type& operator*() const
      { return val; }

   d_relationship_iterator& operator++()
      { read_next(); return *this; }
   const d_relationship_iterator operator++(int)
      {
         d_relationship_iterator tmp = *this;
         read_next();
         return tmp;
      }
};

//==============

template<typename T>
class d_relationship : protected Dyn_obj {
   friend class d_ref<d_relationship>;
   friend class d_relationship_iterator<d_relationship>;

protected:
   d_relationship() {}
   d_relationship(Database* db, int ref, const_name_t tname)
      : Dyn_obj(db, ref, tname)
      {}

private:
   void next(d_string& key, d_ref<T>& val);
   void prev(d_string& key, d_ref<T>& val);
   d_status insert(const d_ref<T>& val);
   d_binary remove(const d_ref<T>& val);

public:
   Database* get_db()
      { return Dyn_obj::get_db(); }
   int get_ref()
      { return Dyn_obj::get_ref(); }

   static d_ref<d_relationship<T> > openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_relationship), &obj, &num_refs))
            return d_ref< d_relationship >(dynamic_cast<d_relationship<T>*>(obj), num_refs);
         else
            return d_ref< d_relationship >(new d_relationship(db, ref, t_RelationshipObj));
      }

   typedef d_ref<T> value_type;
   typedef d_relationship_iterator<d_relationship> iterator;
   typedef d_relationship_iterator<d_relationship> reverse_iterator;

   iterator begin()
      { return ++iterator(this); }
   iterator end()
      { return iterator(this); }

   reverse_iterator rbegin()
      { return ++iterator(this, 0); }
   reverse_iterator rend()
      { return iterator(this); }
};

template<typename T>
void d_relationship<T>::next(d_string& key, d_ref<T>& val)
{
   const int num_args = 1;
   D_type* args[num_args];
   args[0] = &key;

   val = T::openref(get_db(), d_int(run_obj_method(m_GetNext, args, num_args)));
}

template<typename T>
void d_relationship<T>::prev(d_string& key, d_ref<T>& val)
{
   const int num_args = 1;
   D_type* args[num_args];
   args[0] = &key;

   val = T::openref(get_db(), d_int(run_obj_method(m_GetPrevious, args, num_args)));
}

template<typename T>
d_status d_relationship<T>::insert(const d_ref<T>& val)
{
   const int num_args = 1;
   D_type* args[num_args];
   args[0] = &const_cast< d_ref<T>& >(val);

   return d_status(run_obj_method(m_Insert, args, num_args));
}

template<typename T>
d_binary d_relationship<T>::remove(const d_ref<T>& val)
{
   const int num_args = 1;
   D_type* args[num_args];
   args[0] = &const_cast< d_ref<T>& >(val);

   return d_binary(run_obj_method(m_Remove, args, num_args));
}

//==============

#endif
