#ifndef CPPBIND_STREAMS_H
#define CPPBIND_STREAMS_H

#include "dyn_obj.h"

const int io_buf_len = 1024 * 16;
const int io_putback_len = 4;

//==============

class d_stream : protected Dyn_obj {
   friend class d_ref<d_stream>;

public:
   DLL_DECL static d_string str_temp; // for d_streambuf classes
   DLL_DECL static d_binary bin_temp;

private:
   bool is_binary;

protected:
   d_stream()
      { set_char_mode(); }

   d_stream(Database* db, int ref, const_name_t tname)
      : Dyn_obj(db, ref, tname)
      { set_char_mode(); }

public:
   Database* get_db()
      { return Dyn_obj::get_db(); }
   int get_ref()
      { return Dyn_obj::get_ref(); }

   static d_ref<d_stream> openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_stream), &obj, &num_refs))
            return d_ref<d_stream>(dynamic_cast<d_stream*>(obj), num_refs);
         else
            return d_ref<d_stream>(new d_stream(db, ref, t_AbstractStream));
      }

   // Common stream interface

   void set_char_mode()
      { is_binary = false; }
   void set_bin_mode()
      { is_binary = true; }
   bool is_char_mode() const
      { return !is_binary; }

   d_binary oid()
      { return d_binary(run_obj_method(m_GetStreamId, 0, 0)); }

   d_status save()
      { return d_status(run_obj_method(m_SaveStream, 0, 0)); }

   d_status clear()
      { return d_status(run_obj_method(m_Clear, 0, 0)); }

   d_status rewind()
      { return d_status(run_obj_method(m_Rewind, 0, 0)); }

   d_status move_to_end()
      { return d_status(run_obj_method(m_MoveToEnd, 0, 0)); }

   long size()
      { return d_int(run_obj_method(m_SizeGet, 0, 0)); }

   DLL_DECL d_stream& copy(const abs_d_ref& stream);

   // Character stream interface

   DLL_DECL void read(d_int& len, d_string& res);
   DLL_DECL void readline(d_int& len, d_string& res);
   DLL_DECL void write(const d_string& data);

   // Binary stream interface

   DLL_DECL void read(d_int& len, d_binary& res);
   DLL_DECL void write(const d_binary& data);
};

//==============

class d_file_stream : protected d_stream {
protected:
   d_file_stream() {}

   d_file_stream(Database* db, int ref, const_name_t tname)
      : d_stream(db, ref, tname)
      {}

public:
   d_string get_filename()
      { return d_string(run_obj_method(m_FileName, 0, 0)); }
   void set_filename(const d_string& fname);

   d_timestamp last_modified()
      { return d_timestamp(run_obj_method(m_LastModifiedGet, 0, 0)); }

   d_status link_to_file(const_name_t fname);
};

//==============

class d_char_stream : public d_stream {
   friend class d_ref<d_char_stream>;

protected:
   d_char_stream()
      { set_char_mode(); }

   d_char_stream(Database* db, int ref, const_name_t tname)
      : d_stream(db, ref, tname)
      { set_char_mode(); }

public:
   static d_ref<d_char_stream> openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_char_stream), &obj, &num_refs))
            return d_ref<d_char_stream>(dynamic_cast<d_char_stream*>(obj), num_refs);
         else
            return d_ref<d_char_stream>(new d_char_stream(db, ref, t_GlobalCharStream));
      }

   static d_ref<d_char_stream> create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_GlobalCharStream))); }

   static d_ref<d_char_stream> open(Database* db, const d_binary& ident, int concurrency = -1)
      { return openref(db, d_int(db->open_obj(ident, concurrency))); }
};

//==============

class d_bin_stream : public d_stream {
   friend class d_ref<d_bin_stream>;

protected:
   d_bin_stream()
      { set_bin_mode(); }

   d_bin_stream(Database* db, int ref, const_name_t tname)
      : d_stream(db, ref, tname)
      { set_bin_mode(); }

public:
   static d_ref<d_bin_stream> openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_bin_stream), &obj, &num_refs))
            return d_ref<d_bin_stream>(dynamic_cast<d_bin_stream*>(obj), num_refs);
         else
            return d_ref<d_bin_stream>(new d_bin_stream(db, ref, t_GlobalBinStream));
      }

   static d_ref<d_bin_stream> create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_GlobalBinStream))); }

   static d_ref<d_bin_stream> open(Database* db, const d_binary& ident, int concurrency = -1)
      { return openref(db, d_int(db->open_obj(ident, concurrency))); }
};

//==============

class d_file_bin_stream : protected d_file_stream {
   friend class d_ref<d_file_bin_stream>;

protected:
   d_file_bin_stream()
      { set_bin_mode(); }

   d_file_bin_stream(Database* db, int ref, const_name_t tname)
      : d_file_stream(db, ref, tname)
      { set_bin_mode(); }

public:
   static d_ref<d_file_bin_stream> openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_file_bin_stream), &obj, &num_refs))
            return d_ref<d_file_bin_stream>(dynamic_cast<d_file_bin_stream*>(obj), num_refs);
         else
            return d_ref<d_file_bin_stream>(new d_file_bin_stream(db, ref, t_FileBinStream));
      }

   static d_ref<d_file_bin_stream> create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_FileBinStream))); }

   static d_ref<d_file_bin_stream> open(Database* db, const d_binary& ident, int concurrency = -1)
      { return openref(db, d_int(db->open_obj(ident, concurrency))); }
};

//==============

class d_file_char_stream : protected d_file_stream {
   friend class d_ref<d_file_char_stream>;

protected:
   d_file_char_stream()
      { set_char_mode(); }

   d_file_char_stream(Database* db, int ref, const_name_t tname)
      : d_file_stream(db, ref, tname)
      { set_char_mode(); }

public:
   static d_ref<d_file_char_stream> openref(Database* db, int ref, const_name_t tname = 0)
      {
         Obj_t* obj; int* num_refs;
         if (db->get_proxies_info()->get_proxy_info(ref, typeid(d_file_char_stream), &obj, &num_refs))
            return d_ref<d_file_char_stream>(dynamic_cast<d_file_char_stream*>(obj), num_refs);
         else
            return d_ref<d_file_char_stream>(new d_file_char_stream(db, ref, t_FileCharStream));
      }

   static d_ref<d_file_char_stream> create_new(Database* db)
      { return openref(db, d_int(db->make_obj(t_FileCharStream))); }

   static d_ref<d_file_char_stream> open(Database* db, const d_binary& ident, int concurrency = -1)
      { return openref(db, d_int(db->open_obj(ident, concurrency))); }

   d_int is_unicode()
      { return d_int(run_obj_method(m_IsFileUnicode, 0, 0)); }
};

//==============

template<typename charT, typename traits>
class d_absstreambuf : public std::basic_streambuf<charT, traits> {
private:
   d_absstreambuf(const d_absstreambuf&);
   d_absstreambuf& operator=(const d_absstreambuf&);

   typedef typename traits::char_type char_type;
   typedef typename traits::int_type int_type;

protected:
   d_ref<d_stream> device;
   char_type buf[io_buf_len];

public:
   d_absstreambuf(const d_ref<d_stream>& d)
      : device(d)
      {
         if (!(device->is_char_mode()) && (sizeof(char_type) != 1))
            throw Db_err(ERR_USER, "binary stream cannot be unicode");
      }

   d_stream& get_device()
      { return *device; }
};

//==============

template<typename charT, typename traits>
class d_ostreambuf : public virtual d_absstreambuf<charT, traits> {
private:
   d_ostreambuf(const d_ostreambuf&);
   d_ostreambuf& operator=(const d_ostreambuf&);

   typedef typename traits::char_type char_type;
   typedef typename traits::int_type int_type;

protected:
   std::streamsize xsputn(const char_type* s, std::streamsize n);
   int_type overflow(int_type c = traits::eof());
   int sync();

public:
   void reset()
      { setp(this->buf, this->buf + io_buf_len); }

   d_ostreambuf(const d_ref<d_stream>& d)
      : d_absstreambuf<charT, traits>(d)
      { reset(); }
   ~d_ostreambuf()
      { sync(); }

};

template<typename charT, typename traits>
std::streamsize d_ostreambuf<charT, traits>::xsputn(const char_type* s, std::streamsize n)
{
   if (n < this->epptr() - this->pptr())
   {
      memcpy(this->pptr(), s, n * sizeof(char_type));
      this->pbump(n);
      return n;
   }

   for (int ii = 0; ii < n; ++ii)
      sputc(*s++);

   return n;
}

template<typename charT, typename traits>
typename d_ostreambuf<charT, traits>::int_type d_ostreambuf<charT, traits>::overflow(int_type c)
{
   if (sync() != 0)
      return traits::eof();

   if (!traits::eq_int_type(c, traits::eof()))
      return sputc(c);

   return traits::eof();
}

template<typename charT, typename traits>
int d_ostreambuf<charT, traits>::sync()
{
   int cnt = this->pptr() - this->pbase();

   if (cnt > 0)
   {
      if (this->device->is_char_mode())
      {
         d_stream::str_temp.assign(this->buf, cnt);
         this->device->write(d_stream::str_temp);
      }
      else
      {
         d_stream::bin_temp.assign((char*) this->buf, cnt);
         this->device->write(d_stream::bin_temp);
      }
   }

   this->pbump(-cnt);

   return 0;
}

//==============

template<typename charT, typename traits>
class d_basic_ostream : public std::basic_ostream<charT, traits> {
private:
   d_ostreambuf<charT, traits> streambuf;

public:
   d_basic_ostream(const d_ref<d_stream>& device)
      : std::basic_ostream<charT, traits>(0), streambuf(device)
      { rdbuf(&streambuf); }

   d_binary oid()
      { return streambuf.get_device().oid(); }
   long size()
      { return streambuf.get_device().size(); }

   d_status erase()
      { streambuf.reset(); return streambuf.get_device().clear(); }
   d_status save()
      { this->flush(); return streambuf.get_device().save(); }
};

typedef d_basic_ostream< char, std::char_traits<char> > d_ostream;
typedef d_basic_ostream< wchar_t, std::char_traits<wchar_t> > d_wostream;

//==============

template<typename charT, typename traits>
class d_istreambuf : public virtual d_absstreambuf<charT, traits> {
private:
   d_istreambuf(const d_istreambuf&);
   d_istreambuf& operator=(const d_istreambuf&);

   typedef typename traits::char_type char_type;
   typedef typename traits::int_type int_type;

protected:
   std::streamsize xsgetn(char_type* s, std::streamsize n);
   int_type underflow();
   int_type pbackfail(int_type c);

public:
   void reset()
      {
         setg(this->buf + io_putback_len,
              this->buf + io_putback_len,
              this->buf + io_putback_len);
      }

   d_istreambuf(const d_ref<d_stream>& d)
      : d_absstreambuf<charT, traits>(d)
      { reset(); }
};

template<typename charT, typename traits>
std::streamsize d_istreambuf<charT, traits>::xsgetn(char_type* s, std::streamsize n)
{
   if (n < this->egptr() - this->gptr())
   {
      memcpy(s, this->gptr(), n * sizeof(char_type));
      this->gbump(n);
      return n;
   }

   for (int ii = 0; ii < n; ++ii)
      *s++ = this->sbumpc();

   return n;
}

template<typename charT, typename traits>
typename d_istreambuf<charT, traits>::int_type d_istreambuf<charT, traits>::underflow()
{
   int putback_len = min(io_putback_len, this->gptr() - this->eback());

   memmove(this->buf + (io_putback_len - putback_len),
           this->gptr() - putback_len,
           putback_len * sizeof(char_type));

   d_int len = io_buf_len - io_putback_len;

   if (this->device->is_char_mode())
   {
      this->device->read(len, d_stream::str_temp);
      if (len <= 0)
         return traits::eof();

      if (sizeof(char_type) == 1)
      {
         d_stream::str_temp.to_mb();
         len = d_stream::str_temp.get_size();
      }
      else
      {
         d_stream::str_temp.to_uni();
         len = d_stream::str_temp.get_size() / sizeof(char_type);
      }
      memcpy(this->buf + io_putback_len,
             d_stream::str_temp.get_buf(),
             len * sizeof(char_type));
   }
   else
   {
      this->device->read(len, d_stream::bin_temp);
      if (len <= 0)
         return traits::eof();
      memcpy(this->buf + io_putback_len, d_stream::bin_temp.get_buf(), len);
   }

   setg(this->buf + (io_putback_len - putback_len),
        this->buf + io_putback_len,
        this->buf + (io_putback_len + len));

   return traits::to_int_type(*(this->gptr()));
}

template<typename charT, typename traits>
typename d_istreambuf<charT, traits>::int_type d_istreambuf<charT, traits>::pbackfail(int_type c)
{
   if (this->gptr() != this->eback())
   {
      this->gbump(-1);
      *(this->gptr()) = traits::to_char_type(c);
      return traits::not_eof(c);
   }

   return traits::eof();
}

//==============

template<typename charT, typename traits>
class d_basic_istream : public std::basic_istream<charT, traits> {
private:
   d_istreambuf<charT, traits> streambuf;

public:
   d_basic_istream(const d_ref<d_stream>& device)
      : std::basic_istream<charT, traits>(0), streambuf(device)
      { rdbuf(&streambuf); }

   d_binary oid()
      { return streambuf.get_device().oid(); }
   long size()
      { return streambuf.get_device().size(); }

   d_status rewind()
      { streambuf.reset(); this->clear(); return streambuf.get_device().rewind(); }
};

typedef d_basic_istream< char, std::char_traits<char> > d_istream;
typedef d_basic_istream< wchar_t, std::char_traits<wchar_t> > d_wistream;

//==============

template<typename charT, typename traits>
class d_iostreambuf : public d_istreambuf<charT, traits>, public d_ostreambuf<charT, traits> {
private:
   d_iostreambuf(const d_iostreambuf&);
   d_iostreambuf& operator=(const d_iostreambuf&);

public:
   void reset()
      {
         d_istreambuf<charT, traits>::reset();
         d_ostreambuf<charT, traits>::reset();
      }

   d_iostreambuf(const d_ref<d_stream>& d)
      :
      d_istreambuf<charT, traits>(d),
      d_ostreambuf<charT, traits>(d),
      d_absstreambuf<charT, traits>(d)
      {}
};

//==============

template<typename charT, typename traits>
class d_basic_iostream : public std::basic_iostream<charT, traits> {
private:
   d_iostreambuf<charT, traits> streambuf;

public:
   d_basic_iostream(const d_ref<d_stream>& device)
      : std::basic_iostream<charT, traits>(0), streambuf(device)
      { rdbuf(&streambuf); }

   d_binary oid()
      { return streambuf.get_device().oid(); }
   long size()
      { return streambuf.get_device().size(); }

   d_status rewind()
      {
         this->flush(); streambuf.reset(); this->clear();
         return streambuf.get_device().rewind();
      }
   d_status move_to_end()
      {
         streambuf.reset(); this->clear();
         return streambuf.get_device().move_to_end();
      }

   d_status erase()
      { streambuf.reset(); return streambuf.get_device().clear(); }
   d_status save()
      { this->flush(); return streambuf.get_device().save(); }
};

typedef d_basic_iostream< char, std::char_traits<char> > d_iostream;
typedef d_basic_iostream< wchar_t, std::char_traits<wchar_t> > d_wiostream;

#endif
