#ifndef CPPBIND_T_FMT_H
#define CPPBIND_T_FMT_H

#include <string>

#include "dlist_fmt.h"

//==============

class in_fmt {
public:
   virtual ~in_fmt() {}

   virtual void skip_elem(std::istream& in) = 0;

   virtual void get(std::istream& in, __int64* pval, bool* null_val) = 0;
   virtual void get(std::istream& in, double* pval, bool* null_val) = 0;

   virtual void get(std::istream& in,
                    bool* is_uni,
                    char* buf, byte_size_t cap,
                    byte_size_t* psize) = 0;
   virtual void get(std::istream& in, bool* is_uni, std::string* str) = 0;
   virtual void get(std::istream& in, std::string* str)
      { str->erase(); }
};

//==============

class out_fmt {
protected:
   std::ostream* out;

public:
   out_fmt()
      : out(0)
      {}
   virtual ~out_fmt() {}

   void set_output(std::ostream* _out)
      { out = _out; }

   virtual void put(__int64 val) = 0;
   virtual void put(double val) = 0;

   virtual void put(uni_t uni, const char* buf, byte_size_t size) = 0;

   virtual void put_null() = 0;
};

//==============

class dlist_in_fmt : public in_fmt {
public:
   // in_fmt interface

   void skip_elem(std::istream& in)
      { dlist_fmt::skip_elem(in); }

   void get(std::istream& in, __int64* pval, bool* null_val)
      { dlist_fmt::get(in, pval, null_val); }
   void get(std::istream& in, double* pval, bool* null_val)
      { dlist_fmt::get(in, pval, null_val); }

   void get(std::istream& in,
                    bool* is_uni,
                    char* buf, byte_size_t cap,
                    byte_size_t* psize)
      { dlist_fmt::get(in, is_uni, buf, cap, psize); }
   void get(std::istream& in, bool* is_uni, std::string* str)
      { dlist_fmt::get(in, is_uni, str); }

   void get(std::istream& in, std::string* str);
};

//==============

class dlist_out_fmt : public out_fmt {
public:
   // out_fmt interface

   void put(__int64 val)
      { dlist_fmt::put(*out, val); }
   void put(double val)
      { dlist_fmt::put(*out, val); }

   void put(uni_t uni, const char* buf, byte_size_t len)
      { dlist_fmt::put(*out, uni, buf, len); }

   void put_null()
      { dlist_fmt::put_null(*out); }
};

//==============

class str_in_fmt : public in_fmt {
private:
   bool uni;

   void get(std::istream& in, std::string* str) {}
   // not used

public:
   str_in_fmt(bool is_uni = false)
      : uni(is_uni)
      {}

   void set_uni(bool is_uni)
      { uni = is_uni; }

   // in_fmt interface

   void skip_elem(std::istream& in)
      {}

   void get(std::istream& in, __int64* pval, bool* null_val);
   void get(std::istream& in, double* pval, bool* null_val);

   void get(std::istream& in,
            bool* is_uni,
            char* buf, byte_size_t cap,
            byte_size_t* psize)
      { *is_uni = uni; *psize = in.readsome(buf, cap); }
   void get(std::istream& in, bool* is_uni, std::string* str)
      {
         *is_uni = uni; str->erase();
         char c; while (c = in.get(), in.good()) str->append(1, c);
      }
};

#endif
