#ifndef CPPBIND_T_STRBUF_H
#define CPPBIND_T_STRBUF_H

#include <sstream>

#include "common.h"

#if !defined(SPARC) && !defined(HP)
#define __INT std::streamsize
#else
#define __INT long
#endif

// the right type for __INT is int... but compilers don't seem to support it

//==============

class t_ibuf : public std::streambuf {
public:
   t_ibuf()
      {}

   // streambuf interface

   __INT showmanyc()
      { return 0; }
};

//==============

class t_istrbuf : public t_ibuf {
public:
   t_istrbuf(const char* buf, std::streamsize size)
      { set_buf(buf, size); }
   t_istrbuf()
      {}

   void set_buf(const char* buf, std::streamsize size)
      { char* b = (char*) buf; setg(b, b, b + size); }
   void rewind()
      { setg(eback(), eback(), egptr()); }

   // streambuf interface

   __INT showmanyc()
      { return egptr() - gptr(); }

   // string interface

   const char* get_buf() const
      { return eback(); }
   std::streamsize get_size() const
      { return egptr() - eback(); }
};

//==============

class t_ostrbuf : public std::stringbuf {
public:
   void reset()
      { setp(pbase(), epptr()); }

   // string interface

   const char* get_buf() const
      { return pbase(); }
   std::streamsize get_size() const
      { return pptr() - pbase(); }
};

//==============

const int max_putback_size = 3;
const int tcp_ibuf_size = 32 * 1024;

class t_itcpbuf : public t_ibuf {
private:
   void* pm;

   UINT4 size;
   bool last_piece;
   char buf[tcp_ibuf_size];

public:
   t_itcpbuf()
      : pm(0), last_piece(false)
      {}

   void set_messenger(void* m)
      { pm = m; reset(); }

   void reset()
      {
         setg(buf + max_putback_size,
              buf + max_putback_size,
              buf + max_putback_size);
         last_piece = false;
      }
   void get_header();

   __INT showmanyc()
      { return size + egptr() - gptr(); }

   bool at_end()
      { return last_piece && (showmanyc() == 0); }

   void ignore_rest();

   std::streamsize xsgetn(char_type* s, std::streamsize n);
   int_type underflow();
   int_type pbackfail(int_type c);
};

//==============

const int header_size = 14;

class t_otcpbuf : public std::stringbuf {
private:
   void* pm;

   UINT4 get_data_size() const
      { return pptr() - pbase() - header_size; }

public:
   void set_messenger(void* m)
      { pm = m; reset(); }

   void reset()
      {
         setp(pbase(), epptr());
         for (int ii = 0; ii < header_size; ++ii) // pbump(header_size) should work
            sputc(' ');                           // but it doesn't in VC6
      }
   void send(const char* msg);
};

#endif
