#ifndef CPPBIND_UNI_CONV_H
#define CPPBIND_UNI_CONV_H

#include "common.h"

#include <wchar.h>

const int conv_buf_size = 32 * 1024;
DLL_DECL extern char conv_buf[conv_buf_size];
DLL_DECL extern char copy_buf[conv_buf_size];

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus
   int ODBC_DLL UnicodeToCacheUni(
      const wchar_t * lpWideCharStr,  // address of buffer for new string
      int cchWideChar,                // size of buffer in Unicode characters
      unsigned short *lpCacheChar,    // address of Cache wide-character string
      size_t cchCacheChar             // number of bytes in Cache Unicode string
      );

   int ODBC_DLL CacheUniToUnicode(
      const unsigned char* lpChar, // address of wide-character string
      size_t cchCacheChar,         // number of bytes in string
      wchar_t* lpWideCharStr,      // address of buffer for new string
      size_t cchWideChar           // size of buffer in Unicode characters
      );

   int ODBC_DLL CacheUniToMultiByte(
      const unsigned char* lpChar, // address of wide-character string
      size_t cchCacheChar,         // number of bytes in string
      char* lpMultiByteStr,        // address of buffer for new string
      size_t cchMultiByte          // size of buffer in bytes
      );

   int ODBC_DLL UnicodeToMultiByte(
      const wchar_t* lpWideCharStr, // address of wide-character string
      int cchWideChar,              // number of Unicode characters in string
      char* lpMultiByteStr,         // address of buffer for new string
      size_t cchMultiByte           // size of buffer in bytes
      );

   int ODBC_DLL MultiByteToUnicode(
      const char* lpMultiByteStr,    // address of string to map
      int cchMultiByte,              // number of bytes in string
      wchar_t * lpWideCharStr,       // address of wide-character buffer
      size_t cchWideChar             // size of buffer in Wide Characters
      );
#ifdef __cplusplus
}
#endif // _#ifdef __cplusplus

DLL_DECL void _cache_uni_to_uni(const char* src_buf, byte_size_t src_size,
                                const wchar_t** dest_buf, char_size_t* dest_size);

inline void _cache_uni_to_mb(const char* src_buf, byte_size_t src_size,
                            const char** dest_buf, byte_size_t* dest_size)
{
   *dest_size = CacheUniToMultiByte((const unsigned char*) src_buf, src_size,
                                    conv_buf, conv_buf_size);
   *dest_buf = conv_buf;
}

DLL_DECL void _uni_to_cache_uni(const wchar_t* src_buf, char_size_t num_chars,
                                const char** dest_buf, byte_size_t* dest_size);

DLL_DECL void _mb_to_uni(const char* src_buf, byte_size_t src_size,
                         wchar_t** dest_buf, char_size_t* dest_size);

DLL_DECL void _uni_to_mb(const wchar_t* src_buf, char_size_t src_size,
                         char** dest_buf, byte_size_t* dest_size);

DLL_DECL int mb_to_uni(const char* src_buf, byte_size_t src_size,
                       wchar_t* dest_buf, char_size_t dest_cap);

DLL_DECL int uni_to_mb(const wchar_t* src_buf, char_size_t src_size,
                       char* dest_buf, byte_size_t dest_cap);

DLL_DECL char* cppbind_setlocale(int category, const char *locale);

#endif
