#include "../../Sample_Person.h"
#include "../../Sample_Address.h"

#include <iostream>
#include <algorithm>

class Print_color : public std::unary_function<d_string, int> {
private:
   std::ostream& out;
public:
   explicit Print_color(std::ostream& o)
      : out(o)
      {}

   result_type operator()(argument_type p) const
      { out << p << std::endl; return 0; }
};

int main()
{
   // establish the physical connection to Cache'
   Db_err conn_err;
   d_connection conn = tcp_conn::connect("localhost[1972]:Samples",
                                         "_SYSTEM", "SYS", &conn_err);
   if (conn_err)
   {
      std::cerr << conn_err << '\n';
      return -1;
   }

   try {
      // establish the logical connection to Cache'
      Database db(conn);

      // Open an instance of the Sample.Person object
      d_ref<Sample_Person> person = Sample_Person::openid(&db, L"1");

      // Fetch some properties
      std::cout << "Name: " << person->getName() << '\n'
                << "SSN:  " << person->getSSN() << '\n';

      d_ref< d_prim_vector<d_string> > colors = person->getFavoriteColors();

      // Iterate over the FavoriteColors collection
      std::for_each(colors->begin(), colors->end(), Print_color(std::cout));

      std::cout << "Modifying 'FavoriteColors' ...\n";

      // Insert a new element
      colors->push_back(d_string("Orange"));

      // Remove the first element
      colors->erase(colors->begin());

      // Show the changes to the collection
      std::for_each(colors->begin(), colors->end(), Print_color(std::cout));

      return 0;

      // all objects are closed automatically
   }
   catch(const Db_err& err) {
      std::cerr << err << '\n';
      return -1;
   }
}
