/*	cachedblib.h - Cach DBLib header file

	Copyright (c) 2002-2003 by InterSystems Corporation
	Cambridge, Massachusetts, U.S.A.  All rights reserved.
	Confidential, unpublished property of InterSystems.

Modification History: (most recent first)

6/24/2003 Aleksandar Djakovic, add dblogout
2/07/2003 Aleksandar Djakovic, initial version

*****************************************************************************/

#ifndef _cachedblib_h_
#define _cachedblib_h_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef	WIN32
#define DBLIB_API  __stdcall
#else
#define DBLIB_API
#endif

#define CS_PUBLIC DBLIB_API

#if defined( __alpha) || defined(_AIX)
typedef	int				DBINT;
#else
typedef	long			DBINT;
#endif
typedef	unsigned char	DBBOOL;
typedef unsigned char   DBTINYINT;	// 1 byte int
typedef short           DBSMALLINT;	// 2 byte int
typedef char			DBCHAR;		// char type
typedef unsigned char   DBBINARY;	// binary type
typedef unsigned char   DBBIT;		// bit type
typedef float			DBREAL;		// 4 byte float type
typedef	double			DBFLT8;		// float type

#define	FAIL			0
#define	SUCCEED			1

typedef int             RETCODE;    // SUCCEED or FAIL

#define	INT_EXIT		0
#define	INT_CONTINUE	1
#define	INT_CANCEL		2
#define	INT_TIMEOUT		3

// Severity levels
#define	EXINFO			1	// informational, non-error
#define	EXUSER			2	// user error
#define	EXNONFATAL		3	// non-fatal error
#define EXCONVERSION	4	// conversion error
#define EXSERVER		5	// server has returned an error flag
#define EXTIME			6	// timeout error (DBPROCESS is still alive)
#define EXPROGRAM		7	// coding error in user program
#define	EXRESOURCE		8	// out of resources (DBPROCESS may be dead)
#define	EXCOMM			9	// communication error (DBPROCESS is dead)
#define	EXFATAL			10	// fatal error (DBPROCESS is dead)
#define	EXCONSISTENCY	11	// internal software error

#define DBSAVE			1
#define DBNOSAVE		0
#define DBNOERR			-1
 
#define DBMAXNUMLEN		33
#define MAXNAME			30

#define DBSETHOST       1
#define DBSETUSER       2
#define DBSETPWD        3
#define DBSETAPP        5
#define DBSETPORT       8		// InterSystems extension
#define DBSETNAMESPACE  9		// InterSystems extension


#define SYBCHAR			47
#define SYBVARCHAR		39
#define SYBINTN			38
#define SYBINT1			48
#define SYBINT2			52
#define SYBINT4			56
#define SYBINT8			127
#define SYBFLT8			62
#define SYBDATETIME		61
#define SYBBIT			50
#define SYBTEXT			35
#define SYBREAL			59
#define SYBBINARY		45
#define SYBVOID			31
#define SYBVARBINARY	37
#define SYBNVARCHAR		103
#define SYBBITN			104
#define SYBNUMERIC		108
#define SYBDECIMAL		106
#define SYBFLTN			109

#define NOBIND				-1
#define TINYBIND			1
#define SMALLBIND			2
#define INTBIND				3
#define CHARBIND			4
#define TEXTBIND			5
#define BINARYBIND			6
#define ARRAYBIND			7
#define BITBIND				8
#define DATETIMEBIND		9
#define MONEYBIND			10
#define FLT8BIND			11
#define STRINGBIND			12
#define NTBSTRINGBIND		13
#define VARYCHARBIND		14
#define VARYBINBIND			15
#define SMALLDATETIMEBIND   16
#define SMALLMONEYBIND		17
#define REALBIND			18
#define NUMERICBIND			19
#define DECIMALBIND			20
#define SENSITIVITYBIND		21
#define BOUNDARYBIND		22
#define MAXBIND				BOUNDARYBIND

#define REG_ROW				-1
#define MORE_ROWS			-1
#define NO_MORE_ROWS		-2
#define BUF_FULL			-3
#define NO_MORE_RESULTS		2

typedef unsigned char BYTE;

typedef void * LOGINREC;
typedef void * DBPROCESS;

typedef	int	(DBLIB_API * MESSAGE_HANDLER) (DBPROCESS*,DBINT,int,int,char*,char*,char*,int);
typedef int	(DBLIB_API * ERROR_HANDLER)	(DBPROCESS*,int,int,int,char*,char*);

// Supported apis
int			DBLIB_API	dbinit();
int			DBLIB_API	dbsetlname(LOGINREC *loginrec, char *name, int type);
LOGINREC *	DBLIB_API	dblogin();
DBPROCESS *	DBLIB_API	dbopen(LOGINREC *login,char *server);
int			DBLIB_API	dbclose(DBPROCESS *dbprocess);
void		DBLIB_API	dbexit();
int			DBLIB_API	dbsetmaxprocs(int maxprocs);
int			DBLIB_API	dbgetmaxprocs();
int			DBLIB_API	dbsettime(int seconds);
int			DBLIB_API	dbsetlogintime(int seconds);
int			DBLIB_API	dbresults(DBPROCESS *dbproc);
int			DBLIB_API	dbnextrow(DBPROCESS *dbproc);
int			DBLIB_API	dbdatlen(DBPROCESS *dbproc, int column);
int			DBLIB_API	dbcoltype(DBPROCESS *dbproc,int column);
int			DBLIB_API	dbcancel(DBPROCESS *dbproc);
void		DBLIB_API	dbloginfree(LOGINREC *login);
void		DBLIB_API	dbsetuserdata(DBPROCESS *dbproc, BYTE * ptr);
BYTE *		DBLIB_API	dbgetuserdata(DBPROCESS *dbproc);
void		DBLIB_API	dbfreebuf(DBPROCESS *dbproc);
int			DBLIB_API	dbcmd(DBPROCESS *dbproc, char *cmdstring);
int			DBLIB_API	dbfcmd(DBPROCESS * dbproc, char * cmdstring, ...);
int			DBLIB_API	dbsqlexec(DBPROCESS *dbproc);
int			DBLIB_API	dbnumcols(DBPROCESS *dbproc);
int			DBLIB_API	dbbind(DBPROCESS *dbproc, int column, int vartype, DBINT varlen, BYTE *varaddr);
DBBOOL		DBLIB_API	dbdead(DBPROCESS * dbproc);
int			DBLIB_API	dbstrlen(DBPROCESS * dbproc);
int			DBLIB_API	dbstrcpy(DBPROCESS * dbproc, int start, int numbytes, char * dest);
DBINT		DBLIB_API	dbcollen(DBPROCESS *dbproc, int column);
BYTE *		DBLIB_API	dbdata(DBPROCESS *dbproc,int column);
char *		DBLIB_API	dbcolname(DBPROCESS *dbproc,int column);
int			(* DBLIB_API dbmsghandle(int (*handler)(DBPROCESS*,DBINT,int,int,char*, char*,char*,int))) 
					(DBPROCESS*,DBINT,int,int,char*, char*,char*,int);
int			(* DBLIB_API dberrhandle(int (*handler)(DBPROCESS*,int,int,int,char*,char*))) 
					(DBPROCESS*,int,int,int,char*,char*);
int			DBLIB_API	dbcanquery(DBPROCESS *dbproc);
int			DBLIB_API	dbuse(DBPROCESS *dbproc,char *dbname);
DBINT		DBLIB_API	dbretstatus(DBPROCESS *dbproc);
int			DBLIB_API	dbcmdrow(DBPROCESS * dbproc);
int			DBLIB_API	dbrows(DBPROCESS * dbproc);

void		DBLIB_API	dblogout(LOGINREC * login);	// InterSystems proprietary api

int			DBLIB_API	dbsetversion(DBINT version);
int			DBLIB_API	dbgetpacket(DBPROCESS * dbproc);
int			DBLIB_API	dbconvert(DBPROCESS * dbproc, int srctype, BYTE * src, 
							      int srclen, int desttype, BYTE * dest, int destlen);
DBBOOL		DBLIB_API	dbhasretstat(DBPROCESS * dbproc);
char *		DBLIB_API	dbname(DBPROCESS * dbproc);
void		DBLIB_API	dbrecftos(char * filename);
int			DBLIB_API	dbsetllong(LOGINREC * login, long value, int which);
void		DBLIB_API	dbprhead(DBPROCESS * dbproc);   
void		DBLIB_API	dbprrow(DBPROCESS * dbproc);  

#define		DBSETLPACKET(x,y) dbsetllong((x),(y),DBSETPACKET)

#ifdef __VMS
int		DBLIB_API	dbsqlexec_a(DBPROCESS * dbproc,int * final_result, 
								void (*ast_proc)(),BYTE * ast_param);
int		DBLIB_API	dbresults_a(DBPROCESS * dbproc, int * final_result, 
								void (*ast_proc )(),BYTE * ast_param);
int		DBLIB_API	dbnextrow_a(DBPROCESS * dbproc, int * final_result,
								void (* ast_proc)(),BYTE * ast_param);
int		DBLIB_API	dbcanquery_a(DBPROCESS * dbproc, int * final_result, 
								void (*ast_proc)(), BYTE * ast_param);
#endif	// __VMS

// Supported macros
#define DBCMDROW(x)			dbcmdrow(x)
#define DBROWS(x)			dbrows(x)
#define DBDEAD(a)			dbdead(a)
#define DBSETLHOST(a,b)		dbsetlname((a), (b), DBSETHOST)
#define DBSETLUSER(a,b)		dbsetlname((a), (b), DBSETUSER)
#define DBSETLPWD(a,b)		dbsetlname((a), (b), DBSETPWD)
#define DBSETLAPP(a,b)		dbsetlname((a), (b), DBSETAPP)
#define DBSETLPORT(a,b)		dbsetlname((a), (b), DBSETPORT)		// InterSystems extension
#define DBSETLNAMESPACE(a,b)		dbsetlname((a), (b), DBSETNAMESPACE)		// InterSystems extension

// Supported errors
#define SYBEFCON        20002   // connection failed
#define SYBETIME        20003   // connection timed out
#define SYBEMEM         20010   // insufficient memory
#define SYBEDBPS        20011   // maximum number of DBPROCESSes already allocated
#define SYBEPWD         20014   // invalid password
#define SYBECNOR        20026   // column number out of range
#define SYBEABNC        20032   // attempt to bind to a non-existent column
#define SYBEABNP        20034   // attempt to bind using NULL pointers
#define SYBENSIP        20045   // negative starting index passed to dbstrcpy()
#define SYBECLPR        20051   // conversion resulted in loss of precision
#define SYBESMSG        20018   // General SQL Server error: Check messages

#ifndef NULL
#define NULL    0
#endif

#ifdef __cplusplus
}
#endif
 
#endif

