/*	DBLibTest.cpp - Cach DBLib Driver Test Suite

	Copyright (c) 2002 InterSystems, Corp.
	Cambridge, Massachusetts, U.S.A.  All rights reserved.
	Confidential, unpublished property of InterSystems.

Modification History: (most recent first)

JCN523 10/19/04 Jim Nelson, Print any datatype as is done in VMS when passed parameters
JCN444 10/09/03 Jim Nelson, Add VMS ast code to dblibtest
JCN442 10/09/03 Jim Nelson, Pass arguments for connection and SQL statement execution
AND291 02/07/03 Aleksandar Djakovic, consolidate header files
AND290 01/30/03 Aleksandar Djakovic, various DBLib fixes (on site at SEMA)
AND268 11/20/02 Aleksandar Djakovic, DBLib test driver
JCN349 12/18/02 Jim Nelson, Use CS_PUBLIC Macro for VMS support

***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <string>

#include "cachedblib.h"

using std::string;

// Message handler
int CS_PUBLIC msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, 
				char * msgtext, char * srvname, char * procname, int line) {
    printf ("Msg %ld, Level %d, State %d\n",msgno, severity, msgstate);
	if (srvname) {
		printf ("Server '%s', ", srvname);
	}
	if (procname) {
		printf ("Procedure '%s', ", procname);
	}
	if (line > 0) {
		printf ("Line %d", line);
	}
	printf("\n\t%s\n", msgtext);
	return(0);
} 

// Error handler
int CS_PUBLIC err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr,	char * dberrstr,
				char * oserrstr) {
	if ((dbproc == NULL) || (DBDEAD(dbproc))) {
		return(INT_EXIT);
	} else {
		printf("DB-Library error:\n\t%s\n",dberrstr);
		if (oserr != DBNOERR) {
			printf("Operating-system error:\n\t%s\n",oserrstr);
			return(INT_CANCEL);
		}
	} 
	return(INT_EXIT);
}
	
// Parse command line string
int ParseConnectionString(string & c_string, LOGINREC *login)
{
	string	item;
	string	temp;
	string	result;
	size_t				pos;   
	size_t				i;     

#if 0  //JCN462-
	while ((pos = c_string.find_first_of(";")) != string::npos) {
		temp.assign(c_string,0,pos);
		c_string.assign(c_string,pos+1,c_string.length());
#else  //JCN462+
	while ((pos = c_string.find_first_of(";")) && (c_string.length()!=0) ) {
		if (pos == string::npos ) { 
			pos = c_string.length(); 
			temp.assign(c_string,0,pos);
			c_string.assign("");
		} else {
			temp.assign(c_string,0,pos);
			c_string.assign(c_string,pos+1,c_string.length());
		}
#endif  //JCN462=
		i = temp.find_first_of("=");
		item.assign(temp,0,i);
		result.assign(temp,i+1,temp.length()-i);
		if (item == "HOST") {
			DBSETLHOST(login,(char *)result.c_str());
		} else if (item == "PORT") {
			DBSETLPORT(login,(char *)result.c_str());
		} else if (item == "NAMESPACE") {
			DBSETLNAMESPACE(login,(char *)result.c_str());
		} else if (item == "USER") {
			DBSETLUSER(login,(char *)result.c_str());
		} else if (item == "PWD") {
			DBSETLPWD(login,(char *)result.c_str());
		} else if (item == "APP") {
			DBSETLAPP(login,(char *)result.c_str());
		} else {
			printf("\n\n\tdblibtest [arg1] [arg2]");
			printf("\n\targ1: HOST=127.0.0.1;PORT=1972;NAMESPACE=SAMPLES;USER=_SYSTEM;PWD=sys");
			printf("\n\targ2: SQL Statement to execute ie.(\"Select * from sample.person\")\n");

			return 1 ;
		}
	}
	return 0;
}

#if defined(__VMS) && defined(ASYNCH)

	int firstrow;
		struct ASTSTRUCT {
                DBPROCESS * dbproc;
                RETCODE  result;
		} ; 

//prototypes
void proc_resultast(ASTSTRUCT *astr) ;


void process_row(ASTSTRUCT * astr) {
	DBCHAR                  name[50];
	DBCHAR                  dob[12];
	DBCHAR                  state[3];
	DBCHAR                  spouse[50];
	DBCHAR			printfline[1024];
	DBCHAR			colname[50];
	DBCHAR			strval[1024];
	RETCODE			result;
	//DBPROCESS * dbproc=(DBPROCESS*)ast_param;

	if (astr->result!=-1) {

		printf("call to dbnext_row returned async error\n");
		result=dbresults_a(astr->dbproc,&(astr->result),(void (*)())proc_resultast,(BYTE *)astr);
		return;
	}
	strcpy(printfline,"");
	int numcols=dbnumcols(astr->dbproc);
	int colnum;
	int coltype;
	int intval;
	DBINT currow;
	//currow = DBCURROW(astr->dbproc);
	//printf("Current row is %d",currow);
	DBTINYINT tinyintval;
	for (colnum=1;colnum<=numcols;++colnum) {
		coltype=dbcoltype(astr->dbproc,colnum);
		strcpy(colname,(char *)dbcolname(astr->dbproc,colnum));
		if (firstrow==1) {
			printf("column %s is of type %d\n",colname,coltype);
		}
		strcat(printfline,(char *)colname);
		strcat(printfline,"=");
		if (coltype==48) {
			tinyintval=*((int*) dbdata(astr->dbproc,colnum));
			sprintf(strval,"%d",tinyintval);
			strcat(printfline,strval);
		}
                if (coltype==56) {
                        intval=*((int*) dbdata(astr->dbproc,colnum));
                        sprintf(strval,"%d",intval);
                        strcat(printfline,strval);
                }
		else {
			int len = (int) dbdatlen(astr->dbproc,colnum);
			if (len != -1) {strcat(printfline,(char *) dbdata(astr->dbproc,colnum));}
			else {strcat(printfline,"<NULL>");}
		}
		if (colnum<=numcols-1) {strcat(printfline,",");}
	}
	strcat(printfline,"\n\n");
	printf(printfline);
	firstrow=0;

    result=dbnextrow_a(astr->dbproc,&(astr->result),(void (*)())process_row,(BYTE *)astr);
 	if (result!=SUCCEED){
		printf("failure to call nextrow_a from process_rwo");
	        result = dbresults_a(astr->dbproc,&(astr->result),(void(*)())proc_resultast,(BYTE *)astr);
		if (result != SUCCEED){printf("failure to call dbresults_a in process_row");}
		}
	return;	

}
 

void proc_resultast(ASTSTRUCT *astr) {
	//come here if the dbresults_a ast is fired
       if (astr->result == SUCCEED) {
                printf("call to dbresults_a succeeded\n");
                RETCODE result=0;
                result=dbnextrow_a(astr->dbproc,&(astr->result),(void (*)())process_row,(BYTE *)astr);
		printf("dbnextrow_a result= %d \n",result);
	}
	else {
		printf("call to db_results_a failed");
	}
	return;
}

void proc_execast(ASTSTRUCT * astr) {
        // come here if the exec_a ast is fired
	
	printf("process results\n");
        int return_code;
	if (astr->result == FAIL){
		printf("process results FAIL\n");
		return;
	} else
	{
	printf("process results SUCCEED\n");
        return_code = dbresults_a(astr->dbproc,&(astr->result),(void(*)())proc_resultast,(BYTE *)astr);
        if (return_code == FAIL) {
		printf("call to dbresults_a failed");
          	}              
	}
	return;
}

int main(int argc, char* argv[]) {
	DBPROCESS	*	dbproc;
	RETCODE			result;
	LOGINREC	*	login;
	struct ASTSTRUCT	*	astr;
	int			return_code;
	int			len;
	int			rc;
	int			col;
	char		*	sqlcmd;

	printf("DBLib Test Driver\n");
	firstrow=1;
	if (argc<2){
		printf("supply SQL statment as argument on command line");
		return 1;
		}
	sqlcmd=argv[1];
	printf("num params= %d \n",argc);
	for (int i=0;i<argc;i++){
		printf(argv[i]);
		}
//	dbproc=(DBPROCESS * )malloc(sizeof(struct DBPROCESS));
	astr=(ASTSTRUCT *)malloc(sizeof(struct ASTSTRUCT));
	astr->dbproc=dbproc;
	astr->result=result;

	
	// Initialize DB-Library
        if (dbinit() == FAIL) {
		dbexit();
	}
	// Install message and error handlers
	dberrhandle(&err_handler); 
	dbmsghandle(&msg_handler);
	
	// Get a LOGINREC, then fill it with the connection parameters
	login = dblogin();
	DBSETLPWD(login,"sys");
	DBSETLAPP(login,"SAMPLES");
	DBSETLHOST(login,"127.0.0.1");
	DBSETLUSER(login,"_SYSTEM");
	
	// Get a DBPROCESS structure for communication with SQL Server
	astr->dbproc = dbopen(login,0);
	rc = dbuse(astr->dbproc,"SAMPLES");
	dbloginfree(login);

	// Put the command into the command buffer

	rc = dbcmd(astr->dbproc,sqlcmd);


	// test dbstrcpy and dbstrlen
	int sqllen = dbstrlen(astr->dbproc);
	char * sql = new char[sqllen];
	rc = dbstrcpy(astr->dbproc,0,sqllen,sql);
	sql[sqllen] = '\0';
	printf("SQL Text: %s\n\n",sql);

	RETCODE fin_res=0;
	rc = dbsqlexec_a(astr->dbproc,&(astr->result),(void (*)())proc_execast,(BYTE *)astr);

	if (rc==FAIL) {printf ("Failure calling dbssqlexec_a\n");}
	else {printf("Success calling dbsqlexec_a\n");}

	DBINT retstatus;
	retstatus = dbretstatus(astr->dbproc);
	int retstatusint;
	retstatusint=(int) retstatus;
	
	printf("retstatus =  %d\n",retstatusint);

	// Close the connection
	dbclose(astr->dbproc);
	dbexit();
	return 0;
}

#else

#if WIN32
int __cdecl main(int argc, char* argv[]) {
#else
int main(int argc, char* argv[]) {
#endif
	DBPROCESS	*	dbproc;
	LOGINREC	*	login;
	BYTE 			id;
	DBCHAR			name[50]; 
	DBCHAR			ssn[12];
	DBCHAR			dob[12];
	DBCHAR			favorite_colors[100];
	DBCHAR			city[50];
	DBCHAR			state[3];
	DBCHAR			spouse[50];
	DBCHAR			stored_proc_data[100][100];
	int				return_code;
	int				len;
	int				rc;
	int				col;

	printf("DBLib Test Driver\n");

	// Initialize DB-Library
    if (dbinit() == FAIL) {
		printf("\n\n\tFailed to initialize DB-Library");
		dbexit();
		return -1;
	}
	// Install message and error handlers
	dberrhandle(&err_handler); 
	dbmsghandle(&msg_handler);
	
	dbsetlogintime(1);
	dbsettime(1);

	// Get a LOGINREC, then fill it with the connection parameters
	login = dblogin();
	DBSETLPWD(login,"SYS");
	DBSETLAPP(login,"SAMPLES");
	DBSETLHOST(login,"127.0.0.1");
	DBSETLUSER(login,"_SYSTEM");
	DBSETLPORT(login,"1972");
	// Override connection parameters with argv[1]
	if (argc>1) {
		// Print out connection string for debug purposes
		if (argc==2) {
			printf("\n\n\tCommand-line entered: \nargv[0]: %s \nargv[1]: \"%s\"\n\n",argv[0], argv[1]);
		} else if (argc==3) {
			printf("\n\n\tCommand-line entered: \nargv[0]: %s \nargv[1]: \"%s\" \nargv[2]: \"%s\"\n\n",argv[0], argv[1], argv[2]);
		} else {
			printf("\n\n\tCommand-line entered: \nargv[0]: %s \nargv[1]: \"%s\" \nargv[2]: \"%s\" \nargv[3]: \"%s\"\n\n",argv[0], argv[1], argv[2], argv[3]);
		} 
		string connectstring(argv[1]);
		if (ParseConnectionString(connectstring, login)) {
			dbexit();
			return -1;
		}
	}

	// Get a DBPROCESS structure for communication with SQL Server
	dbproc = dbopen(login,0);
	rc = dbuse(dbproc,"USER");  // This is currently a noop
	dbloginfree(login);

	// Test dbgetmaxprocs/dbsetmaxprocs
	printf("dbgetmaxprocs: %d\n",dbgetmaxprocs());
	dbsetmaxprocs(11);
	printf("changed dbgetmaxprocs to: %d\n\n",dbgetmaxprocs());

	// Put the command into the command buffer
	if (argc>2) {
		rc = dbcmd(dbproc,argv[2]);
	} else {
		rc = dbcmd(dbproc,"select ID, Name, SSN, DOB, FavoriteColors, Home_City, Home_State, Spouse ");
		rc = dbfcmd(dbproc,"from Sample.Person where ID < %d",10);
	}
	//rc = dbcmd(dbproc,"call Sample.SP_Sample_By_Na");
	//rc = dbfcmd(dbproc,"me(%s)","A");	
	// test dbstrcpy and dbstrlen
	int sqllen = dbstrlen(dbproc);
	char * sql = new char[sqllen];
	rc = dbstrcpy(dbproc,0,sqllen,sql);
	sql[sqllen] = '\0';
	printf("SQL Text: %s\n\n",sql);

	// Send the command to the server for execution
	rc = dbsqlexec(dbproc);
//	if (rc != 1) {
//		printf("Fatal error!\n");
//		return -1;
//	}
	int columns = dbnumcols(dbproc);
	printf("Number of columns: %d\n",columns);

	for ( col=1;col<=columns;col++) {
		printf("Column %d: Name = %s ",col,dbcolname(dbproc,col));
		printf("Length = %d ",dbcollen(dbproc,col));
		printf("Type = %d\n",dbcoltype(dbproc,col));
	}
	printf("\n");
#if 1  //JCN523+
	// Handle reporting variable types of data
	if (argc>2) { 
		DBCHAR			printfline[2048];
		DBCHAR			colname[50];
		DBCHAR			strval[1024];

		strcpy(printfline,"");
		int numcols=dbnumcols(dbproc);
		int colnum;
		int coltype;
		int intval;
		DBTINYINT tinyintval;
		// Process results
		while ((return_code = dbresults(dbproc)) != NO_MORE_RESULTS) {                       
			if (return_code == SUCCEED) {
				// Retrieve and print the result rows
				while (dbnextrow(dbproc) != NO_MORE_ROWS) { 
					// use dldatlen and dbdata on the rest of the columns
					for (colnum=1;colnum<=numcols;++colnum) {
						coltype=dbcoltype(dbproc,colnum);
						strcpy(colname,(char *)dbcolname(dbproc,colnum));
						strcat(printfline,(char *)colname);
						strcat(printfline,"=");
						if (coltype==48) {
							tinyintval=*((int*) dbdata(dbproc,colnum));
							sprintf(strval,"%d",tinyintval);
							strcat(printfline,strval);
						}
						if (coltype==56) {
							intval=*((int*) dbdata(dbproc,colnum));
						    sprintf(strval,"%d",intval);
						    strcat(printfline,strval);
						} else {
							int len = (int) dbdatlen(dbproc,colnum);
							if (len != -1) 
							{
								strcat(printfline,(char *) dbdata(dbproc,colnum));
							} else {
								strcat(printfline,"<NULL>");
							}
						}
						if (colnum<=numcols-1) {strcat(printfline,",");}
					}
					strcat(printfline,"\n\n");
				}
			} else {
				break;
			}
			printf(printfline);
		}
		goto end;
	} else {
#endif  //JCN523=
		// Process results
		while ((return_code = dbresults(dbproc)) != NO_MORE_RESULTS) {                       
			if (return_code == SUCCEED) {
				// Bind some columns
			//	dbbind(dbproc,1,INTBIND,(DBINT)0,(BYTE *)&id);
				dbbind(dbproc,5,STRINGBIND,(DBINT)101,(unsigned char *)favorite_colors);
				dbbind(dbproc,3,STRINGBIND,(DBINT)12,(unsigned char *)ssn);
				dbbind(dbproc,6,STRINGBIND,(DBINT)50,(unsigned char *)city);
				// Retrieve and print the result rows
				while (dbnextrow(dbproc) != NO_MORE_ROWS) { 
					// use dldatlen and dbdata on the rest of the columns
					len = (int) dbdatlen(dbproc,1);
					if (len != -1) {
						id = * (unsigned char *)dbdata(dbproc,1); 
					}
					len = (int) dbdatlen(dbproc,8);
					if (len != -1) {
						strncpy(spouse,(char *)dbdata(dbproc,8),len); 
						spouse[len] = '\0';
					} else {
						strcpy(spouse,"<NULL>");
					}
					len = (int) dbdatlen(dbproc,2);
					if (len != -1) {
						strncpy(name,(char *)dbdata(dbproc,2),len); 
						name[len] = '\0';
					} else {
						strcpy(name,"<NULL>");
					}
					len = (int) dbdatlen(dbproc,7);
					if (len != -1) {
						strncpy(state,(char *)dbdata(dbproc,7),len); 
						state[len] = '\0'; 
					} else {
						strcpy(state,"<NULL>");
					}
					len = (int) dbdatlen(dbproc,4);
					if (len != -1) {
						strncpy(dob,(char *)dbdata(dbproc,4),len); 
						dob[len] = '\0'; 
					} else {
						strcpy(dob,"<NULL>");
					}
					if (!strlen((const char *) favorite_colors)) {
						strcpy(favorite_colors,"<NULL>");
					}
					if (!strlen((const char *) ssn)) {
						strcpy(ssn,"<NULL>");
					}
					if (!strlen((const char *) city)) {
						strcpy(city,"<NULL>");
					}
					printf("%s = %d, %s = %s, %s = %s, %s = %s, \n    %s = %s, %s = %s, %s = %s, %s = %s \n\n",
						dbcolname(dbproc,1),id,dbcolname(dbproc,2),name,dbcolname(dbproc,3),ssn,
						dbcolname(dbproc,4),dob,dbcolname(dbproc,5),favorite_colors,
						dbcolname(dbproc,6),city,dbcolname(dbproc,7),state,
						dbcolname(dbproc,8),spouse);
				}
			} else {
				break;
			}
		}
#if 1  //JCN523+
	}
#else  //JCN523-
	if (argc>2) { goto end;}  //
#endif  //JCN523=

	rc = dbfcmd(dbproc,"exec Sample.Stored_Procedure_Test \"%s\",@response=%d","sasa",881);
	// test dbstrcpy and dbstrlen
	sqllen = dbstrlen(dbproc);
	sql = new char[sqllen];
	rc = dbstrcpy(dbproc,0,sqllen,sql);
	sql[sqllen] = '\0';
	printf("SQL Text: %s\n\n",sql);

	// Send the command to the server for execution
	rc = dbsqlexec(dbproc);
	if (rc != 1) {
		printf("Fatal error!\n");
		return -1;
	}
	columns = dbnumcols(dbproc);
	printf("Number of columns: %d\n",columns);

	rc = dbcmd(dbproc,"exec Sample.SP_Sample_By_Na");
	rc = dbfcmd(dbproc,"me @name=\"%s\"","A");	
	// test dbstrcpy and dbstrlen
	sqllen = dbstrlen(dbproc);
	sql = new char[sqllen];
	rc = dbstrcpy(dbproc,0,sqllen,sql);
	sql[sqllen] = '\0';
	printf("SQL Text: %s\n\n",sql);

	// Send the command to the server for execution
	rc = dbsqlexec(dbproc);
	if (rc != 1) {
		printf("Fatal error!\n");
		return -1;
	}
	columns = dbnumcols(dbproc);
	printf("Number of columns: %d\n",columns);

	for (col=1;col<=columns;col++) {
		printf("Column %d: Name = %s ",col,dbcolname(dbproc,col));
		printf("Length = %d ",dbcollen(dbproc,col));
		printf("Type = %d\n",dbcoltype(dbproc,col));
		dbbind(dbproc,col,STRINGBIND,(DBINT)101,(unsigned char *)stored_proc_data[col-1]);
	}
	printf("\n");
	// Process results
	while ((return_code = dbresults(dbproc)) != NO_MORE_RESULTS) {                       
		if (return_code == SUCCEED) {
			while (dbnextrow(dbproc) != NO_MORE_ROWS) { 
				for (int col=1;col<=columns;col++) {
					printf("%s = %s  ",dbcolname(dbproc,col),stored_proc_data[col-1]);
				}
				printf("\n");
			}
		}
	}
end:
	dbfreebuf(dbproc);

	// Close the connection
	dbclose(dbproc);
	dblogout(login);  //This is also called by dbexit()
	dbexit();
	return 0;
}

#endif

