/* ====================================================================
 * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission.
 *
 * 5. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Group and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Group and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */

/* 
 * Apache weblink module.
 *
 */


/* Operating System type */

#if defined(_WIN32) /* Windows */

#ifndef MGW_WIN32
#define MGW_WIN32                1
#endif

#elif defined(VMS) || defined(_VMS) || defined(__VMS) || defined(MGW_VMS) /* VMS */

#ifndef MGW_VMS
#define MGW_VMS                  1
#endif

#else /* UNIX */

#ifndef MGW_UNIX
#define MGW_UNIX                 1
#endif

#endif


#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "util_script.h"

/* Apache Version 2.x */

#if !defined(APACHE_RELEASE)

#include "http_request.h"
#include "http_connection.h"
#include "apr_strings.h"
/*
#include "apr_buckets.h"
*/
/*
#include "mod_core.h"
*/
#define APACHE_RELEASE 2000000

#endif


#if defined(_WIN32)

#include <tchar.h>
#include <windows.h>
#include <winsock.h>
#include <fcntl.h>
#include <io.h>
#ifdef MGW_LOG
#include <process.h>
#endif

#elif defined(MGW_UNIX)

#include <sys/signal.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>
#endif

#define SA_IP_ADDRESS      "127.0.0.1"
#define SA_TCP_PORT        "7037"
#define SA_PID_FILE        "wlserver.pid"
#define SA_INI_FILE        "wlserver.ini"
#define SA_PATH_1          "/cachesys/weblink"
#define SA_PATH_2          "/usr/cachesys/weblink"
#define SA_PATH_3          "/etc"


#if APACHE_RELEASE < 1030000

#define MGW_APACHE_VERSION       12

#define MGW_TABLE_GET            table_get
#define MGW_TABLE_DO             table_do
#define MGW_TABLE_SET            table_set
#define MGW_PSTRCAT              pstrcat
#define MGW_PSTRDUP              pstrdup
#define MGW_PCALLOC              pcalloc
#define MGW_RWRITE               rwrite
#define MGW_RPRINTF              rprintf
#define MGW_RPUTS                rputs
#define MGW_SOFT_TIMEOUT         soft_timeout
#define MGW_KILL_TIMEOUT         kill_timeout
#define MGW_SHOULD_CLIENT_BLOCK  should_client_block
#define MGW_SETUP_CLIENT_BLOCK   setup_client_block
#define MGW_ADD_CGI_VARS         add_cgi_vars
#define MGW_GET_CLIENT_BLOCK     get_client_block
#define MGW_ADD_COMMON_VARS      add_common_vars
#define MGW_MAKE_SUB_POOL        make_sub_pool
#define MGW_DESTROY_POOL         destroy_pool
#define MGW_MAKE_TABLE           make_table
#define MGW_SEND_HTTP_HEADER     send_http_header
#define MGW_GET_MODULE_CONFIG    get_module_config

#else

#if APACHE_RELEASE == 2000000

#define MGW_APACHE_VERSION       20

typedef apr_pool_t               pool;
typedef apr_table_t              table;
#define SERVER_ERROR             HTTP_INTERNAL_SERVER_ERROR

#else

#define MGW_APACHE_VERSION       13

#endif


#if MGW_APACHE_VERSION == 20
#define MGW_PCALLOC              apr_pcalloc
#define MGW_TABLE_GET            apr_table_get
#define MGW_TABLE_DO             apr_table_do
#define MGW_TABLE_SET            apr_table_set
#define MGW_PSTRCAT              apr_pstrcat
#define MGW_PSTRDUP              apr_pstrdup
#define MGW_MAKE_TABLE           apr_table_make
#define MGW_MAKE_SUB_POOL        apr_create_pool
#define MGW_DESTROY_POOL         apr_pool_destroy
#else
#define MGW_PCALLOC              ap_pcalloc
#define MGW_TABLE_GET            ap_table_get
#define MGW_TABLE_DO             ap_table_do
#define MGW_TABLE_SET            ap_table_set
#define MGW_PSTRCAT              ap_pstrcat
#define MGW_PSTRDUP              ap_pstrdup
#define MGW_MAKE_TABLE           ap_make_table
#define MGW_MAKE_SUB_POOL        ap_make_sub_pool
#define MGW_DESTROY_POOL         ap_destroy_pool
#endif

#define MGW_RWRITE               ap_rwrite
#define MGW_RPRINTF              ap_rprintf
#define MGW_RPUTS                ap_rputs
#define MGW_SOFT_TIMEOUT         ap_soft_timeout
#define MGW_RESET_TIMEOUT        ap_reset_timeout
#define MGW_KILL_TIMEOUT         ap_kill_timeout
#define MGW_SHOULD_CLIENT_BLOCK  ap_should_client_block
#define MGW_SETUP_CLIENT_BLOCK   ap_setup_client_block
#define MGW_ADD_CGI_VARS         ap_add_cgi_vars
#define MGW_GET_CLIENT_BLOCK     ap_get_client_block
#define MGW_ADD_COMMON_VARS      ap_add_common_vars
#define MGW_SEND_HTTP_HEADER     ap_send_http_header
#define MGW_GET_MODULE_CONFIG    ap_get_module_config

#endif



/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Data declarations.                                                       */
/*                                                                          */
/* Here are the static cells and structure declarations private to our      */
/* module.                                                                  */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*
 * Sample configuration record.  Used for both per-directory and per-server
 * configuration data.
 *
 * It's perfectly reasonable to have two different structures for the two
 * different environments.  The same command handlers will be called for
 * both, though, so the handlers need to be able to tell them apart.  One
 * possibility is for both structures to start with an int which is zero for
 * one and 1 for the other.
 *
 * Note that while the per-directory and per-server configuration records are
 * available to most of the module handlers, they should be treated as
 * READ-ONLY by all except the command and merge handlers.  Sometimes handlers
 * are handed a record that applies to the current location by implication or
 * inheritance, and modifying it will change the rules for other locations.
 */
typedef struct weblink_config {
   int     cmode;      /* Environment to which record applies (directory,  */
                       /* server, or combination).                         */
#define CONFIG_MODE_SERVER 1
#define CONFIG_MODE_DIRECTORY 2
#define CONFIG_MODE_COMBO 3  /* Shouldn't ever happen.                      */
   int     local;      /* Boolean: was "Example" directive declared here?  */
   int     congenital; /* Boolean: did we inherit an "Example"?            */
   char    *trace;     /* Pointer to trace string.                         */
   char    *loc;       /* Location to which this record applies.           */
} weblink_config;

/*
 * Let's set up a module-local static cell to point to the accreting callback
 * trace.  As each API callback is made to us, we'll tack on the particulars
 * to whatever we've already recorded.  To avoid massive memory bloat as
 * directories are walked again and again, we record the routine/environment
 * the first time (non-request context only), and ignore subsequent calls for
 * the same routine/environment.
 */

static char *trace = NULL;
static table *static_calls_made = NULL;

/*
 * To avoid leaking memory from pools other than the per-request one, we
 * allocate a module-private pool, and then use a sub-pool of that which gets
 * freed each time we modify the trace.  That way previous layers of trace
 * data don't get lost.
 */
static pool *weblink_pool = NULL;
static pool *weblink_subpool = NULL;

/*
 * Declare ourselves so the configuration routines can find and know us.
 * We'll fill it in at the end of the module.
 */

#if MGW_APACHE_VERSION == 20
module AP_MODULE_DECLARE_DATA weblink_module;
#else
module weblink_module;
#endif

#ifdef _WIN32
static WORD VersionRequested;
#else
extern int errno;
#endif

static char *CgiEnvVars[] =  {"AUTH_PASSWORD",
                              "AUTH_TYPE",
                              "REMOTE_ADDR",
                              "REMOTE_HOST",
                              "REMOTE_USER",
                              "HTTP_USER_AGENT",
                              "SERVER_PROTOCOL",
                              "SERVER_SOFTWARE",
                              "SERVER_NAME",
                              "GATEWAY_INTERFACE",
                              "SERVER_PORT",
                              "REMOTE_IDENT",
                              "HTTP_ACCEPT",
                              "HTTP_REFERER",
                              "PATH_TRANSLATED",
                              "CONTENT_TYPE",
                              "HTTP_AUTHORIZATION",
                              "HTTP_COOKIE",
                              "CONTENT_LENGTH",
                              "REQUEST_METHOD",
                              "QUERY_STRING",
                              "SCRIPT_NAME",
                              "ALL_ENV",
                              "\0"};


typedef struct tagMEMOBJ {
   int       size;
   int       curr_size;
   int       incr_size;
   char *    lp_buffer;
} MEMOBJ, *LPMEMOBJ;

typedef struct tagCONX {
#ifdef _WIN32
   WSADATA     wsadata;
   SOCKET      sockfd;
#else
   int         sockfd;
#endif
   char        ip_address[32];
   int         port;
   char        error[128];
} CONX, *LPCONX;


typedef struct _tagEXTENSION_CONTROL_BLOCK {
   request_rec    *lp_request_rec;
   char           *lp_method;
   char           *lp_ibuffer;
} EXTENSION_CONTROL_BLOCK, *LPEXTENSION_CONTROL_BLOCK;

#if MGW_APACHE_VERSION == 20
static apr_status_t  weblink_child_exit    (void *data);
#else
static void          weblink_child_exit    (server_rec *s, pool *p);
#endif

int         test_procedure             (LPEXTENSION_CONTROL_BLOCK pECB);

int         MgwGetConfiguration        (LPCONX lp_connection);
int         MgwOpenConnection          (LPCONX lp_connection);
int         MgwCloseConnection         (LPCONX lp_connection);
int         MgwSendDataToNSD           (LPCONX lp_connection, char *data, int data_size);
int         MgwReceiveDataFromNSD      (LPCONX lp_connection, char *data, int data_size);
int         MgwReturnError             (EXTENSION_CONTROL_BLOCK *pECB, char *error);
int         MgwReturnRequestBlock      (EXTENSION_CONTROL_BLOCK *pECB, LPMEMOBJ lp_trans_buffer);
int         MgwWriteString             (EXTENSION_CONTROL_BLOCK *pECB, char *buffer);
int         MgwWriteBuffer             (EXTENSION_CONTROL_BLOCK *pECB, void *buffer, int size);
int         MgwServerVariable          (void *rec, const char *key, const char *value);
int         MgwGetServerVariable       (EXTENSION_CONTROL_BLOCK *pECB, char *VariableName, LPMEMOBJ lp_cgievar);
int         MgwMemInit                 (LPMEMOBJ lp_mem_obj, int size, int incr_size);
int         MgwMemFree                 (LPMEMOBJ lp_mem_obj);
int         MgwMemCpy                  (LPMEMOBJ lp_mem_obj, char *string);
int         MgwMemCat                  (LPMEMOBJ lp_mem_obj, char *string);
int         MgwLCase                   (char *string);



/*--------------------------------------------------------------------------*/
/*                                                                          */
/* The following pseudo-prototype declarations illustrate the parameters    */
/* passed to command handlers for the different types of directive          */
/* syntax.  If an argument was specified in the directive definition        */
/* (look for "command_rec" below), it's available to the command handler    */
/* via the (void *) info field in the cmd_parms argument passed to the      */
/* handler (cmd->info for the examples below).                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*
 * Command handler for a NO_ARGS directive.
 *
 * static const char *handle_NO_ARGS
 *      (cmd_parms *cmd, void *mconfig);
 */
 
/*
 * Command handler for a RAW_ARGS directive.  The "args" argument is the text
 * of the commandline following the directive itself.
 *
 * static const char *handle_RAW_ARGS
 *      (cmd_parms *cmd, void *mconfig, const char *args);
 */

/*
 * Command handler for a TAKE1 directive.  The single parameter is passed in
 * "word1".
 *
 * static const char *handle_TAKE1
 *      (cmd_parms *cmd, void *mconfig, char *word1);
 */

/*
 * Command handler for a TAKE2 directive.  TAKE2 commands must always have
 * exactly two arguments.
 *
 * static const char *handle_TAKE2
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2);
 */

/*
 * Command handler for a TAKE3 directive.  Like TAKE2, these must have exactly
 * three arguments, or the parser complains and doesn't bother calling us.
 *
 * static const char *handle_TAKE3
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a TAKE12 directive.  These can take either one or two
 * arguments.
 * - word2 is a NULL pointer if no second argument was specified.
 *
 * static const char *handle_TAKE12
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2);
 */

/*
 * Command handler for a TAKE123 directive.  A TAKE123 directive can be given,
 * as might be expected, one, two, or three arguments.
 * - word2 is a NULL pointer if no second argument was specified.
 * - word3 is a NULL pointer if no third argument was specified.
 *
 * static const char *handle_TAKE123
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a TAKE13 directive.  Either one or three arguments are
 * permitted - no two-parameters-only syntax is allowed.
 * - word2 and word3 are NULL pointers if only one argument was specified.
 *
 * static const char *handle_TAKE13
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a TAKE23 directive.  At least two and as many as three
 * arguments must be specified.
 * - word3 is a NULL pointer if no third argument was specified.
 *
 * static const char *handle_TAKE23
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a ITERATE directive.
 * - Handler is called once for each of n arguments given to the directive.
 * - word1 points to each argument in turn.
 *
 * static const char *handle_ITERATE
 *      (cmd_parms *cmd, void *mconfig, char *word1);
 */

/*
 * Command handler for a ITERATE2 directive.
 * - Handler is called once for each of the second and subsequent arguments
 *   given to the directive.
 * - word1 is the same for each call for a particular directive instance (the
 *   first argument).
 * - word2 points to each of the second and subsequent arguments in turn.
 *
 * static const char *handle_ITERATE2
 *      (cmd_parms *cmd, void *mconfig, char *word1, char *word2);
 */

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* These routines are strictly internal to this module, and support its     */
/* operation.  They are not referenced by any external portion of the       */
/* server.                                                                  */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*
 * Locate our directory configuration record for the current request.
 */

static weblink_config *our_dconfig(request_rec *r)
{
   return (weblink_config *) MGW_GET_MODULE_CONFIG(r->per_dir_config, &weblink_module);
}


#if MGW_APACHE_VERSION == 20

/*
 * Likewise for our configuration record for a connection.
 */
static weblink_config *our_cconfig(const conn_rec *c)
{
    return (weblink_config *) MGW_GET_MODULE_CONFIG(c->conn_config, &weblink_module);
}

#else

/*
 * Locate our server configuration record for the specified server.
 */

static weblink_config *our_sconfig(server_rec *s)
{
   return (weblink_config *) MGW_GET_MODULE_CONFIG(s->module_config, &weblink_module);
}


/*
 * Likewise for our configuration record for the specified request.
 */

static weblink_config *our_rconfig(request_rec *r)
{
   return (weblink_config *) MGW_GET_MODULE_CONFIG(r->request_config, &weblink_module);
}

#endif


/*
 * This routine sets up some module-wide cells if they haven't been already.
 */

static void setup_module_cells ()
{
   /*
    * If we haven't already allocated our module-private pool, do so now.
    */

   if (weblink_pool == NULL) {

#if MGW_APACHE_VERSION == 20
      apr_pool_create(&weblink_pool, NULL);
#else
      weblink_pool = MGW_MAKE_SUB_POOL (NULL);
#endif

   }

   /*
    * Likewise for the table of routine/environment pairs we visit outside of
    * request context.
    */

   if (static_calls_made == NULL) {
      static_calls_made = MGW_MAKE_TABLE (weblink_pool, 16);
   }
}


/*
 * This routine is used to add a trace of a callback to the list.  We're
 * passed the server record (if available), the request record (if available),
 * a pointer to our private configuration record (if available) for the
 * environment to which the callback is supposed to apply, and some text.  We
 * turn this into a textual representation and add it to the tail of the list.
 * The list can be displayed by the weblink_handler() routine.
 *
 * If the call occurs within a request context (i.e., we're passed a request
 * record), we put the trace into the request pool and attach it to the
 * request via the notes mechanism.  Otherwise, the trace gets added
 * to the static (non-request-specific) list.
 *
 * Note that the r->notes table is only for storing strings; if you need to
 * maintain per-request data of any other type, you need to use another
 * mechanism.
 */

#define TRACE_NOTE "weblink-trace"

static void trace_add(server_rec *s, request_rec *r, weblink_config *mconfig,
                      const char *note)
{

   char    *sofar;
   char    *addon;
   char    *where;
   pool    *p;
   char    *trace_copy;
   weblink_config *rconfig = NULL;

   /*
    * Make sure our pools and tables are set up - we need 'em.
    */

   setup_module_cells ();

   /*
    * Now, if we're in request-context, we use the request pool.
    */

   if (r != NULL) {
      p = r->pool;
      if ((trace_copy = (char *) MGW_TABLE_GET (r->notes, TRACE_NOTE)) == NULL) {
         trace_copy = "";
      }
   } else {

      /*
       * We're not in request context, so the trace gets attached to our
       * module-wide pool.  We do the create/destroy every time we're called
       * in non-request context; this avoids leaking memory in some of
       * the subsequent calls that allocate memory only once (such as the
       * key formation below).
       *
       * Make a new sub-pool and copy any existing trace to it.  Point the
       * trace cell at the copied value.
       */

#if MGW_APACHE_VERSION == 20
      apr_pool_create(&p, weblink_pool);
#else
      p = MGW_MAKE_SUB_POOL (weblink_pool);
#endif

      if (trace != NULL) {
         trace = MGW_PSTRDUP (p, trace);
      }

      /*
       * Now, if we have a sub-pool from before, nuke it and replace with
       * the one we just allocated.
       */

      if (weblink_subpool != NULL) {
         MGW_DESTROY_POOL (weblink_subpool);
      }
      weblink_subpool = p;
      trace_copy = trace;
   }

   /*
    * If we weren't passed a configuration record, we can't figure out to
    * what location this call applies.  This only happens for co-routines
    * that don't operate in a particular directory or server context.  If we
    * got a valid record, extract the location (directory or server) to which
    * it applies.
    */

   where = (mconfig != NULL) ? mconfig->loc : "nowhere";
   where = (where != NULL) ? where : "";

   /*
    * Now, if we're not in request context, see if we've been called with
    * this particular combination before.  The table is allocated in the
    * module's private pool, which doesn't get destroyed.
    */

   if (r == NULL) {
      char    *key;

      key = MGW_PSTRCAT (p, note, ":", where, NULL);
      if (MGW_TABLE_GET (static_calls_made, key) != NULL) {

         /*
          * Been here, done this.
          */

         return;
      } else {

         /*
          * First time for this combination of routine and environment -
          * log it so we don't do it again.
          */

         MGW_TABLE_SET (static_calls_made, key, "been here");
      }
   }
   addon = MGW_PSTRCAT(
                        p,
                        "   <LI>\n",
                        "    <DL>\n",
                        "     <DT><SAMP>",
                        note,
                        "</SAMP>\n",
                        "     </DT>\n",
                        "     <DD><SAMP>[",
                        where,
                        "]</SAMP>\n",
                        "     </DD>\n",
                        "    </DL>\n",
                        "   </LI>\n",
                        NULL
                        );
   sofar = (trace_copy == NULL) ? "" : trace_copy;
   trace_copy = MGW_PSTRCAT (p, sofar, addon, NULL);
   if (r != NULL) {
      MGW_TABLE_SET (r->notes, TRACE_NOTE, trace_copy);
   } else {
      trace = trace_copy;
   }
}



/*--------------------------------------------------------------------------*/
/* We prototyped the various syntax for command handlers (routines that     */
/* are called when the configuration parser detects a directive declared    */
/* by our module) earlier.  Now we actually declare a "real" routine that   */
/* will be invoked by the parser when our "real" directive is               */
/* encountered.                                                             */
/*                                                                          */
/* If a command handler encounters a problem processing the directive, it   */
/* signals this fact by returning a non-NULL pointer to a string            */
/* describing the problem.                                                  */
/*                                                                          */
/* The magic return value DECLINE_CMD is used to deal with directives       */
/* that might be declared by multiple modules.  If the command handler      */
/* returns NULL, the directive was processed; if it returns DECLINE_CMD,    */
/* the next module (if any) that declares the directive is given a chance   */
/* at it.  If it returns any other value, it's treated as the text of an    */
/* error message.                                                           */
/*--------------------------------------------------------------------------*/
/* 
 * Command handler for the NO_ARGS "Example" directive.  All we do is mark the
 * call in the trace log, and flag the applicability of the directive to the
 * current location in that location's configuration record.
 */

static const char *cmd_weblink(cmd_parms *cmd, void *mconfig)
{
   weblink_config *cfg = (weblink_config *) mconfig;

   cfg->local = 1;

   /* trace_add (cmd->server, NULL, cfg, "cmd_weblink()"); */

   return NULL;
}


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now we declare our content handlers, which are invoked when the server   */
/* encounters a document which our module is supposed to have a chance to   */
/* see.  (See mod_mime's SetHandler and AddHandler directives, and the      */
/* mod_info and mod_status examples, for more details.)                     */
/*                                                                          */
/* Since content handlers are dumping data directly into the connexion      */
/* (using the r*() routines, such as rputs() and rprintf()) without         */
/* intervention by other parts of the server, they need to make             */
/* sure any accumulated HTTP headers are sent first.  This is done by       */
/* calling send_http_header().  Otherwise, no header will be sent at all,   */
/* and the output sent to the client will actually be HTTP-uncompliant.     */
/*--------------------------------------------------------------------------*/
/* 
 * Sample content handler.  All this does is display the call list that has
 * been built up so far.
 *
 * The return value instructs the caller concerning what happened and what to
 * do next:
 *  OK ("we did our thing")
 *  DECLINED ("this isn't something with which we want to get involved")
 *  HTTP_mumble ("an error status should be reported")
 */

static int weblink_handler (request_rec *r)
{
   int retval, error_code;
   long read, post_len, n1;
   unsigned int n, n2, len;
   char buffer[256];
   char *lp_cbuffer;
   EXTENSION_CONTROL_BLOCK ECB;
   LPEXTENSION_CONTROL_BLOCK pECB;
   CONX connection;
   LPCONX lp_connection;
   MEMOBJ cgievar, trans_buffer;
   LPMEMOBJ lp_cgievar, lp_trans_buffer;
   table *e = r->subprocess_env;

#if MGW_APACHE_VERSION == 20

   conn_rec *c = r->connection;
   struct ap_filter_t *cur;

#endif


#if MGW_APACHE_VERSION == 20

   if (!r->uri)
      return DECLINED;
   n = (int) strlen(r->uri);
   if (!n)
      return DECLINED;

   if (n < 100) {
      strcpy(buffer, r->uri);
      MgwLCase(buffer);
      if (!strstr(buffer, "/mgwa"))
         return DECLINED;
   }

   n1 = 0;

   for (n --; n >= 0; n --) {
      if (r->uri[n] == '.') {
         for (n ++; r->uri[n]; n ++) {
            if (r->uri[n] == '/' || r->uri[n] == '\\')
               break;
            buffer[n1 ++] = r->uri[n];
         }
         break;
      }
   }

   buffer[n1] = '\0';
   if (!n1)
      return DECLINED;

   MgwLCase(buffer);

   if (strcmp(buffer, "mgw") && strcmp(buffer, "wl") && strcmp(buffer, "wld"))
      return DECLINED;

#endif /* #if MGW_APACHE_VERSION == 20 */


   pECB = &ECB;
   pECB->lp_request_rec = r;

   lp_cgievar = &cgievar;
   lp_trans_buffer = &trans_buffer;

   lp_connection = &connection;

   MgwGetConfiguration(lp_connection);

#if MGW_APACHE_VERSION == 20

   /* get rid of all filters up through protocol...  since we
    * haven't parsed off the headers, there is no way they can
    * work
    */

   cur = r->proto_output_filters;
   while (cur && cur->frec->ftype < AP_FTYPE_CONNECTION) {
      cur = cur->next;
   }
   r->output_filters = r->proto_output_filters = cur;

#endif

   if ((retval = MGW_SETUP_CLIENT_BLOCK(r, REQUEST_CHUNKED_ERROR)))
      return retval;

   MGW_ADD_COMMON_VARS(r);
   MGW_ADD_CGI_VARS(r);

   pECB->lp_method = (char *) r->method;

   MgwMemInit(lp_cgievar, 256, 256);
   MgwMemInit(lp_trans_buffer, 1024, 256);

   MgwGetServerVariable(pECB, "CONTENT_LENGTH", lp_cgievar);
   post_len = strtol(lp_cgievar->lp_buffer, NULL, 10);

   for (n = 0; strlen(CgiEnvVars[n]) > 0; n ++) {

      MgwGetServerVariable(pECB, CgiEnvVars[n], lp_cgievar);

      if (lp_cgievar->curr_size) {
         MgwMemCat(lp_trans_buffer, CgiEnvVars[n]);
         MgwMemCat(lp_trans_buffer, "=");
         MgwMemCat(lp_trans_buffer, lp_cgievar->lp_buffer);
         MgwMemCat(lp_trans_buffer, "\r");
      }
   }

   if (strstr(lp_trans_buffer->lp_buffer, "cmunt~cmunt") || strstr(lp_trans_buffer->lp_buffer, "cmunt=cmunt")) {
      MgwReturnRequestBlock(pECB, lp_trans_buffer);
      /* test_procedure(pECB); */
      goto exit;
   }

   n = MgwOpenConnection(lp_connection);
   if (!n) {
      MgwReturnError(pECB, lp_connection->error);

      goto exit;
   }

   n = MgwSendDataToNSD(lp_connection, lp_trans_buffer->lp_buffer, lp_trans_buffer->curr_size);

   if (MGW_SHOULD_CLIENT_BLOCK(r)) {

      if (post_len) {

         lp_cbuffer = (char *) malloc(2048 * sizeof(char));

         for (n1 = 0, n2 = 0; n1 < post_len; n1 ++) {
            read = MGW_GET_CLIENT_BLOCK(r, buffer, 1);
            if (read < 0) {
               free((void *) lp_cbuffer);
               MgwMemFree(lp_cgievar);
               MgwMemFree(lp_trans_buffer);
               return SERVER_ERROR;
            }
            lp_cbuffer[n2 ++] = buffer[0];
            if (n2 > 2000) {
               n = send(lp_connection->sockfd, lp_cbuffer, n2, 0);
               n2 = 0;
            }
         }
         if (n2) {
            n = send(lp_connection->sockfd, lp_cbuffer, n2, 0);
            n2 = 0;
         }

         free((void *) lp_cbuffer);
         lp_cbuffer = NULL;

      }
   }

#if MGW_APACHE_VERSION != 20
   MGW_SOFT_TIMEOUT ("send WebLink call trace", r);
#endif

   len = 0;
   for (;;) {
      error_code = MgwReceiveDataFromNSD(lp_connection, buffer, 200);

#if MGW_APACHE_VERSION != 20
      MGW_RESET_TIMEOUT (r);
#endif

      if (error_code < 1) {
         break;
      }
      len = error_code;
      if (len)
         MgwWriteBuffer(pECB, buffer, len);
   }
   
   n = MgwCloseConnection(lp_connection);
/*
#if MGW_APACHE_VERSION != 20
   MGW_KILL_TIMEOUT (r);
#endif
*/
exit:

#if MGW_APACHE_VERSION != 20
   MGW_KILL_TIMEOUT (r);
#endif

   MgwMemFree(lp_cgievar);
   MgwMemFree(lp_trans_buffer);

   return OK;
}


/* Simple environment test */

int test_procedure(LPEXTENSION_CONTROL_BLOCK pECB)
{
   request_rec *r = pECB->lp_request_rec;
   table *e = r->subprocess_env;

   weblink_config *dcfg;

   dcfg = our_dconfig (r);
   trace_add (r->server, r, dcfg, "weblink_handler()");

   /*
    * We're about to start sending content, so we need to force the HTTP
    * headers to be sent at this point.  Otherwise, no headers will be sent
    * at all.  We can set any we like first, of course.  **NOTE** Here's
    * where you set the "Content-type" header, and you do so by putting it in
    * r->content_type, *not* r->headers_out("Content-type").  If you don't
    * set it, it will be filled in with the server's default type (typically
    * "text/plain").
    *
    * We also need to start a timer so the server can know if the connexion
    * is broken.
    */

#if MGW_APACHE_VERSION == 20
   ap_set_content_type(r, "text/html");
#else
   r->content_type = "text/html";
#endif

#if MGW_APACHE_VERSION != 20
   MGW_SOFT_TIMEOUT ("send weblink call trace", r);
   MGW_SEND_HTTP_HEADER (r);
#endif

   /*
    * If we're only supposed to send header information (HEAD request), we're
    * already there.
    */

   if (r->header_only) {

#if MGW_APACHE_VERSION != 20
      MGW_KILL_TIMEOUT (r);
#endif

      return OK;
   }

   /*
    * Now send our actual output.  Since we tagged this as being
    * "text/html", we need to embed any HTML.
    */

   MGW_RPUTS ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n", r);
   MGW_RPUTS ("<HTML>\n", r);
   MGW_RPUTS (" <HEAD>\n", r);
   MGW_RPUTS ("  <TITLE>mod_weblink Module Content-Handler Output\n", r);
   MGW_RPUTS ("  </TITLE>\n", r);
   MGW_RPUTS (" </HEAD>\n", r);
   MGW_RPUTS (" <BODY>\n", r);
   MGW_RPUTS ("  <H1><SAMP>mod_weblink</SAMP> Module Content-Handler Output\n", r);
   MGW_RPUTS ("  </H1>\n", r);

   MGW_RPUTS ("  <P>\n", r);
   MGW_RPUTS ("  The format for the callback trace is:\n", r);
   MGW_RPUTS ("  </P>\n", r);
   MGW_RPUTS ("  <DL>\n", r);
   MGW_RPUTS ("   <DT><EM>n</EM>.<SAMP>&lt;routine-name&gt;", r);
   MGW_RPUTS ("(&lt;routine-data&gt;)</SAMP>\n", r);
   MGW_RPUTS ("   </DT>\n", r);
   MGW_RPUTS ("   <DD><SAMP>[&lt;applies-to&gt;]</SAMP>\n", r);
   MGW_RPUTS ("   </DD>\n", r);
   MGW_RPUTS ("  </DL>\n", r);
   MGW_RPUTS ("  <P>\n", r);
   MGW_RPUTS ("  The <SAMP>&lt;routine-data&gt;</SAMP> is supplied by\n", r);
   MGW_RPUTS ("  the routine when it requests the trace,\n", r);
   MGW_RPUTS ("  and the <SAMP>&lt;applies-to&gt;</SAMP> is extracted\n", r);
   MGW_RPUTS ("  from the configuration record at the time of the trace.\n", r); 
   MGW_RPUTS ("  <STRONG>SVR()</STRONG> indicates a server environment\n", r);
   MGW_RPUTS ("  (blank means the main or default server, otherwise it's\n", r);
   MGW_RPUTS ("  the name of the VirtualHost); <STRONG>DIR()</STRONG>\n", r);
   MGW_RPUTS ("  indicates a location in the URL or filesystem\n", r);
   MGW_RPUTS ("  namespace.\n", r);
   MGW_RPUTS ("  </P>\n", r);
   MGW_RPRINTF
      (
         r,
         "  <H2>Static callbacks so far:</H2>\n  <OL>\n%s  </OL>\n",
         trace
      );
   MGW_RPRINTF
      (
         r,
         "  <H2>Request-specific callbacks so far:</H2>\n  <OL>\n%s  </OL>\n",
         MGW_TABLE_GET (r->notes, TRACE_NOTE)
      );
   MGW_RPUTS ("  <H2>Environment for <EM>this</EM> call:</H2>\n", r);
   MGW_RPUTS ("  <UL>\n", r);
   MGW_RPRINTF (r, "   <LI>Applies-to: <SAMP>%s</SAMP>\n   </LI>\n", dcfg->loc);
   MGW_RPRINTF
      (
         r,
         "   <LI>\"Example\" directive declared here: %s\n   </LI>\n",
         (dcfg->local ? "YES" : "NO")
      );
   MGW_RPRINTF
      (
         r,
         "   <LI>\"Example\" inherited: %s\n   </LI>\n",
         (dcfg->congenital ? "YES" : "NO")
      );
   MGW_RPUTS ("  </UL>\n", r);
   MGW_RPUTS (" </BODY>\n", r);
   MGW_RPUTS ("</HTML>\n", r);

   /*
    * We're all done, so cancel the timeout we set.  Since this is probably
    * the end of the request we *could* assume this would be done during
    * post-processing - but it's possible that another handler might be
    * called and inherit our outstanding timer.  Not good; to each its own.
    */

#if MGW_APACHE_VERSION != 20
   MGW_KILL_TIMEOUT (r);
#endif

   /*
    * We did what we wanted to do, so tell the rest of the server we
    * succeeded.
    */

   return OK;
}


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now let's declare routines for each of the callback phase in order.      */
/* (That's the order in which they're listed in the callback list, *not     */
/* the order in which the server calls them!  See the command_rec           */
/* declaration near the bottom of this file.)  Note that these may be       */
/* called for situations that don't relate primarily to our function - in   */
/* other words, the fixup handler shouldn't assume that the request has     */
/* to do with "example" stuff.                                              */
/*                                                                          */
/* With the exception of the content handler, all of our routines will be   */
/* called for each request, unless an earlier handler from another module   */
/* aborted the sequence.                                                    */
/*                                                                          */
/* Handlers that are declared as "int" can return the following:            */
/*                                                                          */
/*  OK          Handler accepted the request and did its thing with it.     */
/*  DECLINED    Handler took no action.                                     */
/*  HTTP_mumble Handler looked at request and found it wanting.             */
/*                                                                          */
/* What the server does after calling a module handler depends upon the     */
/* handler's return value.  In all cases, if the handler returns            */
/* DECLINED, the server will continue to the next module with an handler    */
/* for the current phase.  However, if the handler return a non-OK,         */
/* non-DECLINED status, the server aborts the request right there.  If      */
/* the handler returns OK, the server's next action is phase-specific;      */
/* see the individual handler comments below for details.                   */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * This function is called during server initialisation.  Any information
 * that needs to be recorded must be in static cells, since there's no
 * configuration record.
 *
 * There is no return value.
 */

/*
 * All our module-initialiser does is add its trace to the log.
 */

static void weblink_init (server_rec *s, pool *p)
{
   char    *note;
   char    *sname = s->server_hostname;

   /*
    * Set up any module cells that ought to be initialised.
    */

   setup_module_cells ();

   /*
    * The arbitrary text we add to our trace entry indicates for which server
    * we're being called.
    */

   sname = (sname != NULL) ? sname : "";
   note = MGW_PSTRCAT (p, "weblink_init(", sname, ")", NULL);
   trace_add (s, NULL, NULL, note);
}


/* 
 * This function is called during server initialisation when an heavy-weight
 * process (such as a child) is being initialised.  As with the
 * module-initialisation function, any information that needs to be recorded
 * must be in static cells, since there's no configuration record.
 *
 * There is no return value.
 */

/*
 * All our process-initialiser does is add its trace to the log.
 */

#if MGW_APACHE_VERSION == 20
static void weblink_child_init(pool *p, server_rec *s)
#else
static void weblink_child_init(server_rec *s, pool *p)
#endif
{
   char    *note;
   char    *sname = s->server_hostname;

   /*
    * Set up any module cells that ought to be initialised.
    */

   setup_module_cells();

   /*
    * The arbitrary text we add to our trace entry indicates for which server
    * we're being called.
    */

   sname = (sname != NULL) ? sname : "";
   note = MGW_PSTRCAT (p, "weblink_child_init(", sname, ")", NULL);
   trace_add (s, NULL, NULL, note);

#if MGW_APACHE_VERSION == 20
   apr_pool_cleanup_register(p, s, weblink_child_exit, weblink_child_exit);
#endif

}


/* 
 * This function is called when an heavy-weight process (such as a child) is
 * being run down or destroyed.  As with the child-initialisation function,
 * any information that needs to be recorded must be in static cells, since
 * there's no configuration record.
 *
 * There is no return value.
 */

/*
 * All our process-death routine does is add its trace to the log.
 */

#if MGW_APACHE_VERSION == 20

static apr_status_t weblink_child_exit(void *data)
{
    char *note;
    server_rec *s = data;
    char *sname = s->server_hostname;

    /*
     * The arbitrary text we add to our trace entry indicates for which server
     * we're being called.
     */
    sname = (sname != NULL) ? sname : "";
    note = apr_pstrcat(s->process->pool, "weblink_child_exit(", sname, ")", NULL);
    trace_add(s, NULL, NULL, note);
    return APR_SUCCESS;
}

#else

static void weblink_child_exit(server_rec *s, pool *p)
{
   char *note;
   char *sname = s->server_hostname;

   /*
    * The arbitrary text we add to our trace entry indicates for which server
    * we're being called.
    */

   sname = (sname != NULL) ? sname : "";
   note = MGW_PSTRCAT(p, "weblink_child_exit(", sname, ")", NULL);
   trace_add(s, NULL, NULL, note);
}

#endif


/*
 * This function gets called to create up a per-directory configuration
 * record.  This will be called for the "default" server environment, and for
 * each directory for which the parser finds any of our directives applicable.
 * If a directory doesn't have any of our directives involved (i.e., they
 * aren't in the .htaccess file, or a <Location>, <Directory>, or related
 * block), this routine will *not* be called - the configuration for the
 * closest ancestor is used.
 *
 * The return value is a pointer to the created module-specific
 * structure.
 */

static void *weblink_create_dir_config(pool *p, char *dirspec)
{
   weblink_config *cfg;
   char    *dname = dirspec;

   /*
    * Allocate the space for our record from the pool supplied.
    */

   cfg = (weblink_config *) MGW_PCALLOC (p, sizeof(weblink_config));

   /*
    * Now fill in the defaults.  If there are any `parent' configuration
    * records, they'll get merged as part of a separate callback.
    */

   cfg->local = 0;
   cfg->congenital = 0;
   cfg->cmode = CONFIG_MODE_DIRECTORY;

   /*
    * Finally, add our trace to the callback list.
    */

   dname = (dname != NULL) ? dname : "";
   cfg->loc = MGW_PSTRCAT (p, "DIR(", dname, ")", NULL);
   trace_add (NULL, NULL, cfg, "weblink_create_dir_config()");

   return (void *) cfg;
}


/*
 * This function gets called to merge two per-directory configuration
 * records.  This is typically done to cope with things like .htaccess files
 * or <Location> directives for directories that are beneath one for which a
 * configuration record was already created.  The routine has the
 * responsibility of creating a new record and merging the contents of the
 * other two into it appropriately.  If the module doesn't declare a merge
 * routine, the record for the closest ancestor location (that has one) is
 * used exclusively.
 *
 * The routine MUST NOT modify any of its arguments!
 *
 * The return value is a pointer to the created module-specific structure
 * containing the merged values.
 */

static void *weblink_merge_dir_config(pool *p, void *parent_conf, void *newloc_conf)
{
   weblink_config *merged_config = (weblink_config *) MGW_PCALLOC (p, sizeof(weblink_config));
   weblink_config *pconf = (weblink_config *) parent_conf;
   weblink_config *nconf = (weblink_config *) newloc_conf;
   char    *note;

   /*
    * Some things get copied directly from the more-specific record, rather
    * than getting merged.
    */

   merged_config->local = nconf->local;
   merged_config->loc = MGW_PSTRDUP (p, nconf->loc);

   /*
    * Others, like the setting of the `congenital' flag, get ORed in.  The
    * setting of that particular flag, for instance, is TRUE if it was ever
    * true anywhere in the upstream configuration.
    */

   merged_config->congenital = (pconf->congenital | pconf->local);

   /*
    * If we're merging records for two different types of environment (server
    * and directory), mark the new record appropriately.  Otherwise, inherit
    * the current value.
    */

   merged_config->cmode = (pconf->cmode == nconf->cmode) ? pconf->cmode : CONFIG_MODE_COMBO;

   /*
    * Now just record our being called in the trace list.  Include the     * locations we were asked to merge.
    */

   note = MGW_PSTRCAT
            (
               p,
               "weblink_merge_dir_config(\"",
               pconf->loc,
               "\",\"",
               nconf->loc,
               "\")",
               NULL
            );

    trace_add (NULL, NULL, merged_config, note);

    return (void *) merged_config;
}


/*
 * This function gets called to create a per-server configuration
 * record.  It will always be called for the "default" server.
 *
 * The return value is a pointer to the created module-specific
 * structure.
 */

static void *weblink_create_server_config(pool *p, server_rec *s)
{
   weblink_config *cfg;
   char    *sname = s->server_hostname;

   /*
    * As with the weblink_create_dir_config() routine, we allocate and fill in an
    * empty record.
    */

   cfg = (weblink_config *) MGW_PCALLOC (p, sizeof(weblink_config));
   cfg->local = 0;
   cfg->congenital = 0;
   cfg->cmode = CONFIG_MODE_SERVER;

   /*
    * Note that we were called in the trace list.
    */

   sname = (sname != NULL) ? sname : "";
   cfg->loc = MGW_PSTRCAT (p, "SVR(", sname, ")", NULL);
   trace_add (s, NULL, cfg, "weblink_create_server_config()");

   return (void *) cfg;
}


/*
 * This function gets called to merge two per-server configuration
 * records.  This is typically done to cope with things like virtual hosts and
 * the default server configuration  The routine has the responsibility of
 * creating a new record and merging the contents of the other two into it
 * appropriately.  If the module doesn't declare a merge routine, the more
 * specific existing record is used exclusively.
 *
 * The routine MUST NOT modify any of its arguments!
 *
 * The return value is a pointer to the created module-specific structure
 * containing the merged values.
 */

static void *weblink_merge_server_config(pool *p, void *server1_conf, void *server2_conf)
{
   weblink_config *merged_config = (weblink_config *) MGW_PCALLOC (p, sizeof(weblink_config));
   weblink_config *s1conf = (weblink_config *) server1_conf;
   weblink_config *s2conf = (weblink_config *) server2_conf;
   char    *note;

   /*
    * Our inheritance rules are our own, and part of our module's semantics.
    * Basically, just note whence we came.
    */

   merged_config->cmode = (s1conf->cmode == s2conf->cmode) ? s1conf->cmode : CONFIG_MODE_COMBO;
   merged_config->local = s2conf->local;
   merged_config->congenital = (s1conf->congenital | s1conf->local);
   merged_config->loc = MGW_PSTRDUP (p, s2conf->loc);

   /*
    * Trace our call, including what we were asked to merge.
    */

   note = MGW_PSTRCAT
            (
               p,
               "weblink_merge_server_config(\"",
               s1conf->loc,
               "\",\"",
               s2conf->loc,
               "\")",
               NULL
            );
   trace_add (NULL, NULL, merged_config, note);

    return (void *) merged_config;
}


/*
 * This routine gives our module an opportunity to translate the URI into an
 * actual filename.  If we don't do anything special, the server's default
 * rules (Alias directives and the like) will continue to be followed.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
 * further modules are called for this phase.
 */

static int weblink_translate_handler(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);
   
   /*
    * We don't actually *do* anything here, except note the fact that we were
    * called.
    */

   trace_add (r->server, r, cfg, "weblink_translate_handler()");

   return DECLINED;
}


#if MGW_APACHE_VERSION == 20

/*
 * This routine is called to perform any module-specific fixing of header
 * fields, et cetera.  It is invoked just before any content-handler.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, the
 * server will still call any remaining modules with an handler for this
 * phase.
 */
static void weblink_insert_filter(request_rec *r)
{
    weblink_config *cfg;

    cfg = our_dconfig(r);
    /*
     * Log the call and exit.
     */
    trace_add(r->server, NULL, cfg, "weblink_post_config()");
}


/*
 * XXX fix my comment!!!!!!  this sounds like the comment for a fixup
 *     handler
 *
 * This routine is called to perform any module-specific fixing of header
 * fields, et cetera.  It is invoked just before any content-handler.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, the
 * server will still call any remaining modules with an handler for this
 * phase.
 */
static int weblink_quick_handler(request_rec *r, int lookup_uri)
{
    weblink_config *cfg;

    cfg = our_dconfig(r);
    /*
     * Log the call and exit.
     */
    trace_add(r->server, NULL, cfg, "weblink_post_config()");
    return DECLINED;
}


/*
 * This routine is called to perform any module-specific fixing of header
 * fields, et cetera.  It is invoked just before any content-handler.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, the
 * server will still call any remaining modules with an handler for this
 * phase.
 */
static int weblink_pre_connection(conn_rec *c, void *csd)
{
    weblink_config *cfg;

    cfg = our_cconfig(c);
#if 0
    /*
     * Log the call and exit.
     */
    trace_add(r->server, NULL, cfg, "weblink_post_config()");
#endif
    return OK;
}

#endif


/*
 * This routine is called after the request has been read but before any other
 * phases have been processed.  This allows us to make decisions based upon
 * the input header fields.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
 * further modules are called for this phase.
 */

static int weblink_post_read_request(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig(r);

   /*
    * We don't actually *do* anything here, except note the fact that we were
    * called.
    */

   trace_add(r->server, r, cfg, "weblink_post_read_request()");

   return DECLINED;
}


/*
 * This routine is called to check the authentication information sent with
 * the request (such as looking up the user in a database and verifying that
 * the [encrypted] password sent matches the one in the database).
 *
 * The return value is OK, DECLINED, or some HTTP_mumble error (typically
 * HTTP_UNAUTHORIZED).  If we return OK, no other modules are given a chance
 * at the request during this phase.
 */

static int weblink_check_user_id(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);

   /*
    * Don't do anything except log the call.
    */

   trace_add (r->server, r, cfg, "weblink_check_user_id()");

   return DECLINED;
}


/*
 * This routine is called to check to see if the resource being requested
 * requires authorisation.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
 * other modules are called during this phase.
 *
 * If *all* modules return DECLINED, the request is aborted with a server
 * error.
 */

static int weblink_auth_checker(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);

   /*
    * Log the call and return OK, or access will be denied (even though we
    * didn't actually do anything).
    */

   trace_add (r->server, r, cfg, "weblink_auth_checker()");

   return OK;
}


/*
 * This routine is called to check for any module-specific restrictions placed
 * upon the requested resource.  (See the mod_access module for an example.)
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  All modules with an
 * handler for this phase are called regardless of whether their predecessors
 * return OK or DECLINED.  The first one to return any other status, however,
 * will abort the sequence (and the request) as usual.
 */

static int weblink_access_checker(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);

   trace_add (r->server, r, cfg, "weblink_access_checker()");

   return OK;
}


/*
 * This routine is called to determine and/or set the various document type
 * information bits, like Content-type (via r->content_type), language, et
 * cetera.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
 * further modules are given a chance at the request for this phase.
 */

static int weblink_type_checker(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);

   /*
    * Log the call, but don't do anything else - and report truthfully that
    * we didn't do anything.
    */

   trace_add (r->server, r, cfg, "weblink_type_checker()");

   return DECLINED;
}


/*
 * This routine is called to perform any module-specific fixing of header
 * fields, et cetera.  It is invoked just before any content-handler.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, the
 * server will still call any remaining modules with an handler for this
 * phase.
 */

static int weblink_fixer(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);

   /*
    * Log the call and exit.
    */

   trace_add (r->server, r, cfg, "weblink_fixer()");

   return OK;
}


/*
 * This routine is called to perform any module-specific logging activities
 * over and above the normal server things.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, any
 * remaining modules with an handler for this phase will still be called.
 */

static int weblink_logger(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);
   trace_add (r->server, r, cfg, "weblink_logger()");

   return DECLINED;
}


/*
 * This routine is called to give the module a chance to look at the request
 * headers and take any appropriate specific actions early in the processing
 * sequence.
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, any
 * remaining modules with handlers for this phase will still be called.
 */

static int weblink_header_parser(request_rec *r)
{
   weblink_config *cfg;

   cfg = our_dconfig (r);
   trace_add (r->server, r, cfg, "weblink_header_parser()");

   return DECLINED;
}


#if MGW_APACHE_VERSION == 20

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Which functions are responsible for which hooks in the server.           */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * Each function our module provides to handle a particular hook is
 * specified here.  The functions are registered using 
 * ap_hook_foo(name, predecessors, successors, position)
 * where foo is the name of the hook.
 *
 * The args are as follows:
 * name         -> the name of the function to call.
 * predecessors -> a list of modules whose calls to this hook must be
 *                 invoked before this module.
 * successors   -> a list of modules whose calls to this hook must be
 *                 invoked after this module.
 * position     -> The relative position of this module.  One of
 *                 APR_HOOK_FIRST, APR_HOOK_MIDDLE, or APR_HOOK_LAST.
 *                 Most modules will use APR_HOOK_MIDDLE.  If multiple
 *                 modules use the same relative position, Apache will
 *                 determine which to call first.
 *                 If your module relies on another module to run first,
 *                 or another module running after yours, use the 
 *                 predecessors and/or successors.
 *
 * The number in brackets indicates the order in which the routine is called
 * during request processing.  Note that not all routines are necessarily
 * called (such as if a resource doesn't have access restrictions).
 * The actual delivery of content to the browser [9] is not handled by
 * a hook; see the handler declarations below.
 */

static void weblink_register_hooks(apr_pool_t *p)
{
#if 0

   ap_hook_pre_config(weblink_pre_config, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_post_config(weblink_post_config, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_open_logs(weblink_open_logs, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_child_init(weblink_child_init, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_handler(weblink_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_quick_handler(weblink_quick_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_pre_connection(weblink_pre_connection, NULL, NULL, APR_HOOK_MIDDLE);
/* This module doesn't have a process connection phase, but I am leaving
 * the code in, in-case somebody wants to add one.
 */
/*   ap_hook_process_connection(weblink_fixer_upper, NULL, NULL, APR_HOOK_MIDDLE); */
   /* [1] post read_request handling */
   ap_hook_post_read_request(weblink_post_read_request, NULL, NULL,
                              APR_HOOK_MIDDLE);
   ap_hook_log_transaction(weblink_logger, NULL, NULL, APR_HOOK_MIDDLE);
#if 0
   ap_hook_http_method(weblink_http_method, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_default_port(weblink_default_port, NULL, NULL, APR_HOOK_MIDDLE);
#endif
   ap_hook_translate_name(weblink_translate_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_check_user_id(weblink_check_user_id, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_fixups(weblink_fixer, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_type_checker(weblink_type_checker, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_access_checker(weblink_access_checker, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_auth_checker(weblink_auth_checker, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_insert_filter(weblink_insert_filter, NULL, NULL, APR_HOOK_MIDDLE);

#else

   ap_hook_handler(weblink_handler, NULL, NULL, APR_HOOK_MIDDLE);

#endif

   return;
}

#endif


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* All of the routines have been declared now.  Here's the list of          */
/* directives specific to our module, and information about where they      */
/* may appear and how the command parser should pass them to us for         */
/* processing.  Note that care must be taken to ensure that there are NO    */
/* collisions of directive names between modules.                           */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * List of directives specific to our module.
 */

#if MGW_APACHE_VERSION == 20

static const command_rec weblink_commands[] = {

   AP_INIT_NO_ARGS(
      "WebLink",                       /* directive name */
      cmd_weblink,                     /* action routine for directive */
      NULL,                            /* argument to include in call */
      OR_OPTIONS,                      /* where available */
      "WebLink directive - no arguments"   /* directive description */
   ),
   { NULL }
};

#else

command_rec weblink_commands[] = {
   {
      "WebLink",                          /* directive name */
      cmd_weblink,                        /* action routine for directive */
      NULL,                               /* argument to include in call */
      OR_OPTIONS,                         /* where available */
      NO_ARGS,                            /* arguments */
      "WebLink directive - no arguments"  /* directive description */
   },
   { NULL }
};

#endif

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now the list of content handlers available from this module.             */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * List of content handlers our module supplies.  Each handler is defined by
 * two parts: a name by which it can be referenced (such as by
 * {Add,Set}Handler), and the actual routine name.  The list is terminated by
 * a NULL block, since it can be of variable length.
 *
 * Note that content-handlers are invoked on a most-specific to least-specific
 * basis; that is, a handler that is declared for "text/plain" will be
 * invoked before one that was declared for "text / *".  Note also that
 * if a content-handler returns anything except DECLINED, no other
 * content-handlers will be called.
 */

#if MGW_APACHE_VERSION != 20

static handler_rec weblink_handlers[] = {
   { "weblink-handler", weblink_handler },
   { NULL }
};

#endif

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Finally, the list of callback routines and data structures that          */
/* provide the hooks into our module from the other parts of the server.    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * Module definition for configuration.  If a particular callback is not
 * needed, replace its routine name below with the word NULL.
 *
 * The number in brackets indicates the order in which the routine is called
 * during request processing.  Note that not all routines are necessarily
 * called (such as if a resource doesn't have access restrictions).
 */


#if MGW_APACHE_VERSION == 20

module AP_MODULE_DECLARE_DATA weblink_module = {

   STANDARD20_MODULE_STUFF,
#if 0
   weblink_create_dir_config,    /* per-directory config creater */
   weblink_merge_dir_config,     /* dir config merger - default is to override */
   weblink_create_server_config, /* server config creator */
   weblink_merge_server_config,  /* server config merger */
   weblink_commands,             /* command table */
#else
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
#endif
   weblink_register_hooks,       /* set up other request processing hooks */
};

#else

module weblink_module = {
   STANDARD_MODULE_STUFF,
   weblink_init,                 /* module initializer */
   weblink_create_dir_config,    /* per-directory config creater */
   weblink_merge_dir_config,     /* dir config merger - default is to override */
   weblink_create_server_config, /* server config creator */
   weblink_merge_server_config,  /* server config merger */
   weblink_commands,             /* command table */
   weblink_handlers,             /* [6] list of handlers */
   weblink_translate_handler,    /* [1] filename-to-URI translation */
   weblink_check_user_id,        /* [4] check/validate HTTP user_id */
   weblink_auth_checker,         /* [5] check HTTP user_id is valid *here* */
   weblink_access_checker,       /* [3] check access by host address, etc. */
   weblink_type_checker,         /* [6] MIME type checker/setter */
   weblink_fixer,                /* [7] fixups */
   weblink_logger,               /* [9] logger */
   weblink_header_parser,        /* [2] header parser */

#if MGW_APACHE_VERSION == 13
   weblink_child_init,           /* process initializer */
   weblink_child_exit,           /* process exit/cleanup */
   weblink_post_read_request     /* [1] post read_request handling */
#endif
};

#endif


int MgwGetConfiguration(LPCONX lp_connection)
{
   int len;
   char buffer[128], tcp_port[128], wlserver_ini[128];
   FILE *fp;

   strcpy(lp_connection->ip_address, SA_IP_ADDRESS);
   lp_connection->port = (int) strtol(SA_TCP_PORT, NULL, 10);

   len = 0;
   *tcp_port = '\0';
   *buffer = '\0';

#if _WIN32

   strcpy(wlserver_ini, SA_INI_FILE);
   fp = fopen(wlserver_ini, "r");

   len = GetPrivateProfileString("SYSTEM", "Ip_Address", "", buffer, 31, wlserver_ini);
   if (len)
      strcpy(lp_connection->ip_address, buffer);
   GetPrivateProfileString("SYSTEM", "TCP_Port", "", tcp_port, 31, wlserver_ini);

   if (strlen(tcp_port))
      lp_connection->port = (int) strtol(tcp_port, NULL, 10);

#else

   sprintf(wlserver_ini, "%s/%s", SA_PATH_1, SA_INI_FILE);
   fp = fopen(wlserver_ini, "r");
   if (!fp) {
      sprintf(wlserver_ini, "%s/%s", SA_PATH_2, SA_INI_FILE);
      fp = fopen(wlserver_ini, "r");
      if (!fp) {
         sprintf(wlserver_ini, "%s/%s", SA_PATH_3, SA_INI_FILE);
         fp = fopen(wlserver_ini, "r");
      }
   }

   if (fp) {
      while (fgets(buffer, 64, fp)) {
         len = (int) strlen(buffer) - 1;
         buffer[len] = '\0';
         if (len > 11 && !strncmp(buffer, "Ip_Address=", 11))
            strcpy(lp_connection->ip_address, buffer + 11);
         else if (len > 11 && strncmp(buffer, "TCP_Port=", 9))
            strcpy(tcp_port, buffer + 9);
      }

      fclose(fp);

      if (strlen(tcp_port))
         lp_connection->port = (int) strtol(tcp_port, NULL, 10);
   }

#endif

   return 1;
}


int MgwOpenConnection(LPCONX lp_connection)
{
   int n;
   static struct sockaddr_in cli_addr, srv_addr;

   strcpy(lp_connection->error, "");

#ifdef _WIN32
   VersionRequested = 0x101;

   n = WSAStartup(VersionRequested, &(lp_connection->wsadata));
   if (n != 0) {
      strcpy(lp_connection->error, "Microsoft WSAStartup Failed");
      return 0;
   }
#endif

#ifndef _WIN32
   bzero((char *) &srv_addr, sizeof(srv_addr));
#endif

   srv_addr.sin_port = htons((unsigned short) lp_connection->port);
   srv_addr.sin_family = AF_INET;
   srv_addr.sin_addr.s_addr = inet_addr(lp_connection->ip_address);

   lp_connection->sockfd = socket(AF_INET, SOCK_STREAM, 0);
   if (lp_connection->sockfd < 0) {
      strcpy(lp_connection->error, "Can't open a stream-socket to WebLink");

#ifdef _WIN32
      WSACleanup();
#endif

      return 0;
   }

#ifndef _WIN32
   bzero((char *) &cli_addr, sizeof(cli_addr));
#endif

   cli_addr.sin_family = AF_INET;
   cli_addr.sin_addr.s_addr = htonl(INADDR_ANY);
   cli_addr.sin_port = htons(0);

   n = bind(lp_connection->sockfd, (struct sockaddr *) &cli_addr, sizeof(cli_addr));
   if (n < 0) {
      strcpy(lp_connection->error, "Can't bind to local address for WebLink access");
      MgwCloseConnection(lp_connection);
      return 0;
   }

   n = connect(lp_connection->sockfd, (struct sockaddr *) &srv_addr, sizeof(srv_addr));

   if (n < 0) {
      strcpy(lp_connection->error, "Can't connect to the WebLink NSD");
      MgwCloseConnection(lp_connection);
      return 0;
   }

   return 1;
}


int MgwCloseConnection(LPCONX lp_connection)
{
#ifdef _WIN32
   closesocket(lp_connection->sockfd);
   WSACleanup();
#else
{
int n;
struct linger ling;
ling.l_onoff = 1;
ling.l_linger = 0;

   n = setsockopt(lp_connection->sockfd, SOL_SOCKET, SO_LINGER, (void *) &ling, sizeof(ling));
}

   close(lp_connection->sockfd);
#endif

   return 1;
}


int MgwSendDataToNSD(LPCONX lp_connection, char *data, int data_size)
{
   int n, len;
   char header[16];

   sprintf(header, "%d", data_size);
   len = (int) strlen(header);
   while (len < 9) {
      strcat(header, "#");
      len ++;
   }
   strcat(header, "\r");

   n = send(lp_connection->sockfd, header, 10, 0);
   n = send(lp_connection->sockfd, data, data_size, 0);

   return n;
}


int MgwReceiveDataFromNSD(LPCONX lp_connection, char *data, int data_size)
{
   return recv(lp_connection->sockfd, data, data_size, 0);
}


int MgwReturnError(EXTENSION_CONTROL_BLOCK *pECB, char *error)
{
   MgwWriteString(pECB, "HTTP/1.0 200 OK\r\n");
   MgwWriteString(pECB, "Content-type: text/html\r\n\r\n");
   MgwWriteString(pECB, "<HTML>\r\n");
   MgwWriteString(pECB, "<HEAD><TITLE>Cach&eacute; WebLink Connection Error</TITLE></HEAD>\r\n");
   MgwWriteString(pECB, "<BODY>\r\n");
   MgwWriteString(pECB, "<H2>An error occurred whilst attempting to communicate with the Cach&eacute; Weblink Network Service Daemon</H2>\r\n");
   MgwWriteString(pECB, "<P>\r\n");
   MgwWriteString(pECB, error);
   MgwWriteString(pECB, "\r\n</BODY>\r\n");
   MgwWriteString(pECB, "</HTML>\r\n");

   return 1;
}

int MgwReturnRequestBlock(EXTENSION_CONTROL_BLOCK *pECB, LPMEMOBJ lp_trans_buffer)
{
   int n;
   char cbuffer[4];

   cbuffer[1] = '\0';

   MgwWriteString(pECB, "HTTP/1.0 200 OK\r\n");
   MgwWriteString(pECB, "Content-type: text/html\r\n\r\n");
   MgwWriteString(pECB, "<HTML>\r\n");
   MgwWriteString(pECB, "<HEAD><TITLE>Cach&eacute; WebLink Request Block</TITLE></HEAD>\r\n");
   MgwWriteString(pECB, "<BODY>\r\n");
   MgwWriteString(pECB, "<H2>Cach&eacute; WebLink Request Block ...</H2>\r\n");
   MgwWriteString(pECB, "<P>\r\n");

   for (n = 0; n < lp_trans_buffer->curr_size; n ++) {
      if (lp_trans_buffer->lp_buffer[n] == '\r')
         MgwWriteString(pECB, "\r\n<BR>\r\n");
      else {
         cbuffer[0] = lp_trans_buffer->lp_buffer[n];
         MgwWriteBuffer(pECB, cbuffer, 1);
      }
   }
   MgwWriteString(pECB, "\r\n</BODY>\r\n");
   MgwWriteString(pECB, "</HTML>\r\n");

   return 1;
}


/* Return data to client */

int MgwWriteString(EXTENSION_CONTROL_BLOCK *pECB, char *buffer)
{
   int size, written;

   size = (int) strlen(buffer) * sizeof(char);

   if ((written = MGW_RWRITE(buffer, size, pECB->lp_request_rec)) == EOF)
	   return 0;

   return 1;
}


int MgwWriteBuffer(EXTENSION_CONTROL_BLOCK *pECB, void *buffer, int size)
{
   int written;

   if ((written = MGW_RWRITE(buffer, size, pECB->lp_request_rec)) == EOF)
	   return 0;

   return 1;
}


int MgwServerVariable(void *rec, const char *key, const char *value)
{
   LPMEMOBJ lp_cgievar;

   if (!rec || !key || !value)
      return 0;

   lp_cgievar = (LPMEMOBJ) rec;

   MgwMemCat(lp_cgievar, (char *) key);
   MgwMemCat(lp_cgievar, "=");
   MgwMemCat(lp_cgievar, (char *) value);
   MgwMemCat(lp_cgievar, "<BR>");

   return 1;
}


int MgwGetServerVariable(EXTENSION_CONTROL_BLOCK *pECB, char *VariableName, LPMEMOBJ lp_cgievar)
{
   table *e;
   char *result;

   e = pECB->lp_request_rec->subprocess_env;
   result = NULL;

   MgwMemCpy(lp_cgievar, "");

   /* Mostly, we just grab it from the environment, but there are
    * a couple of special cases
    */

   if (!strcmp(VariableName, "ALL_ENV")) {
      MGW_TABLE_DO(MgwServerVariable, (void *) lp_cgievar, e, NULL);
   }

   if (!strcasecmp(VariableName, "UNMAPPED_REMOTE_USER")) {

	   /* We don't support NT users, so this is always the same as
	    * REMOTE_USER
	    */

      result = (char *) MGW_TABLE_GET(e, "REMOTE_USER");
   }
   else if (!strcasecmp(VariableName, "SERVER_PORT_SECURE")) {

	   /* Apache doesn't support secure requests inherently, so
	    * we have no way of knowing. We'll be conservative, and say
	    * all requests are insecure.
	    */

      result = "0";
   }
   else if (!strcasecmp(VariableName, "URL") || !strcasecmp(VariableName, "SCRIPT_NAME")) {
	   result = pECB->lp_request_rec->uri;
   }
   else if (!strcasecmp(VariableName, "PATH_TRANSLATED")) {

	   result = (char *) MGW_TABLE_GET(e, "DOCUMENT_ROOT");
      if (result) {
         MgwMemCpy(lp_cgievar, result);
         if (pECB->lp_request_rec->uri)
            MgwMemCat(lp_cgievar, pECB->lp_request_rec->uri);

         return lp_cgievar->curr_size;

      }
      else
   	   result = (char *) MGW_TABLE_GET(e, VariableName);

   }
   else {
	   result = (char *) MGW_TABLE_GET(e, VariableName);
   }

   if (result)
      MgwMemCpy(lp_cgievar, result);
   else if (!strcasecmp(VariableName, "CONTENT_LENGTH"))
      MgwMemCpy(lp_cgievar, "0");

   return lp_cgievar->curr_size;

}


int MgwMemInit(LPMEMOBJ lp_mem_obj, int size, int incr_size)
{
   int result;

   lp_mem_obj->lp_buffer = (char *) malloc(sizeof(char) * (size + 1));
   if (lp_mem_obj->lp_buffer) {
      *(lp_mem_obj->lp_buffer) = '\0';
      result = 1;
   }
   else {
      result = 0;
      lp_mem_obj->lp_buffer = (char *) malloc(sizeof(char));
      if (lp_mem_obj->lp_buffer) {
         *(lp_mem_obj->lp_buffer) = '\0';
         size = 1;
      }
      else
         size = 0;
   }

   lp_mem_obj->size = size;
   lp_mem_obj->incr_size = incr_size;
   lp_mem_obj->curr_size = 0;

   return result;
}


int MgwMemFree(LPMEMOBJ lp_mem_obj)
{
   if (lp_mem_obj->lp_buffer)
      free((void *) lp_mem_obj->lp_buffer);

   lp_mem_obj->lp_buffer = NULL;
   lp_mem_obj->size = 0;
   lp_mem_obj->incr_size = 0;
   lp_mem_obj->curr_size = 0;

   return 1;
}


int MgwMemCpy(LPMEMOBJ lp_mem_obj, char *string)
{
   int result, req_size, size, incr_size;

   result = 1;
   req_size = (int) strlen(string);
   if (req_size > lp_mem_obj->size) {
      size = lp_mem_obj->size;
      incr_size = lp_mem_obj->incr_size;
      while (req_size > size)
         size = size + lp_mem_obj->incr_size;
      MgwMemFree(lp_mem_obj);
      result = MgwMemInit(lp_mem_obj, size, incr_size);
   }
   if (result) {
      strcpy(lp_mem_obj->lp_buffer, string);
      lp_mem_obj->curr_size = req_size;
   }

   return result;
}


int MgwMemCat(LPMEMOBJ lp_mem_obj, char *string)
{
   int result, req_size, size, incr_size;
   char *lp_temp;

   result = 1;
   req_size = (int) strlen(string) + lp_mem_obj->curr_size;
   if (req_size > lp_mem_obj->size) {
      size = lp_mem_obj->size;
      incr_size = lp_mem_obj->incr_size;
      while (req_size > size)
         size = size + lp_mem_obj->incr_size;
      lp_temp = lp_mem_obj->lp_buffer;
      result = MgwMemInit(lp_mem_obj, size, incr_size);
      if (result) {
         if (lp_temp) {
            strcpy(lp_mem_obj->lp_buffer, lp_temp);
            free((void *) lp_temp);
         }
      }
      else
         lp_mem_obj->lp_buffer = lp_temp;
   }
   if (result) {
      strcat(lp_mem_obj->lp_buffer, string);
      lp_mem_obj->curr_size = req_size;
   }

   return result;
}


int MgwLCase(char *string)
{

#if defined(_WIN32) && defined(_UNICODE)

   CharLower(string);
   return 1;

#else

   int n, chr;

   n = 0;
   while (string[n] != '\0') {
      chr = (int) string[n];
      if (chr >= 65 && chr <= 90)
         string[n] = (char) (chr + 32);
      n ++;
   }
   return 1;

#endif

}
